/**@name NEURObjects Documentation
   @memo C++ library classes for neural networks development.
   @version	2.0
   @author	Giorgio Valentini - DISI, Universita' di Genova, Italia.
 */

//@{  

  		 /** @name NEURObjects groups of classes
      		 
  		 */
  		 //@{ 		   
         /** @name Data sets
				     Classes implementing data sets for training and testing NEURObjects learning machines.
        	   @memo I/O and data sets pre-processing classes.
         */
           //@{
  			     //@Include: tset.dxx
  			   //@}
         /** @name Automatic data generation
				     Classes for generating synthetic data sets by means of predefined probability density
						 distributions.  
        	   @memo Classes for automatic data set generation.
         */
           //@{
  			     //@Include: patgen.dxx
  			   //@}
  		   /** @name Neural networks
        	   @memo Classes for neural networks training and testing
         */
  		   //@{
           /** @name MLP
        	     @memo Classes implementing Multi Layer Perceptrons 
           */
					 		//@{
  			         //@Include: net.dxx
								 //@Include: build_net.dxx
  			      //@} 
					 /** @name ECOC MLP.
        	     @memo Classes implementing Error Correcting Output Coding Multi Layer Perceptrons
           */
					   //@{
  			         //@Include: ecocnet.dxx
  			     //@}
					 /** @name Dichotomizers
        	     @memo Classes implementing dichotomic MLP
           */ 
             //@{
  			      //@Include: dico.dxx
    		     //@}  
					 /** @name Neural net layers
        	   @memo Classes implementing neural network layers
           */
           //@{
  			     //@Include: layer.dxx
  			   //@}  
    		 //@}
    		 
    		 /** @name Learning algorithms
        		 @memo Classes implementing learning algorithms
    		 */
    		 //@{
  			 //@Include: learn.dxx
    		 //@}
    		 /** @name Ensembles of learning Machines
        		 @memo Classes implementing ensembles of learning machines
    		 */
				    //@{
				         /** @name Output Coding decomposition Ensembles
        		         @memo Classifiers classes based on decomposition Output Coding decopmosition methods
    		         */
								 //@{
  			            /** @name Decomposition schemes
        	              @memo Classes implementing Output Coding decomposition schemes
                    */
                       //@{
  			                 //@Include: decomp.dxx
    		               //@}
                    /** @name PND
        	              @memo Classes implemeting Parallel Non linear Dichotomizers (PND) ensembles
                    */
                       //@{
  			                 //@Include: polidico.dxx
    		               //@}
    		        //@}
				   //@}
  			    		 
    		 /** @name Statistics
    		     Classes for statistical evaluation of neural networks performances. They include classes for
						 evaluating the dependence of output and output errors of multi input multi ouptut (MIMO)
						 learning machines and Receiver Operating Characteristics (ROC) of learning machines 
        		 @memo Classes for statistical evaluation of neural networks performances
    		 */
				 //@{
  			    /** @name Dependence among outputs and output errors
        	     @memo Classes for evaluating the dependence of output and output errors of multi input 
							 multi ouptut (MIMO) learning machines
               */
               //@{
  			       //@Include: mutualinfo.dxx
							 //@Include: outerr.dxx
							 // mutualinfo7.dxx
    		       //@}
						/** @name Statistical tests
							 HTML documentation under construction
        	     @memo Functions implementing statistical tests
               */
							 //@{
  			       //@Include: chi2.dxx
    		       //@} 
				    
  			    /** @name ROC analysis
        	     @memo Classes for evaluating the Receiver Operating Characterisitics of learning
							 machines
               */
               //@{
  			       //@Include: roc.dxx
    		       //@}					    		   
				 //@}   		    		 
    		 /** @name General support classes and utilities
				     Basic general purpose classes implementing lists, queues, vectors, matrices, trees and
						 other basic data types. The  core of these basic no neural specific classes are based on the 
						 C++ Standard Template Library (STL). 
						 Here are collected also utility functions and classes for real vector and matrix manipulation.		 
        	   @memo Basic general purpose classes no  neural computation specific.
         */
				    //@{
						/**
              	@name Look-up tables
	            	Activation function are stored as look-up table to speed-up computation. Sigmoid, hyperbolic
	            	tangent and linear activation functions a re available.
	            	@memo Look-up tables for MLP activation functions 
            	 */						
						  	 //@{
  			        		//@Include: act_fun.dxx
    		      	 //@}
						/**
              	@name Utility functions
	            	Utility functions used by classes and applications of NEURObjects.\\								
	            	@memo General utility functions - HTML documentation under construction.
            	 */						
						  	 //@{
  			        		// utils.dxx
    		      	 //@}	
						/**
              	@name Vectors and matrices
	            	Classes implementing vector and matrices of real numbers.\\								
	            	@memo Classes implementing vector and matrices of real numbers.
            	 */						
						  	 //@{
  			        		//@Include: vect.dxx
										//@Include: matrix.dxx
    		      	 //@}	
						//@}
  		 //@}
  		 /** @name Applications for classification problems.
        	 A set of command-line applications with parameters for traininig and testing
					 neural networks and learning machines for classification problems.
       */
       //@{
  			   //@Include: nn.dxx
  			   //@Include: nn_cv.dxx
  			   //@Include: pnd.dxx  			   
  			   //@Include: pnd_cv.dxx
  			   //@Include: dodata.dxx
					 //@Include: dofold.dxx
					 //@Include: convert_data_format.dxx
					 //@Include: subsample.dxx
           //@Include: subsampleLightData.dxx
           //@Include: bagsvm.dxx
    	 //@}
       //@Include: format.dxx
			 //@Include: excode.dxx		
//@}

