/**@name NEURObjects User Manual
   @memo C++ library classes for neural networks development.
   @version	1.0
   @author	Giorgio Valentini
 */

//@{  
  //@Include: intro.dxx
  /**@name User Manual
    	 	
  */
		 //@{
  		 /** @name Quick start.
      		 To be done ... \\      		 
  		 */

  		 /** @name NEURObjects groups of classes
      		 
  		 */
  		 //@{
  		   /** @name General support classes
  		       Classi di base usate dalle classi proprie di NEURObjects. Sono librerie di dominio 
  		       pubblico le cui classi sono usate da NEURObjects (liste, code, vettori, matrici, alberi,etc).
  		       La principale libreria utilizzata e' LEDA : http://www.mpi-sb.mpg.de/LEDA.
        	   @memo Basic general purpose classes no  neural computation specific.
         */
         /** @name Training set
				     Tali classi generano dati sintetici per il training ed il testing delle reti neurali tramite
             tecniche deterministiche o stocastiche.
        	   @memo I/O and data sets pre-processing classes.
         */
           //@{
  			     //@Include: tset.dxx
  			   //@}
         /** @name Automatic data generation
             Tali classi generano dati sintetici per il training ed il testing delle reti neurali tramite
             tecniche deterministiche o stocastiche.
        	   @memo Classes for automatic data set generation.
         */
           //@{
  			     
  			   //@}
  		   /** @name Neural nets
  		       Le classi descritte in questa sezione implementano i processi di calcolo e di apprendimento
  		       delle reti neurali. Le reti sono strutturate a strati.
        	   @memo Classes for neural networks training and testing
         */
  		   //@{
  		     /** @name Neural net layers
        	   @memo Classi per l' implementazione degli strati delle reti neurali.
           */
           //@{
  			     //@Include: layer.dxx
  			   //@} 
  			   /** @name Reti neurali (uso e testing)
        	   @memo Classi per l' utilizzo ed il testing delle reti neurali.
           */
             //@{
  			       
  			     //@} 
           /** @name Reti neurali (training)
        	     @memo Classi per il training delle reti neurali.
           */
					 		//@{
  			         //@Include: net.dxx
  			      //@} 
					 /** @name Direct ECOC neural nets.
					     Implementano reti neurali per la classificazione tramite codici ECOC. 
							 Ad ogni classe e' associato un codeword calcolato tramite un algoritmo ECOC-BCH.
							 La classe predetta dalla rette e' quella il cui codeword dista meno in norma L1 dal codeword
							 della classe.
        	     @memo Classificatori basati su codifica ECOC delle classi
           */
					   //@{
  			         //@Include: ecocnet.dxx
  			     //@} 
    		 //@}
    		 /** @name Dichotomizers
  		       Le classi descritte in questa sezione implementano dicotomizzatori tramite percettroni a
  		       strati. I dicotomizzatori possono essere utilizzati direttamente per classificazioni binarie.\\
  		       Sono inoltre i componeti di base di classificatori basati sula scomposizione di policotomie
  		       tramite dicotomie.
        	   @memo Dichotomizers training and testing classes
         */ 
         //@{
  			 //@Include: dico.dxx
    		 //@}  
    		 /** @name Learning
        		 Le classi descritte in questa sezione implementano diverse tipologie di algoritmi di
        		 apprendimento utilizzabili per le reti neurali.
        		 @memo Classes for inductive learning
    		 */
    		 //@{
  			 //@Include: learn.dxx
    		 //@}
    		 /** @name Networks ensemble \\
        		 Tali classi generano dati sintetici per il training ed il testing delle reti neurali tramite
             tecniche deterministiche o stocastiche.
        		 @memo Neural networks ensembles classes
    		 */
				 //@{
				 
				 //@}
  			 
				 /** @name Decomposition of polychotomies into dichotomies
        		 @memo Classifiers classes based upon decomposition of polychotomies into dichotomies
    		 */
    		 //@{
  			      /** @name Scompositori
        	     @memo Classi per la scomposizione dicotomica di policotomie
              */
              //@{
  			      //@Include: decomp.dxx
    		      //@}
              /** @name Policotomizzatori dicotomici.\\
               Tali classi costruiscono, addestrano ed effettuano il testing di policotomizzatori
               dicotomici.
        	     @memo Classi che implementano policotomizzatori costruiti tramite  scomposizioni di
        	     policotomie in dicotomie.
              */
              //@{
  			      //@Include: polidico.dxx
    		      //@}
    		 //@}
				 
    		 
    		 /** @name Statistics
    		     Classes for statistical evaluation of neural networks performances. They include classes for
						 evaluating the dependence of output and output errors of multi input multi ouptut (MIMO)
						 learning machines
        		 @memo Classes for statistical evaluation of neural networks performances
    		 */
				 //@{
  			       /** @name Dependence among outputs and output errors
        	     @memo Classes for evaluating the dependence of output and output errors of multi input 
							 multi ouptut (MIMO) learning machines
               */
               //@{
  			       //@Include: mutualinfo.dxx
							 //@Include: mutualinfo7.dxx
    		       //@}
    		 //@}
    		 /** @name Algorithms comparison
    		     Tali classi generano dati sintetici per il training ed il testing delle reti neurali tramite
						              tecniche deterministiche o stocastiche.
        		 @memo Learning algorithms statistical comparison classes.
    		 */
				 //@{
  			     
    		 //@}
    		 
    		 /** @name Utility classes.\\
    		     Tali classi generano dati sintetici per il training ed il testing delle reti neurali tramite
             tecniche deterministiche o stocastiche.
						 @memo Functional general utility classes 
    		 */
    		    //@{
  			      
  			      
    		    //@}
  		 //@}
  		 /** @name Applications for classification problems.
        	 A set of command-line application with parameters for traininig and testing single layer
					 perceptrons,  multi layer perceptrons (MLP) and Parallel Non linear Dichotomizers ensemble
					 for classification problems.
       */
       //@{
  			   //@Include: nn.dxx
  			   //@Include: nn_cv.dxx
  			   //@Include: pd.dxx  			   
  			   //@Include: pd_cv.dxx
  			   //@Include: dodata.dxx
    	 //@}
       //@Include: format.dxx	
		 //@}
//@}

