// analyze_BV_bagsvm_step.cc
// Application for analyzing bias and variance error in LM
#include<iostream.h>
#include<fstream.h>
#include<stdio.h>
#include<time.h>
#include<string.h>
#include<stdlib.h>
#include<math.h>

const char SEP = '\t';
const int DATA_SIZE = 10;
const int DIMBUF = 256;
const int TESTSIZE = 11000;
const int MAXNUMDATASETS = 400;

// global variables
char model[DIMBUF];
int numdatasets;
char dirresults[DIMBUF];
char datatest[DIMBUF];
int dimtestset = TESTSIZE;
int n_base_learn = 60;
  
// vector of the prob. for class 1
float p1[TESTSIZE];
// vector of the Main predictions
int M[TESTSIZE];
// vector of the right labels (targets)
int T[TESTSIZE];
// Bias vector
int B[TESTSIZE];
// Unbiased variance vector
float Vu[TESTSIZE];
// Biased variance vector
float Vb[TESTSIZE];
// Net variance vector
float NV[TESTSIZE];
// Positive Net variance vector
float V2[TESTSIZE];
// Expected error vector
float E[TESTSIZE];
// storage of the output of the numdatasets SVM on the test set
float f[400][TESTSIZE];
// Average bias
float AvB;
// Average variance unbiased
float AvVu;
// Average variance biased
float AvVb;
// Average net-variance
float AvNV;
// Average Error
float AvE;
//
bool b_test_on_train = false;


void print_info_use(void)
{
  cerr << "
  analyze_BV_bagsvm_step. Application for analyzing 
  bagged SVM bias and variance error .
  Usage: analyze_BV_step modelname numdatasets dirresults testdata dimtestset n_base_learn
         modelname     : base name of the predicted classes
         numdatasets   : number of data sets
         dirresults    : directory of the file containing the predicted classes
         testdata      : name of the test set 
         dimtestset    : dimension of the test set
         n_base_learn  : number of the base learners." << endl;
  exit(1);
}
  


// It computes p1 and M
void calc_p1_M(void)
{
  int i,j;
  char filename[DIMBUF];
  ifstream fname;
  
  // open files for reading data and loading them in f
  for (i=1; i<=numdatasets; i++) {   
		sprintf(filename,"%s/%st%d-%d", dirresults, model, i, n_base_learn);	  
    fname.open(filename);
    if (!fname) {
      cerr << "Error opening " << filename << endl;
      exit(1);
    }
    for(j=0; j<dimtestset; j++)
      fname >> f[i-1][j];   
    if (fname.bad())  {
       cerr << "Error reading data file " << filename << endl;
       exit(1);
    }
    fname.close();
  }
   
  // p1 computing
  for(j=0; j<dimtestset; j++) {
    p1[j]=0;
    for (i=0; i<numdatasets; i++) 
      if (f[i][j] == 1) ++p1[j];
    p1[j] /= numdatasets;
  }
  
  // Main decision M computation
  for(j=0; j<dimtestset; j++)
    if (p1[j]> 0.5) M[j] = 1;
    else M[j] = 2;
}



// It computes T reading it from the file datatest
void calc_T(char* datatest) 
{
  const int DIM = 256000;
	int j;
  char buf[DIM];
  
  ifstream data(datatest);
  // discards the first two lines
  data.getline(buf, DIM);
  data.getline(buf, DIM);
  for(j=0; j<dimtestset; j++) {
    data >> T[j]; 
		if ((T[j] != +1) && (T[j] != -1)) {
		  cerr << "Error reading T" << endl;
			exit (1);
		}
		if (T[j] == -1) T[j] = 2;
		data.getline(buf, DIM);
	}
}


// Compute Bias
void calc_Bias(void)
{
  int j;
  
  for(j=0; j<dimtestset; j++)
	  if (M[j] != T[j])
		  B[j] = 1;
		else
		  B[j] = 0;
}


// Compute Variance
void calc_Variance(void) 
{
  int j;

  for(j=0; j<dimtestset; j++) {
    if (B[j] == 0) {
      Vb[j] = 0;
      if (p1[j] > 0.5)
        Vu[j] = 1 - p1[j];       
      else        
	Vu[j] = p1[j];
    }
    else {
      Vu[j] = 0;
      if (p1[j] <= 0.5) 
        Vb[j] = p1[j];       
      else        
	Vb[j] = 1 - p1[j];
    }
  NV[j] = Vu[j]-Vb[j];
  }
}


// Compute Variance. It is an equivalent computation of the net-variance.
void calc_Variance2(void) 
{
  int i,j, cl;

  for(j=0; j<dimtestset; j++) {
    V2[j] = 0;
    for (i=0; i<numdatasets; i++) {
      if (f[i][j] == 1) cl = 1;
      else cl = 2;
      if (cl != M[j]) ++V2[j];
    } 
    V2[j] /= numdatasets;
  }
}


// Compute Expected error
void calc_Error(void) 
{
  int j;
  for(j=0; j<dimtestset; j++)
    E[j] = B[j] + Vu[j] - Vb[j];
}

// Compute average error, bias, biased variance, unbiased variance and net-variance
// Compute Expected error
void calc_Averages(void) 
{
  int j;
  AvB=0;
  AvVu=0;
  AvVb=0;
  AvNV=0;
  AvE=0;
  for(j=0; j<dimtestset; j++) {
    AvB+=B[j];
    AvVu+=Vu[j];
    AvVb+=Vb[j];
    AvNV+=NV[j];
    AvE+=E[j];
  }
  AvB/=dimtestset;
  AvVu/=dimtestset;
  AvVb/=dimtestset;
  AvNV/=dimtestset;
  AvE/=dimtestset;
}
  
void print_Averages(void)  
{
  cout.setf(ios::left, ios::adjustfield);
  cout.precision(4);
  cout.width(12); 
  cout << n_base_learn;
  cout.width(12);
  cout << AvE;
  cout.width(12);
  cout << AvB;
  cout.width(12);
  cout << AvVu;
  cout.width(12);
  cout << AvVb;
  cout.width(12);
  cout << AvNV;
  cout << endl;
}
     

// **************************************
int main(int argc, char* argv[])
{
    
  if (argc !=7) 
     print_info_use();
     
  // reading arguments
  strcpy(model, argv[1]);
  numdatasets = atoi(argv[2]);
	if (numdatasets > MAXNUMDATASETS) {
	  cerr << "Too many data sets" << endl;
		exit(1);
	}
  strcpy(dirresults, argv[3]);
  strcpy(datatest, argv[4]);
	dimtestset = atoi(argv[5]);
	n_base_learn = atoi(argv[6]);
  calc_p1_M();
  calc_T(datatest);
  calc_Bias();
  calc_Variance();
  //calc_Variance2();
  calc_Error();
  calc_Averages();
  print_Averages();  
  return 0;
}
  
