// binomial.h


#ifndef BINOMIAL_H
#define BINOMIAL_H

#include<iostream>
#include<fstream>
#include<vector>
#include<string>

using namespace std;

/**
   Class implemeting the binomial distribution.\\
   It computes the binomial probability distribution, computing both the density and the cumulative probability distributions.
	 This class can be used without create any instance, using only the methods provided by the class itself.
*/ 
class Binomial {
  
	public:
	/**   
	 Computes the factorial function. \\
	 @param n input value
	 @return the factorial of n 
	*/ 
	static double fact (double n);
	
	/**   
	 Computes the factorial function. \\
	 Computes n!/(k!(n-k)!).
	 @param n "numerator" of the binomial coefficient
	 @param k "denominator" of the binomial coefficient
	 @return the binomial coefficient of (n,k)
	*/ 
	static inline double bincoeff (double n, double k);
	
	/**   
	 Computes binomial probability density function \\
	 @param n total number of experiments
	 @param k number of successes
	 @param p probability
	 @return value of the binomial probability density function
	*/ 
	static inline double probability (double n, double k, double p);
	
	/**   
	 Computes binomual cumulative distribution \\
	 It computes the difference between two given values of the cumulative binomial distribution, that is it computes P(kend)-P(kstart),
	 where P is the cumulative binomial distribution 
	 @param n total number of experiments
	 @param kstart 
	 @param kend 
	 @param p probability
	 @return value of the binomial cumulative distribution between kstart and kend
	*/ 
	static double cumulative (double n, double kstart, double kend, double p);
	
};

#endif
