// "bv2decomp.cc"
// November 2002


#include<ctype.h>
#include<stdlib.h>
#include<math.h>
#include<time.h>
#include<iostream.h>
#include <fstream.h>
#include <strstream.h>
#include <string.h>
#include"bootstrapLight.h"
#include"bv2decomp.h"
#include"utils.h"

AverageBV::AverageBV(float x) {
  init(x);	
}

void AverageBV::init (float x) {
  loss = x;
	bias = x;
	net_variance = x;
	variance = x;
	unbias_var = x;
	bias_var = x;
}

void AverageBV::print (ostream& out) {
  
	const int WIDTH = 16;
	out.setf(ios::adjustfield, ios::left);
  out.precision(8);
  
  out.width(WIDTH); out << loss;
  out.width(WIDTH); out << bias;
  out.width(WIDTH); out << net_variance;
  out.width(WIDTH); out << unbias_var;
  out.width(WIDTH); out << bias_var;
	out.width(WIDTH); out << variance;
  out << endl;
}
	


///////////////////////// class BV2decomp

/// Dummy constructor
BV2decomp::BV2decomp() :
numdatasets(0), dimtestset(0), AvBV(0), bsm(NULL)
{}


/**
	    Constructor.
			It loads the predictions mase by th base learner in the matrix P.			
			@param testset test set file name
			@param predictedbasename basename of the files containing the predicted outputs.
			The files are predictedbasename.out, predictedbasename.out, ... predictedbasenamendatasets.out.
			@param ndatasets number of the prediction files
*/
BV2decomp::BV2decomp(char* testset, char* predictedbasename, int ndatasets) : AvBV(0), bsm(NULL)
{
  strcpy(datatest, testset);
	strcpy(predbase, predictedbasename);
	numdatasets = ndatasets;	
	dimtestset = count_data_lines(testset);
	p1 = new float[dimtestset];
	/// vector of the Main predictions
	Main = new int[dimtestset];
	/// vector of the labels (targets): +1 class 1, -1 class 2. 
	T = new int[dimtestset];
	/// Bias vector. The bias can be 0 or 1
	B = new int[dimtestset];
	/// Unbiased variance vector
	Vu = new float[dimtestset];
	/// Biased variance vector
	Vb = new float[dimtestset];
	/// Net variance vector
	NV = new float[dimtestset];
	/// Variance vector
	V = new float[dimtestset];
	/// Expected error vector
	E = new float[dimtestset];
	/// Margins vector
	M = new float[dimtestset];
	/// matrix of the output of the numdatasets SVM on the test set
	P = new (float*)[numdatasets];
	for (int i=0; i<numdatasets; i++) 
	  P[i] = new float[dimtestset];
	load_predictions_matrix();
}

/**
	    Destructor.
*/
BV2decomp::~BV2decomp(void) {
  
	delete [] p1;
	delete [] Main;
	delete [] T;
	delete [] B;
	delete [] Vu;
	delete [] Vb;
	delete [] NV;
	delete [] V;
	delete [] E;
	delete [] M;
	if (P!=NULL) {
	  for (int i=0; i<numdatasets; i++) 
	    delete [] P[i];
	  delete [] P;
	}
	if (bsm!=NULL) {
	  for (int i=0; i<numdatasets; i++)
		  delete [] bsm[i];
		delete [] bsm;
	}
}

/**
	    Il loads the matrix of the predictions.
			The preictions performed by the base learners are read from the files
			predbase1, ..., predbasenumdatasets and stored in the matrix P.
			@see P
*/
void BV2decomp::load_predictions_matrix(void) {

  char filename[DIMBUF];
  ifstream fname;
  ostrstream obasename(filename,DIMBUF,ios::out);
  
	// open files for reading data and loading them in P
  for (int i=1; i<=numdatasets; i++) {
	  obasename.seekp(0,ios::beg);
		obasename << predbase << i << ".out" << ends;
    fname.open(filename);
    if (!fname) 
		  error_exit("Error opening ", filename);
    for(int j=0; j<dimtestset; j++)
      fname >> P[i-1][j];   
    if (fname.bad())  
		  error_exit("Error reading ", filename);
    fname.close();
  }
}





int BV2decomp::count_data_lines(char* dataset) {
  
	int nl = 0;
	string line;
  ifstream from(dataset);
  if (!from) 
	  error_exit("BV2decomp::count_data_lines. Cannot open input file ", dataset);
  
	while (getline(from,line))
	  if ((line.length()!=0) && (line[0] != '#'))
       nl++;
  if (!from.eof())
    error_exit("Error reading file in BV2decomp::count_data_lines");
  from.close(); 
  return nl;

}

/**
  It computes the probabilities for class 1 
*/
void BV2decomp::calc_p1(void) {
   
  // p1 computing
  for(int j=0; j<dimtestset; j++) {
    p1[j]=0;
    for (int i=0; i<numdatasets; i++) 
      if (P[i][j] > 0) ++p1[j];
    p1[j] /= numdatasets;
  }
}

/**
  It computes the probabilities for class 1 using out-of-bag predictions
	@param m matrix storing the out of bag examples for each predictor. m[i][j] = 1 means that the ith predictions
	on the jth examples is an out of bag prediction.
*/
void BV2decomp::calc_p1_oob(short ** m) {
  
	int K;
  for(int j=0; j<dimtestset; j++) {
    p1[j]=0;
    K=0;
		for (int i=0; i<numdatasets; i++) 
		  if (m[i][j] == 1)  { // oob example 
			  K++;
        if (P[i][j] > 0) ++p1[j];
			}
    if (K==0)  {  // estimates on all the data set
      for (int i=0; i<numdatasets; i++) 
        if (P[i][j] > 0) ++p1[j];
      p1[j] /= numdatasets;
    }
    else
      p1[j] /= K;
  }	
}

/**
  It computes the main predictions by majority
*/
void BV2decomp::calc_Main(void) {
 
  // Main decision M computation
  for(int j=0; j<dimtestset; j++)
    if (p1[j]> 0.5) Main[j] = +1;
    else Main[j] = -1;
}

/**
  It load the target vector.\\
	It reads the labels of the targets from the datatest
*/
void BV2decomp::load_T(void) {
  
  string line;
	ifstream data(datatest);
	 if (!data)
		 error_exit("Error opening ", datatest);	
	for  (int i=0; i<2; i++) {
	  getline(data,line);
		if (line[0] != '#')
		  error_exit("BV2decomp::load_T. Error reading header of ", datatest);
	}
  for(int j=0; j<dimtestset; j++) {
    data >> T[j]; 
		if ((T[j] != +1) && (T[j] != -1)) 
		  error_exit("Error reading ", datatest);
		getline(data,line);
	}
}

/// Compute Bias
void BV2decomp::calc_Bias(void) {
 
  for(int j=0; j<dimtestset; j++)
    B[j] = abs((Main[j]-T[j])/2); 
}


/**
  It computes unbiased, biased and net variance
*/
void BV2decomp::calc_Net_Variance(void) {
  int j;
  for(j=0; j<dimtestset; j++) {
    if (B[j] == 0) {
      Vb[j] = 0;
      if (p1[j] > 0.5)
        Vu[j] = 1 - p1[j];       
      else        
	Vu[j] = p1[j];
    }
    else {
      Vu[j] = 0;
      if (p1[j] <= 0.5) 
        Vb[j] = p1[j];       
      else        
	Vb[j] = 1 - p1[j];
    }
  NV[j] = Vu[j]-Vb[j];
  }
}

/**
  It computes  variance
*/
void BV2decomp::calc_Variance(void) {
  int cl;
  for(int j=0; j<dimtestset; j++) {
    V[j] = 0;
    for (int i=0; i<numdatasets; i++) {
      if (P[i][j] > 0) 
			  cl = +1;
      else cl = -1;
      if (cl != Main[j]) ++V[j];
    } 
    V[j] /= numdatasets;
  }
}


/**
  Computes Expected error
*/
void BV2decomp::calc_Error(void) {
  for(int j=0; j<dimtestset; j++)
    E[j] = B[j] + NV[j];
}


/**
  Computes the average values of bias-variance decomposition
*/
void BV2decomp::calc_Averages(void) {

  AvBV.init(0);
	
	for(int j=0; j<dimtestset; j++) {
    AvBV.bias+=B[j];
    AvBV.unbias_var+=Vu[j];
    AvBV.bias_var+=Vb[j];
    AvBV.net_variance+=NV[j];
		AvBV.variance+=V[j];
    AvBV.loss+=E[j];
  }
  AvBV.bias /= dimtestset;
  AvBV.unbias_var /= dimtestset;
  AvBV.bias_var /= dimtestset;
  AvBV.net_variance /= dimtestset;
  AvBV.variance /= dimtestset;
  AvBV.loss /= dimtestset;  
}


void BV2decomp::calc_margins(void) {
  
	for(int j=0; j<dimtestset; j++) 
	  M[j] = (2*B[j]-1) * (2*V[j]-1);
}

void BV2decomp::calc_cumulative_distr_margins(void) {
  
	float * m = new float[dimtestset];
	int curr = 0;
	int curr_counter = 0;
	int j = 0;
	float curr_cumul_value = 0;
	
	Mvalues.clear();
	Mcumul.clear();
	for(int i=0; i<dimtestset; i++) 
	  m[i] = M[i];
	sort(m, m+dimtestset);
		
	while (j<dimtestset) {
		if (m[j] == m[curr]) {
		  curr_counter++;
			j++; }
		else {
		  curr_cumul_value+=curr_counter;
			Mvalues.push_back(m[curr]);
			Mcumul.push_back(curr_cumul_value);
			curr = j;
			curr_counter = 0;		
		}
	}
	curr_cumul_value+=curr_counter;
	Mvalues.push_back(m[curr]);
	Mcumul.push_back(curr_cumul_value);	 
	int n = Mcumul.size();
	for (int i=0; i<n; i++)
	  Mcumul[i]/=dimtestset;
		
	delete [] m;
}

void BV2decomp::save_cumulative_distr_margins(char* file) {
  
	const int WIDTH = 16;
	ofstream ofs(file);
	if (!ofs)
	  error_exit("Cannot open ", file);
	ofs.setf(ios::left, ios::adjustfield);
  ofs.precision(8);
  for (int i=0; i<Mvalues.size(); i++)  {
	  ofs.width(WIDTH);
		ofs <<  Mvalues[i];
		ofs <<  Mcumul[i] << endl;
	}
	if (ofs.bad())
	  error_exit("Error writing ", file);
}


void BV2decomp::calc_bv_decomposition(AverageBV& AvgBV) {

  load_T();
	calc_p1();
	calc_Main();
	calc_Bias();
	calc_Net_Variance();
	calc_Variance();
	calc_Error();
  calc_Averages();
  AvgBV = AvBV;
}

void BV2decomp::calc_oob_bv_decomposition(AverageBV& AvgBV, short **m) {

  load_T();
	calc_p1_oob(m);
	calc_Main();
	calc_Bias();
	calc_Net_Variance();
	calc_Variance();
	calc_Error();
  calc_Averages();
  AvgBV = AvBV;
}

void BV2decomp::save_bv_per_sample(char* file) {
  
	const int WIDTH = 16;
  ofstream out(file);
	if (!out)
	  error_exit("Cannot open ", file);
	
	out.setf(ios::adjustfield, ios::left);
  out.precision(8);
  
	// header
	out << "# " << file << endl;
	out << "# Number of examples: " << dimtestset << endl;
	
	
	out.width(WIDTH); out << "# Loss";
	out.width(WIDTH); out << "Bias";
	out.width(WIDTH); out << "Net-var";
	out.width(WIDTH); out << "Unb-var";
	out.width(WIDTH); out << "Bias-var";
	out.width(WIDTH); out << "Variance" << endl << endl;
	
	for (int i=0; i<dimtestset; i++) {
	  out.width(WIDTH); out << E[i];
		out.width(WIDTH); out << B[i];
		out.width(WIDTH); out << NV[i];
		out.width(WIDTH); out << Vu[i];
		out.width(WIDTH); out << Vb[i];
		out.width(WIDTH); out << V[i];
	  out << endl;	
	}	
}


short ** BV2decomp::get_bs_matrix(char* bsmatrix) {
   
	ifstream ifs(bsmatrix);
	if (!ifs)
	  error_exit("Cannot open ", bsmatrix);
	// creating the matrix bsm
	bsm = new (short*)[numdatasets];
	for(int i=0; i<numdatasets; i++)	
		bsm[i] = new short[dimtestset];
	// loading the matrix
  for(int i=0; i<numdatasets; i++)
	   for(int j=0; j<dimtestset; j++)
		   ifs >> bsm[i][j];
  if (ifs.bad())
	  error_exit("Error reading ", bsmatrix);
	return bsm;
}

