#include<stdio.h>
#include<stdlib.h>
#include<math.h>
#include"chi2.h"
#include"utils.h"


#define ITMAX 1000
#define EPS 3.0e-7


float gammln(float xx)
{
	double x,tmp,ser;
	static double cof[6]={76.18009173,-86.50532033,24.01409822,
		-1.231739516,0.120858003e-2,-0.536382e-5};
	int j;

	x=xx-1.0;
	tmp=x+5.5;
	tmp -= (x+0.5)*log(tmp);
	ser=1.0;
	for (j=0;j<=5;j++) {
		x += 1.0;
		ser += cof[j]/x;
	}
	return -tmp+log(2.50662827465*ser);
}


void gser(float *gamser, float a, float x, float *gln)
{
	int n;
	float sum,del,ap;
	
	*gln=gammln(a);
	if (x <= 0.0) {
		if (x < 0.0) error_exit("chi2 : ", "x less than 0 in routine GSER");
		*gamser=0.0;
		return;
	} else {
		ap=a;
		del=sum=1.0/a;
		for (n=1;n<=ITMAX;n++) {
			ap += 1.0;
			del *= x/ap;
			sum += del;
			if (fabs(del) < fabs(sum)*EPS) {
				*gamser=sum*exp(-x+a*log(x)-(*gln));
				return;
			}
		}
		error_exit("chi2: ", "a too large, ITMAX too small in routine GSER");
		return;
	}
}



void gcf(float *gammcf, float a, float x, float *gln)
{
	int n;
	float gold=0.0,g,fac=1.0,b1=1.0;
	float b0=0.0,anf,ana,an,a1,a0=1.0;
	*gln=gammln(a);
	a1=x;
	for (n=1;n<=ITMAX;n++) {
		an=(float) n;
		ana=an-a;
		a0=(a1+a0*ana)*fac;
		b0=(b1+b0*ana)*fac;
		anf=an*fac;
		a1=x*a0+anf*a1;
		b1=x*b0+anf*b1;
		if (a1) {
			fac=1.0/a1;
			g=b1*fac;
			if (fabs((g-gold)/g) < EPS) {
				*gammcf=exp(-x+a*log(x)-(*gln))*g;
				return;
			}
			gold=g;
		}
	}
	error_exit("chi2: ", "a too large, ITMAX too small in routine gcf");
}



float gammp(float a, float x)
{
	float gamser,gammcf,gln;

	if (x < 0.0 || a <= 0.0) error_exit("chi2 : ", "Invalid arguments in routine GAMMP");
	if (x < (a+1.0)) {
		gser(&gamser,a,x,&gln);
		return gamser;
	} else {
		gcf(&gammcf,a,x,&gln);
		return 1.0-gammcf;
	}
}


float gammq(float a, float x)
{
	float gamser,gammcf,gln;

	if (x < 0.0 || a <= 0.0) error_exit("chi2 : ", "Invalid arguments in routine GAMMQ");
	if (x < (a+1.0)) {
		gser(&gamser,a,x,&gln);
		return 1.0-gamser;
	} else {
		gcf(&gammcf,a,x,&gln);
		return gammcf;
	}
}


/**
  Level of significativity Chi square test function. It returns the significativity level by 
	which the null hypothesis can be retained.
	@param pearson_stat value of the computed Pearson statistics
	@param degree degree of freedom for the chi square test.
	@return significativity level  by which the null hypothesis can be retained.
*/
float Prob_chi2(float pearson_stat, float degree) 
{
  return gammq(degree/2, pearson_stat/2);
}


/**
  Chi square test function. It returns if the null hypothesis can be discarded at significativity
	level sig_level.
	@param pearson_stat value of the computed Pearson statistics
	@param freedom_degree degree of freedom for the chi square test.
	@param sig_level significativity level  by which the null hypothesis can be discarded.
	@return 1 if the null hyphotesis can be discarded at significativity level sig_level, 0 otherwise.
*/
int Test_chi2(float pearson_stat, float freedom_degree, float sig_level) 
{
  float p; 
	p = gammp(freedom_degree/2, pearson_stat/2);
	if (p >= (1.0 - sig_level))
	  return 1;
	else
	  return 0;
}




#undef ITMAX
#undef EPS
