//  "compute_roc.cc"
// August 2004

/*
  Application for computing the ROC curves. \\
	Input: two files corresponding to the predicted and the true classes
	Output: a file with 4-plets (one for each row) corresponding to sensitivity, specificity, 1-specificity and precision.
	The last row of the file on stdout corresponds to the best specificity achieved when sensitivity=1, precison and corresponding
	threshold, that is: \\
	sensitivity  specificity  precision  threshold
*/

#include<stdlib.h>
#include"roc.h"
#include"utils.h"


static unsigned const SIZE = 256;

static char predicted[SIZE];  
static char trueclasses[SIZE];
static char rocfile[SIZE];



///////////////////////////////////  
static void print_info_use(void);


////////////////////////////////////////////////////////
int main(int argc, char* argv[])
{
	
  float thresh = 0;	
	ROCvalue roc_value;
	
  if (argc < 4) 
     print_info_use();
 
		 
	strcpy(predicted, argv[1]);
	strcpy(trueclasses, argv[2]);
	strcpy(rocfile, argv[3]);
	
	ROC roc(predicted, trueclasses);
	
	roc.compute_ROC_curve();
	roc_value = roc.compute_best_specificity (thresh);
		
	ofstream froc(rocfile);
  if (!froc) error_exit("compute_roc: Cannot open ROC file ", rocfile);
	roc.print_ROC_curve_all (froc);
	
	
	// print summary results on the last row:
	cout.precision(8);
  cout.setf(ios::left, ios::adjustfield);
  cout << endl << "Best Sensitivity-specificity:" << endl;
  cout.width(24);  cout << "sensitivity";
  cout.width(24);  cout << "specificity";
  cout.width(24);  cout << "precision";
  cout.width(24);  cout << "threshold" << endl;  
	cout.width(24);  cout << roc_value.get_sensitivity();
	cout.width(24);  cout << roc_value.get_specificity();
	cout.width(24);  cout << roc_value.get_precision();
  cout.width(24);  cout << thresh << endl;

  return 0;
}


void print_info_use(void)
{
  cerr << "
  compute_roc. NEURObjects 2.0 application for computing the ROC curves.
  Usage: compute_roc predicted trueclasses rocfile
  predicted     :  file with the values corresponding to the predicted classes
  trueclasses   :  file with the labels of the true classes
  rocfile       :  file with the ROC values (sensitivity, specificity, 1-specificity, precision)
  " << endl;
  exit(1);
}



