// "decomp.cc"

#include<iostream.h>
#include<math.h>
#include"decomp.h"


d_OPC::d_OPC (unsigned nclass, unsigned ndico) :
   n_dico(nclass),  n_class(nclass),
	 D(n_dico, n_class)
{
  unsigned i; 
  //pD = new matrix(n_dico, n_class);
  //D = *pD;
  Init_D(LOW);
  for (i=0; i<n_class; i++)
      D(i,i) = HIGH;
} 


d_OPC::d_OPC (char* dec)
{
  unsigned i,j;
	ifstream f(dec);
  if (!f) error_exit("Cannot open ", dec);
	f >> n_class;
	f >> n_dico;
  //pD = new matrix(n_dico, n_class);
  //D = *pD;
	matrix DD(n_dico,n_class);
	D = DD;
  for (i=0; i<n_dico; i++)
	  for (j=0; j<n_class; j++)
		  f >> D(i,j);
	if (f.bad())
    error_exit("Error reading Decomposition matrix file", dec); 
} 



d_OPC::d_OPC(const d_OPC& d)
{
  n_class = d.n_class;
  n_dico = d.n_dico;
  //pD = new matrix(d.D.dim1(), d.D.dim2());  
  //D = *pD;
  D = d.D;  
}
d_OPC& d_OPC::operator=(const d_OPC& d)
{
  n_class = d.n_class;
  n_dico = d.n_dico;
  //delete pD;  
  //pD = new matrix(d.D.dim1(), d.D.dim2());  
  D = d.D; 
  return *this; 
}
void d_OPC::Init_D(matrix_val val)
{
  unsigned i,j;  
  for (i=0; i<n_dico; i++)
    for (j=0; j<n_class; j++)
      D(i,j) = val;
}
void d_OPC::print_matrix(void)
{
  cout << "Decomposition matrix:" << endl;
  D.print();
}
void d_OPC::save_matrix(ofstream& f)
{
  unsigned i,j;
	f << n_class << endl;
	f << n_dico << endl;
  f.setf(ios::fixed, ios::floatfield);
  f.setf(ios::showpos);
  f.precision(0); 
  for (i=0; i<n_dico; i++)  {
    for (j=0; j<n_class; j++) 
      f << D(i,j) << " ";
    f << endl;
  }
  f << flush;
	if (f.bad())
    error_exit("Error writing file ");
} 

unsigned d_OPC::Hamming_distance(const vect& v1, const vect& v2)
{
	if (v1.dim() != v2.dim())
	  error_exit("HammingDistance: ","vector dimensions disagree");
	unsigned dim = v1.dim();	
	unsigned dist = 0;
	for (int i = 0; i<dim; i++)
	  if (v1[i] != v2[i]) dist++;
	return dist;
}


unsigned d_OPC::MinHamming(unsigned nclass)
{
  register unsigned i, j;
  unsigned mindist = 10000;
	unsigned dist;
  for (i=0; i<(n_class-1); i++) {
    for(j=i+1; j<n_class; j++)  {
		  dist = Hamming_distance(D.col(i),D.col(j));
			if (dist < mindist)
			  mindist = dist;
		}
	}
	return mindist;	  
}


float d_OPC::Avg_Hamming_distance(void)
{
  register int i, j;
	unsigned dist = 0;
  for (i=0; i<(n_class-1); i++) 
    for(j=i+1; j<n_class; j++)  
		  dist += Hamming_distance(D.col(i),D.col(j));
	float n =(float)n_class*(n_class-1)/2.0;
	return dist/n;	  
}


unsigned d_OPC::Min_Dichotomy_distance() {
  register int i, j, k;
  unsigned mindist = 10000;
	unsigned dist;
  for (i=0; i<(n_dico-1); i++) {
    for(j=i+1; j<n_dico; j++)  {
		  dist = Hamming_distance(D.row(i),D.row(j));
			if (dist < mindist)
			  mindist = dist;
		}
	}
	return mindist;	 
}  

float d_OPC::Avg_Dichotomy_distance(void)
{
  register int i, j;
	unsigned dist = 0;
  for (i=0; i<(n_dico-1); i++) 
    for(j=i+1; j<n_dico; j++)  
		  dist += Hamming_distance(D.row(i),D.row(j));
	float n =(float)n_dico*(n_dico-1)/2.0;
	return dist/n;	  
}

bool d_OPC::Banal_dichotomy(void) 
{
  vect v(n_class,1.0);
	for (int i=0; i<n_dico; i++)
	  if (fabs(v*D.row(i)) == (double)n_class)
		  return true;
	for (int i=0; i<(n_dico-1); i++) 
    for(int j=i+1; j<n_dico; j++)  
		  if ((D.row(i) * D.row(j)) == ((double)-n_class))
		    return true;						
  return false;
}

d_PWC::d_PWC (unsigned nclass, unsigned ndico) 
{    	
  n_dico= nclass*(nclass-1)/2;
	n_class = nclass;
	matrix DD (n_dico, n_class);
	D = DD; 
	Do_PWC_matrix();  
}

d_PWC::d_PWC (char* file) : d_OPC(file)
{	}

void d_PWC::Do_PWC_matrix(void)
{
  unsigned i,j,r;  
  Init_D(IGNORE);
  r = 0;
  for (i=0; i<(n_class-1); i++)  {
    j = i + 1;
    while (j<n_class)  {
      D(r,i) = HIGH;
      D(r,j) = LOW;
      j++; r++;
    }
  }
}
d_PWC::d_PWC(const d_PWC& d)
{
  n_class = d.n_class;
  n_dico = d.n_dico;
  //pD = new matrix(d.D.dim1(), d.D.dim2());  
  //D = *pD;
  D = d.D;  
}
d_PWC& d_PWC::operator=(const d_PWC& d)
{
  n_class = d.n_class;
  n_dico = d.n_dico;
  //delete pD;  
  //pD = new matrix(d.D.dim1(), d.D.dim2());  
  D = d.D; 
  return *this; 
}
d_CC::d_CC (unsigned nclass, unsigned ndico) 
{    
  n_dico= nclass*(nclass-1)/2;
	n_class=nclass;
	matrix DD (n_dico, n_class);
	D = DD; 
	Do_CC_matrix();  
}

d_CC::d_CC (char* file) : d_OPC(file)
{
  
}

void d_CC::Do_CC_matrix(void)
{
  unsigned i,j,r;  
  Init_D(LOW);
  r = 0;
  for (i=0; i<(n_class-1); i++)  {
    j = i + 1;
    while (j<n_class)  {
      D(r,i) = HIGH;
      D(r,j) = HIGH;
      j++; r++;
    }
  }
}
d_CC::d_CC(const d_CC& d)
{
  n_class = d.n_class;
  n_dico = d.n_dico;
  //pD = new matrix(d.D.dim1(), d.D.dim2());  
  //D = *pD;
  D = d.D;  
}
d_CC& d_CC::operator=(const d_CC& d)
{
  n_class = d.n_class;
  n_dico = d.n_dico;
  //delete pD;  
  //pD = new matrix(d.D.dim1(), d.D.dim2());  
  D = d.D; 
  return *this; 
}
d_ECOC_ex::d_ECOC_ex (unsigned nclass, unsigned ndico) 
{    
  n_dico = (unsigned)pow(2.0,(double)nclass-1) - 1;
	n_class=nclass;
	matrix DD (n_dico, n_class);
	D = DD; 
	Do_ECOC_matrix_exaustive();
}

d_ECOC_ex::d_ECOC_ex (char* file) : d_OPC(file)
{
  
}

void d_ECOC_ex::Do_ECOC_matrix_exaustive(void)
{  
  unsigned i,j,r,slides;
  unsigned block;  
  for (j=0; j<n_dico; j++)
    D(j,0) = HIGH;
  for (i=1; i< n_class; i++) {
    slides =(unsigned)pow(2.0,(double)i-1) - 1;
    block = (unsigned)pow(2.0,(double)n_class-i-1);
    r = 0;
    for (j=1; j<=slides; j++)  {   
      Append(i,r,block,LOW);
      r+=block;
      Append(i,r,block,HIGH);
      r+=block;
    }
    Append(i,r,block,LOW);  
    r+=block;
    Append(i,r,block-1,HIGH);   
    r+=block-1;
  }
}
void d_ECOC_ex::Append(unsigned col, unsigned rig, unsigned n, matrix_val val)
{
  unsigned j;
  unsigned endrig = rig + n;
  for (j=rig; j<endrig; j++)
    D(j,col) = val;
}
d_ECOC_ex::d_ECOC_ex(const d_ECOC_ex& d)
{
  n_class = d.n_class;
  n_dico = d.n_dico;
  //pD = new matrix(d.D.dim1(), d.D.dim2());  
  //D = *pD;
  D = d.D;  
}
d_ECOC_ex& d_ECOC_ex::operator=(const d_ECOC_ex& d)
{
  n_class = d.n_class;
  n_dico = d.n_dico;
  //delete pD;  
  //pD = new matrix(d.D.dim1(), d.D.dim2());  
  D = d.D; 
  return *this; 
}

d_ECOC_BCH::d_ECOC_BCH (unsigned nclass, unsigned ndico)   
{    
	n_dico=ndico;
	n_class=nclass;
	while  (n_class > pow(2.0,(double)n_dico)) {
		cerr << "read_p: Codeword length and number of classes not allowed" << endl;
		n_dico++;
	}
	matrix DD (n_dico, n_class);
	D = DD; 
  Do_ECOC_matrix_BCH();
}

d_ECOC_BCH::d_ECOC_BCH (char* file) : d_OPC(file) 
{}

void d_ECOC_BCH::Do_ECOC_matrix_BCH(void)
{  
  int i,j;
	
	
  compute_p(n_dico);      
	generate_gf();          
	gen_poly(n_class);      
	
	for (j=0; j<n_class; j++)  {	
		DecToBin(j, data, k);	 
		encode_bch();          
		 
		for (i = 0; i < n_dico - k; i++)
			recd[i] = bb[i];
		for (i = 0; i < k; i++)
			recd[i + n_dico - k] = data[i];
  	 
		for (i = 0; i < n_dico; i++)
		  if (recd[i] == 0)
			   D(i,j) = -1;	
		  else
			   D(i,j) = 1;
	}
	
	
	 
	vector<int> v(n_dico);
	for (i = 0; i < n_dico; i++) v[i]=0;
	int toeliminate = 0;
	for (i = 0; i < n_dico; i++){
	  for(j=0; j<n_class; j++)
		  if (D(i,j) == 1)
			   break;
		if (j == n_class) {
		  v[i] = 1;
			toeliminate ++;
	 }
	}
	
	 
	 
	 
	 
	int eliminated = toeliminate;
	int last = n_dico;
	j=-1;
	while (toeliminate) {	  		 
		while ((v[++j] == 0)) ;
		while ((v[--last] == 1)) ;
		toeliminate--;
		if (last < j)
		  break;
		for (i=0; i<n_class; i++)
		  D(j,i) = D(last,i);
	}	  
	n_dico-=eliminated;  
		
	 
	i=0;
	last = n_dico - 1;
	toeliminate = 0;
	
	while (i<=(last-1)) {
	  j=i+1;
		while (j<=last)  {
		  if (equalrow(i,j))  {
			  D.row(j) = D.row(last);
				last--;
				toeliminate++;
				j--;
		  }
			j++; 
		}
		i++;
	}
	n_dico-=toeliminate;  
	// Reducing the number of rows of the decomposition matrix
	matrix DD(n_dico, n_class);
	for (int i=0; i<n_dico; i++)
	  for (int j=0; j<n_class; j++)
	    DD(i,j) = D(i,j);
	D = DD;	
}
	
	
int d_ECOC_BCH::equalrow(int r1, int r2)
{
  int k=0;
	int res = 0;
	
	do {
	  if (D(r1,k) != D(r2,k))
		  break;
		k++;
	} while (k<n_class);
	if (k == n_class) res = 1;
	return res;
}
	
	
d_ECOC_BCH::d_ECOC_BCH(const d_ECOC_BCH& d)
{
	n_class = d.n_class;
  n_dico = d.n_dico;
  //pD = new matrix(d.D.dim1(), d.D.dim2());  
  //D = *pD;
  D = d.D;  
}
d_ECOC_BCH& d_ECOC_BCH::operator=(const d_ECOC_BCH& d)
{
	n_class = d.n_class;
  n_dico = d.n_dico;
  //delete pD;  
  //pD = new matrix(d.D.dim1(), d.D.dim2());  
  D = d.D; 
  return *this; 
}
void d_ECOC_BCH::compute_p(const int length)
{
	int			i, ninf;
	
	 
	 
	m = (int) ceil(log(n_dico+1.0)/log(2.0));
		
	for (i=1; i<m; i++)
		p[i] = 0;
	p[0] = p[m] = 1;
	if (m == 2)			p[1] = 1;
	else if (m == 3)	p[1] = 1;
	else if (m == 4)	p[1] = 1;
	else if (m == 5)	p[2] = 1;
	else if (m == 6)	p[1] = 1;
	else if (m == 7)	p[1] = 1;
	else if (m == 8)	p[4] = p[5] = p[6] = 1;
	else if (m == 9)	p[4] = 1;
	else if (m == 10)	p[3] = 1;
	else if (m == 11)	p[2] = 1;
	else if (m == 12)	p[3] = p[4] = p[7] = 1;
	else if (m == 13)	p[1] = p[3] = p[4] = 1;
	else if (m == 14)	p[1] = p[11] = p[12] = 1;
	else if (m == 15)	p[1] = 1;
	else if (m == 16)	p[2] = p[3] = p[5] = 1;
	else if (m == 17)	p[3] = 1;
	else if (m == 18)	p[7] = 1;
	else if (m == 19)	p[1] = p[5] = p[6] = 1;
	else if (m == 20)	p[3] = 1;
	
  n = 1;
	for (i = 0; i <= m; i++) 
    n *= 2;
	n = n / 2 - 1;
	ninf = (n + 1) / 2 - 1;
}
void d_ECOC_BCH::generate_gf()
{
	register int i, mask;
	mask = 1;
	alpha_to[m] = 0;
	for (i = 0; i < m; i++) {
		alpha_to[i] = mask;
		index_of[alpha_to[i]] = i;
		if (p[i] != 0)
			alpha_to[m] ^= mask;
		mask <<= 1;
	}
	index_of[alpha_to[m]] = m;
	mask >>= 1;
	for (i = m + 1; i < n; i++) {
		if (alpha_to[i - 1] >= mask)
		  alpha_to[i] = alpha_to[m] ^ ((alpha_to[i - 1] ^ mask) << 1);
		else
		  alpha_to[i] = alpha_to[i - 1] << 1;
		index_of[alpha_to[i]] = i;
	}
	index_of[0] = -1;
}
void d_ECOC_BCH::gen_poly(const int n_class)
{
	register int	ii, jj, ll, kaux;
	register int	test, aux, nocycles, root, noterms, rdncy;
	int             cycle[1024][21], size[1024], min[1024], zeros[1024];
	 
	cycle[0][0] = 0;
	size[0] = 1;
	cycle[1][0] = 1;
	size[1] = 1;
	jj = 1;			 
	
	do {
		 
		ii = 0;
		do {
			ii++;
			cycle[jj][ii] = (cycle[jj][ii - 1] * 2) % n;
			size[jj]++;
			aux = (cycle[jj][ii] * 2) % n;
		} while (aux != cycle[jj][0]);
		 
		ll = 0;
		do {
			ll++;
			test = 0;
			for (ii = 1; ((ii <= jj) && (!test)); ii++)	
			 
			  for (kaux = 0; ((kaux < size[ii]) && (!test)); kaux++)
			     if (ll == cycle[ii][kaux])
			        test = 1;
		} while ((test) && (ll < (n - 1)));
		if (!(test)) {
			jj++;	 
			cycle[jj][0] = ll;
			size[jj] = 1;
		}
	} while (ll < (n - 1));
	nocycles = jj;		 
	t = (n_dico/4) + 1;
	do {  	
		t--;
		d = 2 * t + 1;
		 
		kaux = 0;
		rdncy = 0;
		for (ii = 1; ii <= nocycles; ii++) {
			min[kaux] = 0;
			test = 0;
			for (jj = 0; ((jj < size[ii]) && (!test)); jj++)
				for (root = 1; ((root < d) && (!test)); root++)
					if (root == cycle[ii][jj])  {
						test = 1;
						min[kaux] = ii;
					}
			if (min[kaux]) {
				rdncy += size[min[kaux]];
				kaux++;
			}
		}
		noterms = kaux;
		kaux = 1;
		for (ii = 0; ii < noterms; ii++)
			for (jj = 0; jj < size[min[ii]]; jj++) {
				zeros[kaux] = cycle[min[ii]][jj];
				kaux++;
			}
		k = n_dico - rdncy;
    if (t<0)  
	    error_exit("d_ECOC_BCH::gen_poly : ", "Parameter error: t<0");
			
	} while (n_class > pow((double)2.0,(double)k));
	
	  
	 
	 
	g[0] = alpha_to[zeros[1]];
	g[1] = 1;		 
	for (ii = 2; ii <= rdncy; ii++) {
	  g[ii] = 1;
	  for (jj = ii - 1; jj > 0; jj--)
	    if (g[jj] != 0)
	      g[jj] = g[jj - 1] ^ alpha_to[(index_of[g[jj]] + zeros[ii]) % n];
	    else
	      g[jj] = g[jj - 1];
	  g[0] = alpha_to[(index_of[g[0]] + zeros[ii]) % n];
	}
}
void d_ECOC_BCH::encode_bch()
{
	register int    i, j;
	register int    feedback;
	for (i = 0; i < n_dico - k; i++)
		bb[i] = 0;
	for (i = k - 1; i >= 0; i--) {
		feedback = data[i] ^ bb[n_dico - k - 1];
		if (feedback != 0) {
			for (j = n_dico - k - 1; j > 0; j--)
				if (g[j] != 0)
					bb[j] = bb[j - 1] ^ feedback;
				else
					bb[j] = bb[j - 1];
			bb[0] = g[0] && feedback;
		} else {
			for (j = n_dico - k - 1; j > 0; j--)
				bb[j] = bb[j - 1];
			bb[0] = 0;
		}
	}
}
void d_ECOC_BCH::DecToBin(int num, int bin[], int len)
{
  int j =0;
	int res = num;
	int x;
	
	do {
	  x = res;
		res = res / 2;
		bin[j++] = x % 2;
  } while (res > 0);
	while (j<len)
	  bin[j++] = 0;
}
		
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
	  
d_Rand_gr_equal_constraints::d_Rand_gr_equal_constraints (unsigned nclass, unsigned ndico, unsigned min_dicho_dist, 
                                                          unsigned min_codeword_dist, unsigned ntimes) 
{    
	n_dico=ndico;
	n_class=nclass;
	matrix DD (n_dico, n_class);
	D = DD; ; 
	times = ntimes;
	seed = (long) -time(NULL);
  Do_Rand_matrix(min_dicho_dist, min_codeword_dist, ntimes);
}

d_Rand_gr_equal_constraints::d_Rand_gr_equal_constraints (char* file) : d_OPC(file)
{
  
}


void d_Rand_gr_equal_constraints::Make_Rand_matrix(unsigned min_dicho_dist)
{  
  const unsigned MAXIT = 100000;
	unsigned count = 0;
	unsigned i,j;
  bool found = false; 
	float res; 
	
	while (!found) {
	for (i=0; i<n_dico; i++)
	  for (j=0; j<n_class; j++) {
		   res = ran1(&seed);
			  
			 if (res<0.5)
			    D(i,j) = HIGH;
			 else 
			    D(i,j) = LOW;
		} 
	if ((Min_Dichotomy_distance() >= min_dicho_dist) && (!Banal_dichotomy()) && (MinHamming()>0))
	  found = true;
	if (++count > MAXIT)
	  error_exit("d_Rand::Make_Rand_matrix: max iterations reached.");
	}
}
unsigned d_Rand_gr_equal_constraints::Do_Rand_matrix(unsigned min_dicho_dist, unsigned min_codeword_dist, unsigned ntimes)
{
  unsigned i,min,ro;
	unsigned j,k;
	unsigned swap = 0;
	
	min=0;
	matrix MinD(n_dico, n_class);
	
	
	for (i=0; i<ntimes; i++)  {
	  Make_Rand_matrix(min_dicho_dist);
		ro = MinHamming(n_class);
		if (ro > min) {
		  swap = 1;
		  min = ro;
			for (j=0; j<n_dico; j++)
			  for (k=0; k<n_class; k++)
				   MinD(j,k) = D(j,k);
		}
		if (min >= min_codeword_dist) break;
	}
	if (swap)
	for (j=0; j<n_dico; j++)
			  for (k=0; k<n_class; k++)
				   D(j,k) = MinD(j,k);
	if (min < min_codeword_dist) {
	  cerr << "Do_Rand_matrix: Minimum codeword distance not reached " << endl;
		cerr << "Minimum Hamming distance = " << min << endl;
	}
	return min;
}		


vect& d_Rand_gr_equal_constraints::generate_row(vect& x, bool pos, int n_pos)
{
  float x1,x2;
	unsigned place;
	unsigned i, dim = x.dim();
	if (pos == false){
	  x1  = -1;  x2 = +1;
	}
	else {
	  x1  = +1;  x2 = -1;
	}
	for (i=0; i<dim; i++) 
	  x[i] = x1;
	for (i=0; i<n_pos; i++) {
	  do {
	  place = (unsigned)(ran1(&seed)*n_class);
		} while (x[place] == x2);
		x[place] = x2;
	}
  return x;
}


d_Rand_gr_equal_constraints * d_Rand_gr_equal_constraints::decrementHD(int n_pos, int ntimes, bool pos)
{
  unsigned i,j,row;
	unsigned minHamdist = MinHamming();
	unsigned mindico = Min_Dichotomy_distance();
	d_Rand_gr_equal_constraints * d;
	vect x(n_class);
	unsigned maxit = n_class*n_dico;
	for (j=0; j<ntimes; j++) {
		for (i=0; i<maxit; i++) {
	  	d = new d_Rand_gr_equal_constraints(*this);
			row = (unsigned)(ran1(&seed) * n_dico);
			d->D.row(row) = generate_row(x, pos, n_pos);	
			if ((d->Min_Dichotomy_distance() >= mindico) && (!d->Banal_dichotomy()) && (d->MinHamming()<minHamdist))
		  	return d;
			else
		  	delete d;
		}	
	}
	return NULL;
}

d_Rand_gr_equal_constraints * d_Rand_gr_equal_constraints::incrementHD(int n_pos, int ntimes, bool pos)
{
  unsigned i,j,row;
	unsigned minHamdist = MinHamming();
	unsigned mindico = Min_Dichotomy_distance();
	d_Rand_gr_equal_constraints * d;
	vect x(n_class);
	unsigned maxit = n_class*n_dico;
	for (j=0; j<ntimes; j++) {
		for (i=0; i<maxit; i++) {
	  	d = new d_Rand_gr_equal_constraints(*this);
			row = (unsigned)(ran1(&seed) * n_dico);
			d->D.row(row) = generate_row(x, pos, n_pos);	
			if ((d->Min_Dichotomy_distance() >= mindico) && (!d->Banal_dichotomy()) && (d->MinHamming()>minHamdist))
		  	return d;
			else
		  	delete d;
		}	
	}
	return NULL;  
}

		
 d_Rand_gr_equal_constraints:: d_Rand_gr_equal_constraints(const  d_Rand_gr_equal_constraints& d)
{
  n_class = d.n_class;
  n_dico = d.n_dico;
	seed = (long) -time(NULL);
  //pD = new matrix(d.D.dim1(), d.D.dim2());  
  //D = *pD;
  D = d.D;  
}
d_Rand_gr_equal_constraints& d_Rand_gr_equal_constraints::operator=(const d_Rand_gr_equal_constraints& d)
{
  n_class = d.n_class;
  n_dico = d.n_dico;
	seed = d.seed;
  //delete pD;  
  //pD = new matrix(d.D.dim1(), d.D.dim2());  
  D = d.D;
  return *this;  
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////

	  
d_Rand_equal_constraints::d_Rand_equal_constraints (unsigned nclass, unsigned ndico, unsigned min_dicho_dist, 
                                                    unsigned min_codeword_dist, unsigned ntimes) 
{    
	n_dico=ndico;
	n_class=nclass;
	matrix DD (n_dico, n_class);
	D = DD; ; 
	times = ntimes;
	seed = (long) -time(NULL);
  Do_Rand_matrix(min_dicho_dist, min_codeword_dist, ntimes);
}

d_Rand_equal_constraints::d_Rand_equal_constraints (char* file) : d_Rand_gr_equal_constraints(file)
{
  
}


unsigned d_Rand_equal_constraints::Do_Rand_matrix(unsigned min_dicho_dist, unsigned min_codeword_dist, unsigned ntimes)
{
  unsigned i,min,ro;
	unsigned j,k;
	unsigned swap = 0;
	
	min=100000;
	matrix MinD(n_dico, n_class);
	
	
	for (i=0; i<ntimes; i++)  {
	  Make_Rand_matrix(min_dicho_dist);
		ro = MinHamming(n_class);
		if (ro == min_codeword_dist) {
		  swap = 1;
		  min = ro;
			for (j=0; j<n_dico; j++)
			  for (k=0; k<n_class; k++)
				   MinD(j,k) = D(j,k);
			break;
		}
		else if (ro < min){
		  swap = 1;
		  min = ro;
			for (j=0; j<n_dico; j++)
			  for (k=0; k<n_class; k++)
				   MinD(j,k) = D(j,k);
		}
	}
	if (swap)
	for (j=0; j<n_dico; j++)
			  for (k=0; k<n_class; k++)
				   D(j,k) = MinD(j,k);
	if (ro != min_codeword_dist) {
	  cerr << "Do_Rand_matrix: Desired  codeword distance not reached " << endl;
		cerr << "Minimum Hamming distance = " << min << endl;
	}
	return min;
}		
		
 d_Rand_equal_constraints:: d_Rand_equal_constraints(const  d_Rand_equal_constraints& d)
{
  n_class = d.n_class;
  n_dico = d.n_dico;
	seed = d.seed;
  //pD = new matrix(d.D.dim1(), d.D.dim2());  
  //D = *pD;
  D = d.D;  
}
d_Rand_equal_constraints& d_Rand_equal_constraints::operator=(const d_Rand_equal_constraints& d)
{
  n_class = d.n_class;
  n_dico = d.n_dico;
	seed = (long) -time(NULL);
  //delete pD;  
  //pD = new matrix(d.D.dim1(), d.D.dim2());  
  D = d.D;
  return *this;  
}


////////////////////////////////////////////////////////////////////////////////////////////////////////////////

	  
d_Rand::d_Rand(unsigned nclass, unsigned ndico, unsigned min_dicho_dist) 
{    
	n_dico=ndico;
	n_class=nclass;
	matrix DD (n_dico, n_class);
	D = DD; ; 
	seed = (long) -time(NULL);
  Do_Rand_matrix(min_dicho_dist, 1, 1);
}

d_Rand::d_Rand (char* file) : d_Rand_gr_equal_constraints(file)
{
  
}


unsigned d_Rand::Do_Rand_matrix(unsigned min_dicho_dist, unsigned min_codeword_dist, unsigned ntimes)
{
  unsigned ro;	
	Make_Rand_matrix(min_dicho_dist);
  ro = MinHamming(n_class);
	return ro;
}		
		
 d_Rand:: d_Rand(const  d_Rand& d)
{
  n_class = d.n_class;
  n_dico = d.n_dico;
  //pD = new matrix(d.D.dim1(), d.D.dim2());  
  //D = *pD;
  D = d.D;  
}
d_Rand& d_Rand::operator=(const d_Rand& d)
{
  n_class = d.n_class;
  n_dico = d.n_dico;
	seed = (long) -time(NULL);
  //delete pD;  
  //pD = new matrix(d.D.dim1(), d.D.dim2());  
  D = d.D;
  return *this;  
}


////////////////////////////////////////////////////////////////////////////////////////////////////////////////

	  
d_Rand_constraints_multiple::d_Rand_constraints_multiple (unsigned nclass, unsigned ndico, char *file, unsigned min_dicho_dist, 
                                     unsigned min_codeword_dist, unsigned max_codeword_dist, unsigned ntimes)                                      
{    
	n_dico=ndico;
	n_class=nclass;
	matrix DD (n_dico, n_class);
	D = DD; ; 
	times = ntimes;
	seed = (long) -time(NULL);
  Do_Rand_matrix_multiple(min_dicho_dist, min_codeword_dist, max_codeword_dist, ntimes, file);
}

d_Rand_constraints_multiple::d_Rand_constraints_multiple (char* file) : d_Rand_gr_equal_constraints(file)
{}


void d_Rand_constraints_multiple::Do_Rand_matrix_multiple (unsigned min_dicho_dist, unsigned min_codeword_dist, 
                                                         unsigned max_codeword_dist, unsigned ntimes, char *file)
{
  unsigned i,ro;
	const unsigned DIMBUF = 128;  
  char namebuf[DIMBUF];
  bool v[n_dico];
  
  for (i=0; i < n_dico; i++)
    v[i] = false;
		
	for (i=0; i<ntimes; i++)  {
	  Make_Rand_matrix(min_dicho_dist);
		ro = MinHamming(n_class);
		if ((ro >= min_codeword_dist) && (ro <= max_codeword_dist) && (v[ro] == false)){
      v[ro] = true;
      ostrstream ost(namebuf,DIMBUF);
      ost << file << ro << '\0'; 
      ofstream fcv(namebuf);
      if (!fcv) error_exit("Cannot open file ", namebuf);
      save_matrix(fcv);
    }   		 
	}	
  cout << "Generated DM with minimum Hamm. dist: " << endl;
	for (i=0; i < n_dico; i++)
    if (v[i] == true)
      cout << i << "  ";
  cout << endl;
}		
		
 d_Rand_constraints_multiple:: d_Rand_constraints_multiple(const  d_Rand_constraints_multiple& d)
{
  n_class = d.n_class;
  n_dico = d.n_dico;
	seed = d.seed;
  //pD = new matrix(d.D.dim1(), d.D.dim2());  
  //D = *pD;
  D = d.D;  
}
d_Rand_constraints_multiple& d_Rand_constraints_multiple::operator=(const d_Rand_constraints_multiple& d)
{
  n_class = d.n_class;
  n_dico = d.n_dico;
	seed = (long) -time(NULL);
  //delete pD;  
  //pD = new matrix(d.D.dim1(), d.D.dim2());  
  D = d.D;
  return *this;  
}









