// dico.h


#ifndef DICO_H
#define DICO_H

#include<iostream.h>
#include<queue>
#include"act_fun.h"
#include"utils.h"
#include"layer.h"
#include"tset.h"
#include"learn.h"
#include"net.h"
#include"decomp.h"
#include"matrix.h"


/**
  Dichotomizer implemented through a single layer perceptron.\\
  @memo Simple perceptron dichotomizer - HTML documentation under construction.
*/  
class Dico1 : public  OneLayer {

protected:
 
  vect vectorclass;
    
  static const int IGNORE_CLASS  =  10;                 

public:
  
	enum binary_classification {OK, NOTOK}; 
	
  Dico1() {n_inputs = 0; n_outputs = 1; n_layers = 1;}
  
  Dico1(unsigned nin, vect& vectclass, act_func fout=sigmoid);
 
  ~Dico1() {}        
  
	
	
  virtual void init_weights_norm(int seed = 0); 
	
	     
 
  stop_cond Learn_by_pattern(TrainingSet& t_set, Learning& learn,
                                     unsigned& iter, double& error,  
                                     double errtol = ERR_TOL, unsigned maxits = MAXITS,
																		 bool printout = true);
                                     
 
  stop_cond Learn_by_epoch(TrainingSet& t_set, Learning& learn,
                                   unsigned& iter, double& error,
                                   double errtol = ERR_TOL, unsigned maxits = MAXITS,
																	 bool printout = true);           
 
  unsigned test(TrainingSet& tset); 
  
 
  void save_weights(char* namefile);
  
 
  virtual void save_weights(ofstream& f);
  
  double Calc_treshold(void);
	
	int bin_class(vect& out, double target, double treshold);
  
protected:
  
  void Do_vectorclass(vect& vectclass);

  
  double Calc_square_error(vect& t, vect& out);
  
  
  double calc_denom_RMS(vect& t, unsigned n_train); 
      
 
};



/**
  Dichotomizer implemented through a MLP with one hidden layer.\\
  @memo Two-Layer MLP dichotomizer - 	HTML documentation under construction.
*/  
class Dico2 : public Dico1 {

public:
  
  Dico2() {n_hidden1 = 0; n_layers = 2;}
  
  Dico2(unsigned nhidden, unsigned nin, vect& vectclass, act_func fout=sigmoid, act_func fhid1=sigmoid);
  
  ~Dico2() {}
  
 
  inline unsigned Nhidden1() const {return n_hidden1;} 
  
 
  void init_weights_norm(int seed = 0); 
  
  
  void init_weights_const(double x);
  
  
  void print_weights(void); 
  
  
  vect& compute (vect& vinput);  
  
  
  void backprop(vect& t, vect& x);
  
  
  void print_info_grad(void); 
  
  void save_weights(ofstream& f);
 
protected:
 
  unsigned n_hidden1;
};



/**
  Dichotomizer implemented through a MLP with two hidden layers.\\
  @memo Three-Layer MLP dichotomizer - HTML documentation under construction.
*/  
class Dico3 : public Dico2 {

public:
  
  Dico3() {n_hidden2 = 0; n_layers = 3;}
  
  
  Dico3(unsigned nhidden1, unsigned nhidden2, unsigned nin, vect& vectclass, 
       act_func fout=sigmoid, act_func fhid1=sigmoid, act_func fhid2=sigmoid);
       
 
  ~Dico3() {}
  
  
  inline unsigned Nhidden2() const {return n_hidden2;} 
  
  
  void init_weights_norm(int seed = 0); 
  
 
  void init_weights_const(double x);
  
  
  void print_weights(void); 
  
  
  vect& compute (vect& vinput);  
  
 
  void backprop(vect& t, vect& x);
  
  
  void print_info_grad(void); 
  
  
  void save_weights(ofstream& f);
  
 
protected:
 
  unsigned n_hidden2;
  
};






#endif
  
