//dodecomp.cc
#include<iostream>
#include<cstdlib>
#include"utils.h"
#include"decomp.h"

using namespace std;


static const unsigned DEF = 1;  
static const unsigned NCLASS = 4;
static const int DIMBUF = 128; 
static char namefile[DIMBUF];  
static unsigned type = DEF;   
static unsigned nc = NCLASS; 
static unsigned codlen = NCLASS; 
static unsigned mindico = DEF; 
static unsigned mincod = DEF; 
static unsigned maxcod = DEF*2; 
static unsigned ntimes = DEF;
static void print_info_use(void);
static int doarg(char* arg[], int i);

int main(int argc, char* argv[])
{
  d_Rand_gr_equal_constraints* D;
  
  if (argc < 2) print_info_use();
  strcpy(namefile, argv[1]);   
  for (int i=2; i<argc; i+=2)
    if (doarg(argv, i))
      print_info_use();
      
 
 // Decomposition matrix creation     
 switch (type)  {  
    case 0:  D = new d_Rand_constraints_multiple (nc, codlen, namefile, mindico, mincod, maxcod, ntimes);                              	                  
            break;  
    case 1:  D = new d_Rand_gr_equal_constraints(nc, codlen, mindico, mincod, ntimes);				                  
            break;    
    case 2:  D = new d_Rand_equal_constraints(nc, codlen, mindico, mincod, ntimes);				                  
            break;     
    case 3:  D = new d_Rand(nc, codlen, mindico);				                  
            break;   
    default: error_exit ("Random decomposition type must be between 1 and 3."); break;
  }
 
  if (type != 0){
  // Statisitcs 
  float MHD = D->MinHamming();
	float AHD = D->Avg_Hamming_distance();
	float ERP = MHD/codlen;
	float MDD = D->Min_Dichotomy_distance();
	float ADD = D->Avg_Dichotomy_distance();
 
 // Statistics outputs
 cout << "Results summary:" << endl;
 cout << "Error recovering power of the decomposition  : " << ERP << endl;
 cout << "Minimum Hamming distance between codewords   : " << MHD << endl;
 cout << "Average Hamming distance between codewords   : " << AHD << endl;
 cout << "Minimum dichotomy distance between codewords : " << MDD << endl;
 cout << "Average dichotomy distance between codewords : " << ADD << endl;
 cout <<  endl;
 cout <<  ERP << endl;
 cout <<  MHD << endl;
 cout <<  AHD << endl;
 cout <<  MDD << endl;
 cout <<  ADD << endl;
 
 // save decomposition matrix
 ofstream f(namefile);
 if (!f) error_exit("Cannot open file ", namefile);
 D->save_matrix(f);
 if (f.bad()) error_exit("Error writing file ", namefile);
 }
 
 delete D;
  
 return 0;
}

static void print_info_use(void)
{
  cout << "
  dodecomp. NEURObjects application for  automatic construction
            of random constrained decomposition matrices.
  Usage: dodecomp  matrixfile [options]
  matrixfile (string) : file that stores the decomposition matrix.
 Options:
 -t unsigned         type of the random decomposition matrix
                   0 : d_Rand_constraints_multiple
                   1 : d_Rand_gr_equal_constraints (default)
                   2 : d_Rand_equal_constraints
                   3 : d_Rand
 -nc unsigned        number of classes  (default = 4)
 -codlen unsigned    codeword length (default = number of classes)
 -mindico unsigned   minimun Hamming distance between dichotomies (default 1)
 -mincod unsigned    minimun Hamming distance between codewords   (default 1)
 -maxcod unsigned    maximun of the minimun Hamming distance between codewords   
 -ntimes unsigned    number of iterations (default 1)
	" << endl;  
  exit(1);
}

int doarg(char* arg[], int i)
{
  if (!strcmp(arg[i], "-t"))
    type = atoi(arg[i+1]);
  else if (!strcmp(arg[i], "-nc"))
    nc = atoi(arg[i+1]);
  else if (!strcmp(arg[i], "-codlen"))
    codlen = atoi(arg[i+1]);
  else if (!strcmp(arg[i], "-mindico"))
    mindico = atoi(arg[i+1]);
  else if (!strcmp(arg[i], "-mincod"))
    mincod = atoi(arg[i+1]);
  else if (!strcmp(arg[i], "-maxcod"))
    maxcod = atoi(arg[i+1]);
  else if (!strcmp(arg[i], "-ntimes"))
    ntimes = atoi(arg[i+1]);
  else 
    return 1;
  return 0;
}  















