// "ecocnet.cc"


#include<time.h>
#include<math.h>
#include<iostream.h>
#include<fstream.h>
#include<iomanip.h>
#include<string.h>
#include"net.h"
#include"utils.h"
#include"ecocnet.h"


OneLayerECOC::OneLayerECOC(unsigned nclass, unsigned nin, unsigned length, act_func fout)
: OneLayer(length) 
{
  n_layers = 1;
  n_inputs = nin;
	n_class = nclass;
	 
  scomp = new d_ECOC_BCH(nclass, length);
	minHam = scomp->MinHamming(nclass);  
	n_outputs = scomp->Read_n_dico();   
	
  layer[OUTPUT] = LayerNetTrain(n_outputs, nin, fout);
  Init_C();
	
	delete scomp;
}
void OneLayerECOC::Init_C(void)
{
  double on, off;
  int i, j;
	matrix& D = scomp->Read_matrix();
	 
	matrix m(n_outputs, n_class);
	C = m;
		
	 
	 
	act_func g = layer[OUTPUT].Read_g();  
  analyze (g, on, off);
  for (i=0; i<n_outputs; i++)
    for (j=0; j<n_class; j++)
		  if (D(i,j) == 1)
        C(i,j) = on;
      else
        C(i,j) = off;       
}
void OneLayerECOC::save_out(TrainingSet& tset, char* file)
{
  unsigned i,j;  
  int dim = layer[OUTPUT].Read_noutputs();  
  unsigned num_train = tset.Ntrain();
  vect& t = tset.Read_target();  
  matrix& x = tset.Read_data();    
  char buf[128];
  unsigned selected_class;
  strcpy(buf, file);
  strcat(buf, EOUT_EXT);
  ofstream f(buf,ios::out|ios::app);   
  if (!f) error("Cannot open output file ", buf);	
	
	save_decomp_matrix(f);
  f.setf(ios::fixed, ios::floatfield);
  f.setf(ios::showpos);
  f.precision(4);
  for (i = 0; i < num_train; i++) {
    vect& input = x.row(i);
    vect& out = compute(input);
    for (j=0; j<dim; j++)
      f << out[j] << " ";
    selected_class = computeclass(out);
    f << selected_class << " ";
    f << (unsigned)t[i] << endl; 
  }
  f << flush;
  if (f.bad())
    error("Errore writing output file ", buf);
}
	
TwoLayerECOC::TwoLayerECOC(unsigned nclass, unsigned nhidden, unsigned nin, unsigned length,
	                         act_func fout, act_func fhid1)
: OneLayerECOC(nclass, nhidden, length, fout) 
{
  n_layers = 2;
  n_inputs = nin;
  n_hidden1 = nhidden;
  layer[HIDDEN1] = LayerNetTrain(nhidden, nin, fhid1);
}
void TwoLayerECOC::init_weights_norm(int seed)
{ 
  layer[OUTPUT].init_norm_weights(seed);
  layer[HIDDEN1].init_norm_weights(seed);
  TwoLayerECOC::seed = seed;
}
void TwoLayerECOC::init_weights_const(double x)
{
  layer[OUTPUT].init_weights(x);
  layer[HIDDEN1].init_weights(x);
} 
void TwoLayerECOC::print_weights(void)
{
  cout << "Weight matrix od Two-Layer ECOC net :" << endl;
  cout << "Output layer:" << endl;
  layer[OUTPUT].print_weights();
  cout << "Hidden layer :" << endl;
  layer[HIDDEN1].print_weights();
} 
vect& TwoLayerECOC::compute (vect& vinput)
{
  vect& hidout = layer[HIDDEN1].compute(vinput);  
  return (layer[OUTPUT].compute (hidout));   
}
void TwoLayerECOC::backprop(vect& t, vect& x)
{
   layer[OUTPUT].calc_grad(t, layer[HIDDEN1].Read_out());  
   layer[HIDDEN1].backprop(layer[OUTPUT].Read_delta(), layer[OUTPUT].Read_W(), x);
}
void TwoLayerECOC::print_info_grad(void)
{
  int i;
  unsigned n_layers = Nlayers();
  cout << "Two-Layer ECOC net : Delta and gradient values. " << endl;
  for (i=0; i<n_layers; i++)  {
    if (i == 0)
      cout << "Output layer: " << endl;
    else
      cout << "Hidden layer : " << endl;
    layer[i].print_delta();
    layer[i].print_grad();
    layer[i].print_grad_bias();
  }
} 
ThreeLayerECOC::ThreeLayerECOC(unsigned nclass, unsigned nhidden1, unsigned nhidden2, unsigned nin, 
  unsigned length, act_func fout, act_func fhid1, act_func fhid2)
: TwoLayerECOC(nclass, nhidden1, nhidden2, length, fout, fhid1)
{
  n_layers = 3;
  n_hidden2 = nhidden2;
  n_inputs = nin;
  layer[HIDDEN2] = LayerNetTrain(nhidden2, nin, fhid2);
}
void ThreeLayerECOC::init_weights_norm(int seed)
{ 
  TwoLayerECOC::init_weights_norm(seed);
  layer[HIDDEN2].init_norm_weights(seed);
}
void ThreeLayerECOC::init_weights_const(double x)
{
  TwoLayerECOC::init_weights_const(x);
  layer[HIDDEN2].init_weights(x);
} 
void ThreeLayerECOC::print_weights(void)
{
  cout << "Weight matrix of the Three-Layer ECOC net :" << endl;
  cout << "Output layer :" << endl;
  layer[OUTPUT].print_weights();
  cout << "I hidden layer :" << endl;
  layer[HIDDEN1].print_weights();
  cout << "II hidden layer :" << endl;
  layer[HIDDEN2].print_weights();
} 
vect& ThreeLayerECOC::compute (vect& vinput)
{
  vect& hidout2 = layer[HIDDEN2].compute(vinput);  
  vect& hidout1 = layer[HIDDEN1].compute(hidout2);  
  return (layer[OUTPUT].compute (hidout1));   
}
void ThreeLayerECOC::backprop(vect& t, vect& x)
{
   layer[OUTPUT].calc_grad(t, layer[HIDDEN1].Read_out());  
   layer[HIDDEN1].backprop(layer[OUTPUT].Read_delta(), layer[OUTPUT].Read_W(), layer[HIDDEN2].Read_out()); 
   layer[HIDDEN2].backprop(layer[HIDDEN1].Read_delta(), layer[HIDDEN1].Read_W(), x); 
}
void ThreeLayerECOC::print_info_grad(void)
{
  int i;
  unsigned n_layers = Nlayers();
  cout << "Three-Layer ECOC : Delta and gradient values. " << endl;
  for (i=0; i<n_layers; i++)  {
    switch(i) {
       case 0 : cout << "Output layer : " << endl; break;
       case 1 : cout << "I hidden layer : " << endl; break;
       case 2 : cout << "II hidden layer : " << endl; break;
       default : error ("ThreeLayer::print_info_grad : error"); break;
    }    
    layer[i].print_delta();
    layer[i].print_grad();
    layer[i].print_grad_bias();
  }
} 
	
	
	
