//extract_data.cc
// Usage : extract_data filenamebase number_of_hidden_units

#include<iostream>
#include<fstream>
#include<strstream>
#include<cstdlib>

#define WIDTH 15

typedef struct {
 unsigned nclass; // Number of classes 
 unsigned codlen; // codeword length
 double ERP;      // Error recovering power of the decomposition
 unsigned MHD;    // Minimum Hamming distance between codewords
 double AHD;      // Average Hamming distance between codewords
 unsigned MDD;    // Minimum dichotomy distance between codewords
 double ADD;      // Average dichotomy distance between codewords
 double ErrorL1;  // Error rate L1 norm
 double ErrorL2;  // Error rate L2 norm
 double ErrorHD;  // Error rate Hamming distance
 double StdevL1;  // St.dev. error rate L1 norm
 double StdevL2;  // St.dev. error rate L2 norm
 double StdevHD;  // St.dev. error rate Hamming distance
 double ADE;      // Avg. dichotomizer error
 double MinDE;    // Min. dichotomizer error
 double MaxDE;    // Max. dichotomizer error
 double StdevDE;  // St.dev. dichotomizer error
} record;



using namespace std;


void read_record(ifstream& f, record rec[], unsigned i) {
  double x;
	f >> rec[i].nclass; 
	f >> rec[i].codlen; 
	f >> rec[i].ERP;
	f >> rec[i].MHD;
	f >> rec[i].AHD;
	f >> rec[i].MDD;
	f >> rec[i].ADD;
	f >> rec[i].ErrorL1; 
	f >> rec[i].StdevL1;
	f >> rec[i].ErrorL2;
	f >> rec[i].StdevL2;
	f >> rec[i].ErrorHD;
	f >> rec[i].StdevHD;
	f >> rec[i].ADE; 
	f >> rec[i].MinDE;
	f >> rec[i].MaxDE;
	f >> rec[i].StdevDE;
  if (f.bad()) {
	  cerr << "Error reading data file " << endl;
		exit(1);
	}
}


int main (int argc, char *argv[]) {
 
 const unsigned DIMBUF = 256;
 char datainput[DIMBUF];
 char dataoutput[DIMBUF];
 const unsigned vMHDdim = 15;
 unsigned vMHD[] = {1,2,3,4,5,6,7,8,9,10,11,12,13,14,15};
 record rec[vMHDdim];
 
 
 
 if (argc != 3) {
	 cerr << "extract_data filenamebase number_of_hidden_units" << endl;
	 exit(1);
 }
 
 ostrstream ostin(datainput,DIMBUF);
 ostrstream ostout(dataoutput,DIMBUF);
 ifstream f;
 ofstream of;
 // reading data files
 for (int i=1; i<=vMHDdim; i++) {
   ostin.seekp(0,ios::beg); 
	 ostin << argv[1] << "-" << vMHD[i-1] << "-h" << argv[2] << ".tail" << '\0'; 
	 f.open(datainput);
	 if (!f) { 
  	 cerr << "Cannot open file " <<  datainput << endl;
		 exit(1);
	 }
	 read_record(f,rec,i-1);
	 f.close();
 }
 
 // writing data files
 // writing MHD-error
 ostout << argv[1] << "-h" << argv[2] << "-MHD-error.data" << '\0'; 
 of.open(dataoutput);
 if (!of) { 
  	 cerr << "Cannot open file " <<  dataoutput << endl;
		 exit(1);
	 }
	 of << "# Data file : " << dataoutput << endl;
	 of << "# MHD   ErrorL1   St.devErrorL1   ADE   StdevDE" << endl;
 for (int i=0; i<vMHDdim; i++) { 	 	 
	 of.width(WIDTH); of.setf(ios::left,ios::adjustfield);
	 of << rec[i].MHD ; 
	 of.width(WIDTH); of.setf(ios::left,ios::adjustfield);
	 of << rec[i].ErrorL1; 
	 of.width(WIDTH); of.setf(ios::left,ios::adjustfield);
	 of << rec[i].StdevL1; 
	 of.width(WIDTH); of.setf(ios::left,ios::adjustfield);
	 of << rec[i].ADE; 
	 of.width(WIDTH); of.setf(ios::left,ios::adjustfield);
	 of << rec[i].StdevDE << endl;
 }
 of.close();
 
 // writing AHD-error
 ostout.seekp(0,ios::beg);
 ostout << argv[1] << "-h" << argv[2] << "-AHD-error.data" << '\0'; 
 of.open(dataoutput);
 if (!of) { 
  	 cerr << "Cannot open file " <<  dataoutput << endl;
		 exit(1);
	 }
	 of << "# Data file : " << dataoutput << endl;
	 of << "# AHD   ErrorL1   St.devErrorL1   ADE   StdevDE" << endl;
 for (int i=0; i<vMHDdim; i++) { 	 	 
	 of.width(WIDTH); of.setf(ios::left,ios::adjustfield);
	 of << rec[i].AHD ; 
	 of.width(WIDTH); of.setf(ios::left,ios::adjustfield);
	 of << rec[i].ErrorL1; 
	 of.width(WIDTH); of.setf(ios::left,ios::adjustfield);
	 of << rec[i].StdevL1; 
	 of.width(WIDTH); of.setf(ios::left,ios::adjustfield);
	 of << rec[i].ADE; 
	 of.width(WIDTH); of.setf(ios::left,ios::adjustfield);
	 of << rec[i].StdevDE << endl;
 }
 of.close();
 
 // writing MHD-decoding
 ostout.seekp(0,ios::beg);
 ostout << argv[1] << "-h" << argv[2] << "-MHD-decoding.data" << '\0'; 
 of.open(dataoutput);
 if (!of) { 
  	 cerr << "Cannot open file " <<  dataoutput << endl;
		 exit(1);
	 }
	 of << "# Data file : " << dataoutput << endl;
	 of << "# MHD   ErrorL1   St.devErrorL1   ErrorL2  St.devErrorL2  ErrorHD   St.devErrorHD" << endl;
 for (int i=0; i<vMHDdim; i++) { 	 	 
	 of.width(WIDTH); of.setf(ios::left,ios::adjustfield);
	 of << rec[i].MHD ; 
	 of.width(WIDTH); of.setf(ios::left,ios::adjustfield);
	 of << rec[i].ErrorL1; 
	 of.width(WIDTH); of.setf(ios::left,ios::adjustfield);
	 of << rec[i].StdevL1; 
	 of.width(WIDTH); of.setf(ios::left,ios::adjustfield);
	 of << rec[i].ErrorL2; 
	 of.width(WIDTH); of.setf(ios::left,ios::adjustfield);
	 of << rec[i].StdevL2;
	 of.width(WIDTH); of.setf(ios::left,ios::adjustfield);
	 of << rec[i].ErrorHD; 
	 of.width(WIDTH); of.setf(ios::left,ios::adjustfield);
	 of << rec[i].StdevHD << endl;
 }
 of.close();
 

 return 0;
}
