// layer.h


#ifndef LAYER_H
#define LAYER_H

#include<iostream.h>
#include"act_fun.h"
#include"utils.h"
#include"tset.h"
#include"matrix.h"




/**
  The class implements a layer of a neural net. \\
	@memo Class implementing a layer of a neural net - HTML documentation under construction.
*/  

class LayerNet  {
 
  friend class Learning;
  friend class GradientDescent;
  friend class MomentumDescent;
public:
  
  LayerNet() {n_inputs = 0 ;n_outputs = 0;}
  
  LayerNet (unsigned num_outputs, unsigned num_inputs, act_func f=sigmoid);
 
  virtual ~LayerNet () {}
  
  
  vect& compute (vect& vinput); 
  
  void print_weights(); 
  
  void print_out(); 
  
  void print_dim() const;  
  inline unsigned Read_ninputs() const {return n_inputs;} 
  inline unsigned Read_noutputs() const {return n_outputs;} 
  inline vect& Read_bias() {return bias;}
  inline vect& Read_out() {return out;}
  inline vect& Read_a() {return a;}
  inline matrix& Read_W() {return W;}  
  inline act_func Read_g() {return g;}
  inline act_func Read_dg() {return dg;} 
  void Print_info();                    
  
         
  void init_weights(double x); 
  
  void init_norm_weights(long seed = 0); 
  
  void load_weights(istream& f);  

protected:
  
  static  const unsigned WEIGHT_PRECISION = 8; 
	
	unsigned n_inputs;  
  
  unsigned n_outputs; 
  
  
  matrix W;    
       
  vect bias; 
        
  vect out; 
         
  vect a;  
  
           
  act_func g;   
      
  act_func dg;        
  
  act_func do_deriv (act_func f); 
}; 




/**
  Class implementing a layer of a neural net with learning for backpropagation learning. \\
	@memo Class implementing a layer of a neural net - HTML documentation under construction.
*/  

class LayerNetTrain : public LayerNet  {
  
  friend class Learning;
  friend class GradientDescent;
  friend class MomentumDescent;
public:
    
  
  LayerNetTrain() {n_inputs = 0 ;n_outputs = 0;}
 
  LayerNetTrain (unsigned num_outputs, unsigned num_inputs, act_func f=sigmoid);
  
  ~LayerNetTrain () {}
  
  
  
  matrix& calc_grad(vect& t, vect& vinput); 
  
  
  matrix& backprop(vect& delta_up, matrix& W_up, vect& vinput); 
                                                        
  
  inline vect& Read_delta() {return delta;}
  inline matrix& Read_grad() {return grad;}
  inline matrix& Read_DeltaW() {return DeltaW;}
  inline vect& Read_grad_bias()   {return grad_bias;}
  inline vect& Read_Delta_bias()  {return Delta_bias;} 
 
  void print_delta(void);
 
  void print_grad(void);
  
  void print_DeltaW(void);
  
  void print_grad_bias(void);
  
  void print_Delta_bias(void);
  void print_info(void);  
  
  
  void save_weights(ostream& f);  
                                    
protected: 
  
  vect delta; 
  
  matrix grad;  
  
  matrix DeltaW; 
                                                        
      
  vect grad_bias; 
  
  vect Delta_bias; 
  
  
  vect& calc_output_delta(vect& t); 
  
  vect& calc_hidden_delta(vect& delta_up, matrix& W_up); 
  
}; 


#endif
