// learn.h 

#ifndef LEARN_H
#define LEARN_H

#include<iostream.h>
#include"utils.h"
#include"tset.h"
#include"layer.h"
#include"matrix.h"

#define MAXLAYERS 3




/**
  Basic abstract learning class.\\
	@memo Virtual base class for learning - HTML documentaiton under construction.   
*/  
class Learning {

protected:
  
	static const double DEFAULT_ETA = 0.1;
  static const double DEFAULT_DECR_FACTOR = 0.998;  // default decrement rate factor
  static const double MIN_ETA = 2.0e-03;  // default starting rate
  static const double DEFAULT_DECR_LIN_FACTOR = 2.0e-04;   // default decrement linear rate factor
  static const double DEFAULT_ALPHA = 0.5; // default momentum coefficient
	static const double DEFAULT_ETA_INCR = 1.04; // default learning rate increment factor (bold)	
  static const double DEFAULT_ETA_DECR = 0.5; // default learning rate decrement factor  (bold)	
  
  double eta;   
public:
 
  virtual void Update_delta_weights(unsigned n_layers, LayerNetTrain layer[]) = 0; 
  
  virtual void Update_weigths(unsigned n_layers, LayerNetTrain layer[]) = 0;
  
  virtual void Update_param(double err, unsigned iter) = 0; 
	
	void Init_rate (double val=DEFAULT_ETA) {eta = val;}
	
	double Read_rate(void) {return eta;}
}; 


/**
  Gradient descent learning class. It updates weights through simple gradient descent with fixed
	learning rate.\\
	@memo Gradient descent learning algorithm - HTML documentaiton under construction.   
*/  
class GradientDescent : public Learning {

protected:
  
  void Zero_Delta_W(unsigned n_layers, LayerNetTrain layer[]);  
  
public:
  
  
  GradientDescent(double x = DEFAULT_ETA) {eta = x;}
  
  virtual ~GradientDescent() {}
  
  
  void Update_delta_weights(unsigned n_layers, LayerNetTrain layer[] ) ;
                      
  
  void Update_weigths(unsigned n_layers, LayerNetTrain layer[]);
                               

  void Update_param(double err, unsigned iter){} 
	
}; 




/**
  Gradient descent learning class with exponential decrement of learning rate. \\
	It updates weights through  gradient descent with exponential decrement of
	learning rate.\\
	@memo Gradient descent with exponential decrement of learning rate - HTML documentaiton under construction.   
*/  
class GradDescDecrEtaExp : public GradientDescent {

protected:
  
  double decr_factor;
  
public:
  
  GradDescDecrEtaExp(double x = DEFAULT_ETA, double d_fact = DEFAULT_DECR_FACTOR) 
  : GradientDescent(x)
  { decr_factor = d_fact;}
 
  ~GradDescDecrEtaExp() {}
  

void Update_param(double err, unsigned iter); 
}; 


/**
  Gradient descent learning class with linear decrement of learning rate. \\
	It updates weights through  gradient descent with linear decrement of
	learning rate.\\
	@memo Gradient descent with linear decrement of learning rate -
		HTML documentaiton under construction.   
*/  
class GradDescDecrEtaLin : public GradientDescent {

protected:
 
  double decr_factor;
  
public:
  
  GradDescDecrEtaLin(double x = DEFAULT_ETA, double d_fact = DEFAULT_DECR_LIN_FACTOR) 
  : GradientDescent(x)
  { decr_factor = d_fact;}
 
  ~GradDescDecrEtaLin() {}
 
  
  void Update_param(double err, unsigned iter); 
}; 


/**
  Gradient descent learning class with momentum \\
	It updates weights through  gradient descent with momentum term.\\
	@memo Gradient descent with  momentum - HTML documentaiton under construction.   
*/  
class MomentumDescent : public  GradientDescent {

protected:
  
  double alpha;  
  
  matrix* Delta_W_total[MAXLAYERS]; 
  
  vect* delta_bias_total[MAXLAYERS]; 
  
  
  void Zero_Delta_W_total(unsigned n_layers);
  
public:
  
  MomentumDescent(unsigned n_output, unsigned n_input,
                  double x, double y=DEFAULT_ALPHA ); 
  
  MomentumDescent(unsigned n_output, unsigned n_hidden, unsigned n_input,
                  double x, double y=DEFAULT_ALPHA); 
  
  MomentumDescent(unsigned n_output, unsigned n_hidden1, unsigned n_hidden2, unsigned n_input,
                  double x, double y=DEFAULT_ALPHA);
  
  virtual ~MomentumDescent();
  
  
  void Update_weigths(unsigned n_layers, LayerNetTrain layer[] );                      
}; 




/**
  Gradient descent learning class with exponential decrement of learning rate and momentum. \\
	It updates weights through  gradient descent with exponential decrement of
	learning rate and momentum term.\\
	@memo Gradient descent with exponential decrement of learning rate and momentum.	
	      HTML documentaiton under construction.   
*/  
class MomentumDescentDecrEtaExp : public MomentumDescent {

protected:
 
  double decr_factor;
  
public:
  
  MomentumDescentDecrEtaExp(unsigned n_output, unsigned n_input,
                  double x, double y, double d_fact ); 
 
  MomentumDescentDecrEtaExp(unsigned n_output, unsigned n_hidden, unsigned n_input,
                  double x, double y, double d_fact); 
 
  MomentumDescentDecrEtaExp(unsigned n_output, unsigned n_hidden1, unsigned n_hidden2, unsigned n_input,
                  double x, double y, double d_fact); 
 
  ~MomentumDescentDecrEtaExp() {}
  
  
  void Update_param(double err, unsigned iter);                      
}; 


/**
  Gradient descent learning class with linear decrement of learning rate and momentum. \\
	It updates weights through  gradient descent with linear decrement of
	learning rate and momentum term.\\
	HTML documentaiton under construction.
	@memo Gradient descent with linear decrement of learning rate and momentum.   
*/  
class MomentumDescentDecrEtaLin : public  MomentumDescent {

protected:
 
  double decr_factor;
public:
 
  MomentumDescentDecrEtaLin(unsigned n_output, unsigned n_input,
                  double x, double y, double d_fact = DEFAULT_DECR_LIN_FACTOR ); 
  
  MomentumDescentDecrEtaLin(unsigned n_output, unsigned n_hidden, unsigned n_input,
                  double x, double y, double d_fact = DEFAULT_DECR_LIN_FACTOR); 
  
  MomentumDescentDecrEtaLin(unsigned n_output, unsigned n_hidden1, unsigned n_hidden2, unsigned n_input,
                  double x, double y, double d_fact = DEFAULT_DECR_LIN_FACTOR); 
  
  ~MomentumDescentDecrEtaLin() {}
  
  
  void Update_param(double err, unsigned iter);                      
}; 



/**
  Bold driver learning class. \\
	It updates weights through  gradient descent with adaptive
	learning rate.\\
	@memo Bold driver learning algorithm - HTML documentaiton under construction.   
*/  
class BoldDriver : public GradientDescent {

protected:
  
  double eta_incr;
  
  double eta_decr;
  
  double err_prev;
	
 
public:
 
  BoldDriver(double x = DEFAULT_ETA, double incr = DEFAULT_ETA_INCR, double decr = DEFAULT_ETA_DECR) 
  : GradientDescent(x)
  {eta_incr = incr; eta_decr = decr; err_prev = 0.0;}
 
  ~BoldDriver() {}
                                 
 
  void Update_param(double err, unsigned iter); 
  
  
  void Zero_err_prev(void){err_prev = 0;}
}; 




/**
  Bold driver with momentum learning class. \\
	It updates weights through  gradient descent with adaptive
	learning rate and momentum term.\\
	@memo Bold driver with momentum learning algorithm - 	HTML documentaiton under construction.   
*/  
class BoldMomentum : public  MomentumDescent {

protected:
 
  double eta_incr;
  
  double eta_decr;
 
  double err_prev;
  
public:
 
  BoldMomentum(unsigned n_output, unsigned n_input,
                  double x, double y, double incr = DEFAULT_ETA_INCR, double decr = DEFAULT_ETA_DECR ); 
                  
 
  BoldMomentum(unsigned n_output, unsigned n_hidden, unsigned n_input,
                  double x, double y, double incr = DEFAULT_ETA_INCR, double decr = DEFAULT_ETA_DECR ); 
                  
  
  BoldMomentum(unsigned n_output, unsigned n_hidden1, unsigned n_hidden2, unsigned n_input,
                  double x, double y, double incr = DEFAULT_ETA_INCR, double decr = DEFAULT_ETA_DECR );
                  
 
  ~BoldMomentum() {}
  
 
  void Update_param(double err, unsigned iter);                     
}; 




/**
  Class for storing info about learning algorithm.\\
	@memo It stores info about learning algorithm - HTML documentaiton under construction.  
*/  
class LearnType  {
  
public:
 
 enum learn_mode {BYPATTERN, BYEPOCH};
 
  
  Learning* learn;
  
  
  unsigned maxits;
  
  
  double err_tol;
  
  
  learn_mode mode;
 
 
 LearnType(Learning* l, unsigned maxit, double errmax, learn_mode m)
 {
   learn = l;
   maxits = maxit;
   err_tol = errmax;
   mode = m;
 }
 
 
 ~LearnType() {}   
 
 
};



#endif



