// lobag.cc

#include<strstream.h>
#include"lobag.h"
#include"utils.h"

void par_model::print (ostream& out) {
  const int WIDTH = 16;
	const int WIDTH_RID = 8;
	out.setf(ios::adjustfield, ios::left);
	switch (type) {
	  case 0:  out.width(WIDTH_RID); out << C;   
		        break;
		case 1:  out.width(WIDTH_RID); out << C;
		        out.width(WIDTH_RID); out << degree;  
						break;
		case 2: case 3: out.width(WIDTH_RID); out << C;
		             out.width(WIDTH_RID); out << s;  
						break;
    default:  cout << " par_model::print: Not known SVM kernel type" << endl;
	           exit(1); 
	}
}


Lobag::Lobag() : base(NULL), bagSVM(NULL), min_bias(), min_loss()
{}
										 
             
Lobag::Lobag ( char* modelfile, char* data, int nf, int seed, char* dir) :
 base(NULL), bagSVM(NULL), min_bias(), min_loss()
{
  char basebs[256];
	strcpy(learning_set,data);	
  B_min = L_min = B_L_min = 1;
	B = nf;
	ndata = wc(data) - 2;
	dirbs = dir;   
	ostrstream obasebs(basebs,256,ios::out);
	obasebs << dir << "/" << BS_NAME << ends;
	b = new BootstrapLight (data, basebs, B, ndata, seed);
	diminput = wc_word(b->get_strings()[0]) - 1;
	svm_type = load_model_parameters(modelfile);
}

Lobag::~Lobag() {
 if (b!=NULL)
    delete b;
 if (base != NULL)
	    delete base;
 if (bagSVM != NULL)
	  delete bagSVM;
}


int Lobag::load_model_parameters(char* modelfile) {

	par_model pm;
	char buf[256];
	int svm_t;
	deque<float> C;
	deque<float> sigma;
	deque<float> degree;
	typedef deque<float>::iterator Itf;
	
	ifstream ifs (modelfile);
	if (!ifs)
	  error_exit("Cannot open file ", modelfile);
	// reading svm type
	ifs >> buf;
	ifs >> svm_t;
	// reading C parameters
	read_param(ifs, C);
		
	switch (svm_t) {
	  case 0:  for (Itf j = C.begin(); j != C.end(); j++) {
	            pm.C = *j;
							pm.type = svm_t;
		          Param.push_back(pm);
	          } 		        
		        break;
		case 1:  // reading degree parameters
		        read_param(ifs, degree);
						for (Itf j = C.begin(); j != C.end(); j++) {
	            pm.C = *j;
							pm.type = svm_t;
		          for (Itf i = degree.begin(); i != degree.end(); i++) {
							  pm.degree = (int)(*i);
								Param.push_back(pm);
						  }
						} 		 
						break;
		case 2: case 3:  // reading sigma parameters
		        read_param(ifs, sigma);
						for (Itf j = C.begin(); j != C.end(); j++) {
	            pm.type = svm_t;
							pm.C = *j;
		          for (Itf i = sigma.begin(); i != sigma.end(); i++) {
							  pm.s = *i;
								Param.push_back(pm);
							}
	          } 		 
						break;
    default:  cout << "load_model_parameters: Not known SVM kernel type" << endl;
	           exit(1); 
	}

  return svm_t;
}


void Lobag::read_param(ifstream &ifs, deque<float>& d) {
  
	const char * END = "end";
	char buf[256];
	while (!ifs.eof() ) {
	  ifs >> buf;
		if (!strcmp(buf,END))
		  break;
		d.push_back(atof(buf));
	}
	if (strcmp(buf,END))
	  error_exit ("Lobag::read_param: bad format");
}



void Lobag::do_BV_decomposition(par_model pm,  char* dir_models, char* dir_res, model_BV& mbv) {
  
	const int SIZE = 256;
	char basetrain [SIZE];
	char basemodel [SIZE];
  
	base = build_base_learner (pm);
	// building the bagged ensemble of SVM
	if (bagSVM != NULL) {
	  delete bagSVM; bagSVM=NULL;}
	bagSVM = new Bagging(base);

	// training of the bagged SVM ensemble on the bootstrapped samples
	strcpy(basetrain, dirbs);
	strcat(basetrain, "/"); strcat(basetrain, BS_NAME);
	strcpy(basemodel, dir_models); 
	strcat(basemodel, "/"); strcat(basemodel, MODEL_NAME);
	bagSVM->train_base_model (basetrain, basemodel, 1, B, false, false);

	// testing on the overall learning set. Prediction are stored in files dir_res/BASRESn.out and dir_res/BASRESn
	bagSVM->test_base_learners(learning_set, 1, B, dir_res);

	// evaluating bias-variamce decomposition on the current model
  char predictedbasename [256];
	ostrstream opred(predictedbasename,256,ios::out);
	opred << dir_res << "/" << BASRES << ends;
	BV2decomp bv(learning_set, predictedbasename, B);
	AverageBV AvgBV;
  bv.calc_oob_bv_decomposition(AvgBV, b->get_m());			
	// storing bias-variance decompostion results
	mbv.P = pm;
	mbv.AvBV = AvgBV;
}


BaseSVM_light_common* Lobag::build_base_learner (par_model pm) {
    
		 float C, degree, sigma, s;
		// preparing the base learner
		if (base != NULL) {
	    delete base; base=NULL;}
		C = pm.C;
	  switch (pm.type) {
	  case 0:  // linear kernel						 
		        base = new BaseSVM_light_linear (C);
		        break;
		case 1:  // poly kernel
						s = 1/(float)diminput;
						degree = pm.degree;						
		        base = new BaseSVM_light_poly ((int)degree, C, s);
						break;
		case 2:  // gaussian kernel
						sigma = pm.s;
						base = new BaseSVM_light_gaussian (1.0/sigma, C);
						break;
		case 3:  // sigmoid kernel
		        s = pm.s;
						base = new BaseSVM_light_sigmoid (C, s);
						break;
    default:  cout << "Not known SVM kernel type" << endl;
	            exit(1); 
		}
		if (base == NULL)
	  	error_exit("Lobag::build_base_learner", "Error creating base SVM");
		return base;
}


void Lobag::exhaustive_BV_decomposition(char* dir_models, char* dir_res) {

	typedef deque<par_model>::iterator It_par_model;
	model_BV mbv;
	
	for (It_par_model pm = Param.begin(); pm != Param.end(); pm++) {		
		do_BV_decomposition(*pm, dir_models, dir_res, mbv);
		V.push_back(mbv);		
	} 	
}
  
/** 
   Prints the values of bias-variance decompostion for all the models.
	 It prints the values of the loss, bias, net, unbiased and biased variance and of the pure variance
	 of all the considered models.
	 @param out output stream where the values are printed
*/
void Lobag::print_BV(ostream& out) {
  	
	typedef deque<model_BV>::iterator Itm;
	const int WIDTH = 16;
	const int WIDTH_RID = 8;
  
  out.setf(ios::adjustfield, ios::left);
	// header
	out << "# " << learning_set << endl;
	out << "# Number of examples: " << ndata << endl;
		
	switch (svm_type) {
	  case 0:  out.width(WIDTH_RID); out << "# C";    
		        break;
		case 1:  out.width(WIDTH_RID); out << "# C";    
		        out.width(WIDTH_RID); out << "degree";
						break;
		case 2: out.width(WIDTH_RID); out << "# C";    
		       out.width(WIDTH_RID); out << "sigma";
						break;
		case 3: out.width(WIDTH_RID); out << "# C";    
		       out.width(WIDTH_RID); out << "s";
						break;
    default:  cout << "Lobag::print_BV: Not known SVM kernel type" << endl;
	           exit(1); 
	}
	out.width(WIDTH); out << "Loss";
	out.width(WIDTH); out << "Bias";
	out.width(WIDTH); out << "Net-var";
	out.width(WIDTH); out << "Unb-var";
	out.width(WIDTH); out << "Bias-var";
	out.width(WIDTH); out << "Variance" << endl << endl;
  
	// BV values
	for (Itm p = V.begin(); p != V.end(); p++) 
		(*p).print(out);
}	


/** 
   Save the values of bias-variance decompostion for all the models.
	 It saves the values of the loss, bias, net, unbiased and biased variance and of the pure variance
	 of all the considered models.
	 @param file file where the values will be saved
*/
void Lobag::save_BV(char* file) {
  ofstream ofs(file);
	if (!ofs)
	  error_exit("Cannot open ", file);
	print_BV(ofs);
	if (ofs.bad())
	  error_exit("Error writing ", file);
}



void Lobag::Select_model(void) {

  typedef deque<model_BV>::iterator Itm;
  B_min = L_min = B_L_min = 1;
	
	for (Itm p = V.begin(); p != V.end(); p++) {
	  model_BV m = *p;
		if (m.AvBV.loss < L_min) {
		  L_min = m.AvBV.loss;
			min_loss = m;		
		}
		if (m.AvBV.bias < B_min) {
		  B_min = m.AvBV.bias;
			min_bias = m;
		}		
	}
  B_L_min = min_loss.AvBV.bias;
}


void Lobag::print_oob_error(ostream& out) {
  
  const int WIDTH = 16;
	const int WIDTH_RID = 8;
  
  out.setf(ios::adjustfield, ios::left);
	// header
	out << "# " << "Out of bag estimates of the error of the lobag and bagged ensemble, and of the single SVM." <<
	endl;
	out.width(WIDTH); out << "Lobag :";
	print_oob_parameters(min_bias, out);
	out.width(WIDTH); out << "Error = "; 	
	out.width(WIDTH); out << B_min << endl; 	  
	
	out.width(WIDTH); out << "Bagging :";
	print_oob_parameters(min_loss, out);
	out.width(WIDTH); out << "Error = "; 	
	out.width(WIDTH); out << B_L_min << endl; 	  
	
	out.width(WIDTH); out << "Single SVM :";
	print_oob_parameters(min_loss, out);
	out.width(WIDTH); out << "Error = "; 	
	out.width(WIDTH); out << L_min << endl << endl; 	  
}


void Lobag::print_oob_parameters(const model_BV& mbv, ostream& out) {

  const int WIDTH = 16;
	switch (mbv.P.type) {
	  case 0:  out.width(WIDTH); out << "linear, C = ";
            out.width(WIDTH); out << mbv.P.C; 	           
		        break;
		case 1:  out.width(WIDTH); out << "polynomial, C = ";
            out.width(WIDTH); out << mbv.P.C;   
		        out.width(WIDTH); out << "degree = ";
            out.width(WIDTH);  out << mbv.P.degree;
						break;
		case 2: out.width(WIDTH); out << "gaussian, C = ";
          out.width(WIDTH); out<< mbv.P.C;   
		       out.width(WIDTH); out << "sigma = ";
           out.width(WIDTH); out  << mbv.P.s;
						break;
		case 3: out.width(WIDTH); out << "sigmoid, C = ";
           out.width(WIDTH); out << mbv.P.C;   
		       out.width(WIDTH); out << "s = ";
           out.width(WIDTH); out   << mbv.P.s;
						break;
    default:  cout << "Not known SVM kernel type" << endl;
	           exit(1); 
	}
}

