//  "lobagsvm.cc"
// December 2002
/**
  Application  implementing the lobag algorithm.\\
	SVMs are used as base learners. Itvprovides an out-of-bag estimate of the error for lobag
	ensemble of SVMs, using the exhaustive algorithm.

*/

#include<stdlib.h>
#include"utils.h"
#include"lobag.h"

static unsigned const SIZE = 256;
// default directory for the bootstrapped samples
static  char* const DIRBS_LSVM = "tmpbs";
// default directory for the models
static char* const DIRMODELS_LSVM = DIRBS_LSVM;
// default directory for the results computed by the base learners
static  char* const DIR_RES_LSVM = DIRBS_LSVM;
static bool b_save_BVfile = false;

// file that the describe the model to trained and tested
static char modelfile[SIZE]; 
// dataset from which samples will be bootstrapped  
static char dataset[SIZE];  
// file of the bias--varaince values computed for each model
static char BVfile[SIZE];
// number of bootstrapped samples
int nsample = 60;
// seed for the random generator for bootstrap
int seed = 0;


// directory where the bootstrap samples are stored
static char dir_bs[SIZE];
// directory where the svm models are stored 
static char dir_models[SIZE]; 
// directory where the predictions made by the models are stored 
static char dir_res[SIZE];  

 
static void print_info_use(void);

int doarg(char* arg[], int i);

////////////////////////////////////////////////////////
int main(int argc, char* argv[])
{
  
  if ((argc < 3) || ((argc%2) == 0))
	  print_info_use();
  

  strcpy(modelfile, argv[1]);
  strcpy(dataset, argv[2]);
  strcpy(dir_bs, DIRBS_LSVM);
  strcpy(dir_models, DIRMODELS_LSVM);
	strcpy(dir_res, DIR_RES_LSVM);
	
	for (int i=3; i<argc; i+=2) 
     if (doarg(argv, i))
        print_info_use();
	 
	// Trainnig and oob evaluation of the error for lobag ensembles of SVMs
	Lobag lobagsvm(modelfile, dataset, nsample, seed, dir_bs);
	lobagsvm.exhaustive_BV_decomposition(dir_models, dir_res);
	lobagsvm.Select_model();
	lobagsvm.print_oob_error();
	
	if (b_save_BVfile)
	  lobagsvm.save_BV(BVfile);
		
  return 0;
}



void print_info_use(void)
{
  cerr << "
  lobagsvm. NEURObjects 2.0 application implemeting the lobag
            algorithm for ensembles of SVMs
  Usage:    lobagsvm modelfile dataset [options]
            modelfile  file storing the parameters of the model to be evaluated
            dataset    data set used for bootstraping samples
  Options:
  -dirbs  string  directory storing the bootstrapped samples [def. tmpbs]
  -dirmod string  directory storing the models [def. tmpbs]
  -dirres string  directory storing the predictions [def. tmpbs]
  -bv     string  file  storing the bias variance decompostion for all the models
  -ns     integer number of bootstrap samples  [def. 60]
  -seed   integer seed of the random generator for bootstrapping [def. 0]  
  " << endl;
  exit(1);
}


int doarg(char* arg[], int i)
{ 	
  if (!strcmp(arg[i], "-dirbs")) 
		strcpy(dir_bs, arg[i+1]);
	else if (!strcmp(arg[i], "-dirmod")) 
		strcpy(dir_models, arg[i+1]);
  else if (!strcmp(arg[i], "-dirres")) 
		strcpy(dir_res, arg[i+1]);
	else if (!strcmp(arg[i], "-ns")) 
		nsample = atoi(arg[i+1]);
	else if (!strcmp(arg[i], "-seed")) 
		seed = atoi(arg[i+1]);
	else if (!strcmp(arg[i], "-bv")) {
    b_save_BVfile = true;
		strcpy(BVfile, arg[i+1]);}
  else 
    return 1;
  return 0;
} 
