//  "lobagsvm_multiple.cc"
// December 2002
/**
  Application  implementing the lobag algorithm with mutliple hold-out r ccross validation.\\
	SVMs are used as base learners. It provides an out-of-bag estimate of the error for lobag
	ensemble of SVMs, using the exhaustive algorithm. It provides also a multiple hold-out or cross validation
	estimate of the error for lobag ensembles.

*/

#include<stdlib.h>
#include"utils.h"
#include"multipleLobag.h"

static unsigned const SIZE = 256;
// default directory for the bootstrapped samples
static  char* const DIRSPLIT_LSVM = "dirsplit";
// default directory for the results computed by the base learners
static  char* const DIRRES_LSVM = "dirres";
// default name of the predicition files
static  char* const BASEPRED_LSVM = "predictions";

static bool b_save_BVfile = false;

// file that the describe the model to trained and tested
static char modelfile[SIZE]; 
// dataset from which samples will be bootstrapped  
static char dataset[SIZE];  
// dataset used for testing by appl_type = 3
static char testset[SIZE];  
// basename for the prediction files
static char basepred[SIZE];  
// type of the base learner: it is used only as prefix for oob_error_per_dataset and test_error_per_dataset
static char type[SIZE];  
// file of the estimated oob error for each data set (used if appl_type = 3)
static char oob_error_per_dataset[SIZE];
// file of the estimated test error for each data set (used if appl_type = 3)
static char test_error_per_dataset[SIZE];
// file of the bias--variance values computed for each model
static char BVfile[SIZE];
// number of bootstrapped samples
int nbootstrap = 60;
// number of splitted hold-out or folds or bootstrapped training sets
int ntrain = 10;
// size of the  bootstrapped training sets (used only if appl_type = 3)
int sizebs = 10;
// seed for the random generator for bootstrap
int seed = 0;
// proportion of the data used for the training set in multiple hold-out.
float ratio = 0.7;
/* type of application:
   0: training and test data sets just stored in dirsplit
	 1: multiple hold-out
	 2: cross-validation
	 3: training on multiple bootstrapped samples and test on a single test set
*/
int appl_type = 2;

// directory where the trainnig and testing data are stored
static char dirsplit[SIZE]; 
// directory where the predictions made by the models are stored 
static char dirres[SIZE];  

 
static void print_info_use(void);
static void make_per_data_set_names(void);
static int doarg(char* arg[], int i);

////////////////////////////////////////////////////////
int main(int argc, char* argv[])
{
  
  MultipleLobag * mlobag;
	MultipleLobagHoldOut * mlobaghold;
	MultipleLobagCrossValidation * mlobagcv;
	MultipleLobagBootstrap * mlobagboot;
	
	if ((argc < 3) || ((argc%2) == 0))
	  print_info_use();
  

  strcpy(modelfile, argv[1]);
  strcpy(dataset, argv[2]);
	strcpy(testset, dataset);
	strcpy(basepred, BASEPRED_LSVM);
	strcpy(type, "");
  strcpy(dirsplit, DIRSPLIT_LSVM);
	strcpy(dirres, DIRRES_LSVM);
	
	for (int i=3; i<argc; i+=2) 
     if (doarg(argv, i))
        print_info_use();
	 
	switch (appl_type) {
	  case 0:  mlobag = new MultipleLobag(modelfile, dataset, nbootstrap, ntrain, seed, dirsplit, dirres);
		        mlobag->Multiple_test(basepred, seed); 
						mlobag->print_oob_error(cout);
	          mlobag->print_multiple_test_error(cout);
						if (b_save_BVfile)
	            mlobag->save_BV(BVfile);
						delete mlobag;			   
		        break;
		case 1:  mlobaghold = new MultipleLobagHoldOut(modelfile, dataset, nbootstrap, ntrain, seed, dirsplit, dirres);
		        mlobaghold->Multiple_hold_out(ntrain, ratio, dirsplit, basepred, seed); 
						mlobaghold->print_oob_error(cout);
	          mlobaghold->print_multiple_test_error(cout);
						if (b_save_BVfile)
	            mlobaghold->save_BV(BVfile);		 		        
						delete mlobaghold;
						break;
		case 2:  mlobagcv = new MultipleLobagCrossValidation(modelfile, dataset, nbootstrap, ntrain, seed, dirsplit, dirres);
		        mlobagcv->Cross_validation (ntrain, dirsplit, basepred, seed); 
						mlobagcv->print_oob_error(cout);
	          mlobagcv->print_multiple_test_error(cout);
						if (b_save_BVfile)
	            mlobagcv->save_BV(BVfile);		  		       
						delete mlobagcv;		
	  case 3:  mlobagboot = new MultipleLobagBootstrap(modelfile, dataset, testset, nbootstrap, ntrain, sizebs, seed, dirsplit, dirres);		
						mlobagboot->Multiple_bootstrap (dirsplit, basepred, seed); 
						mlobagboot->print_oob_error(cout);
	          mlobagboot->print_multiple_test_error(cout);
						make_per_data_set_names();
						mlobagboot->save_oob_error_per_dataset(oob_error_per_dataset);
						mlobagboot->save_test_error_per_dataset(test_error_per_dataset);						
						if (b_save_BVfile)
	            mlobagboot->save_BV(BVfile);		  		       
						delete mlobagboot;		
						break;
    default:  error_exit("multiple-lobag: application not supported"); 
	}
	
  return 0;
}


void make_per_data_set_names(void) {
  char basetest [256];
	strcpy(basetest, testset);
	delete_dir(basetest);
  strcpy(oob_error_per_dataset, dirres);
	strcat(oob_error_per_dataset, "/");
	strcat(oob_error_per_dataset, basetest);
	strcat(oob_error_per_dataset, type);
	strcat(oob_error_per_dataset, ".oob_error_per_dataset");
	strcpy(test_error_per_dataset, dirres);
	strcat(test_error_per_dataset, "/");
	strcat(test_error_per_dataset, basetest);
	strcat(test_error_per_dataset, type);
	strcat(test_error_per_dataset, ".test_error_per_dataset");
}


void print_info_use(void)
{
  cerr << "
  lobagsvm-multiple. NEURObjects 2.0 application implementing the lobag
            algorithm for ensembles of SVMs with multiple hold-out or cross validation
  Usage:    lobagsvm-multiple modelfile dataset [options]
            modelfile  file storing the parameters of the model to be evaluated
            dataset    learning set
  Options:
  -t        type of application: [def. 2]
            0: multiple training and test data sets stored in dirsplit
            1: multiple hold-out
            2: cross-validation
            3: training on bootstrapped samples and test on a single test set
  -dirsplit string  directory storing the train and test data [def. dirsplit]
  -dirres   string  directory storing the predictions [def. dirres]
  -bv       string  file  storing the bias variance decompostion for all the models
  -n        integer number of multiple hold-out or folds or bootstrapped training sets [def. 10]
  -ratio    float    proportion of the data used in the training set for hold-out [def. 0.7]
  -nbs      integer number of bootstrap samples  [def. 60]
  -seed     integer seed of the random generator [def. 0] 
  -test     string  test set (used if -t 3 is selected) [def. dataset] 
  -basepred string basename of the prdiction files [def. dataset] 
  -sizebs   integer size of the train sets bootstrapped from dataset 
            (used if -t 3 is selected) [def. 10] 
  " << endl;
  exit(1);
}


int doarg(char* arg[], int i)
{ 	
  if (!strcmp(arg[i], "-dirsplit")) 
		strcpy(dirsplit, arg[i+1]);
  else if (!strcmp(arg[i], "-dirres")) 
		strcpy(dirres, arg[i+1]);
	else if (!strcmp(arg[i], "-nbs")) 
		nbootstrap = atoi(arg[i+1]);
	else if (!strcmp(arg[i], "-n")) 
		ntrain = atoi(arg[i+1]);
	else if (!strcmp(arg[i], "-t")) 
		appl_type = atoi(arg[i+1]);
	else if (!strcmp(arg[i], "-test")) 
		strcpy(testset, arg[i+1]);
	else if (!strcmp(arg[i], "-basepred")) 
		strcpy(basepred, arg[i+1]);
	else if (!strcmp(arg[i], "-type")) 
		strcpy(type, arg[i+1]);
	else if (!strcmp(arg[i], "-ratio")) 
		ratio = atof(arg[i+1]);
	else if (!strcmp(arg[i], "-seed")) 
		seed = atoi(arg[i+1]);
	else if (!strcmp(arg[i], "-sizebs")) 
		sizebs = atoi(arg[i+1]);
	else if (!strcmp(arg[i], "-bv")) {
    b_save_BVfile = true;
		strcpy(BVfile, arg[i+1]);}
  else 
    return 1;
  return 0;
} 
