//  "make_rs_data.cc"
// May 2004
/*
  Application to generate random subspace data.\\
	
*/

#include<stdlib.h>
#include<time.h>
#include <sys/times.h>
#include"utils.h"
#include"basesvml.h"
#include"tset.h"
#include"matrix.h"
#include <unistd.h>

static unsigned const SIZE = 256;
static unsigned const DEF_DIM_INPUT = 3;
static unsigned const DEF_SUB_DIM = 2;
static unsigned const DEF_NPATTERN = 10;
static unsigned const NO = 1;
static unsigned const light = 2;
static unsigned const DEF_DATAFORMAT = NO;
static unsigned const WITH_INDEX = 2;
static unsigned const NO_INDEX = 1;
static unsigned const DEF_NFILES = 10;

static  unsigned dim_input = DEF_DIM_INPUT;
static  unsigned npattern = DEF_NPATTERN;	
static  unsigned  sub_dim = DEF_SUB_DIM;	
static  unsigned  nfiles = DEF_NFILES;	
static 	unsigned  dataformat = DEF_DATAFORMAT;
static char datafile[SIZE]; 
static unsigned sindex = NO_INDEX; 



static void print_info_use(void);

static int doarg(char* arg[], int i);

////////////////////////////////////////////////////////
int main(int argc, char* argv[])
{
  
	char filetarget[SIZE];
	TrainingSet::typeformat tformat;
		
  if (argc < 2) 
     print_info_use();
  if ((argc%2) == 1)
     print_info_use();
		 
	strcpy(datafile, argv[1]);
	
	for (int i=2; i<argc; i+=2) 
     if (doarg(argv, i))
        print_info_use();
	
  // print general info to stdout
	cout << "Random subspace data generation --" << endl;	
  cout << "Number of patterns: " << npattern << endl;
	cout << "Input dimension: " << dim_input << endl;
	cout << "Subspace dimension: " << sub_dim << endl;
  cout << "Number of files to be generated: " << nfiles << endl;
	switch (dataformat) {
	  case NO: cout << "NEURObjects data format" << endl;
		         tformat = TrainingSet::NO;
		         break;
		case light: cout << "SVMlight data format" << endl;
		            tformat = TrainingSet::light;
						    break;
    default:  cout << "Not supported data type" << endl;
	            exit(1); 
	}
  
	
	TrainingSet tset(dim_input, npattern,  datafile);
	if (sindex == NO_INDEX) 
	  tset.generate_multiple_random_subspace(datafile, nfiles, sub_dim, tformat);
	else
	  tset.generate_predefined_multiple_random_subspace(datafile, tformat);
	// saving the file with the labels of the class
	
	strcat(datafile, ".labels");
	tset.save_target(datafile);
  return 0;
}



void print_info_use(void)
{
  cerr << "
  make_rs_data. NEURObjects 2.0 application for generating
          random subspace data.
  Usage: make_rs_data datafile [options]
  datafile is the base data set from which the random subspace data are generated.
  Options:
  -dim unsigned       dimension of the data (def.3)
  -npattern unsigned  cardinality of the input data (def.10)
  -subdim unsigned    dimension of the subspace (def.2)
  -nfiles unsigned    number of random subspace data file (def.10)
  -format unsigned    format of the random subspace data to be generated:
                      1--> NEURObjects (def.); 2 --> SVMlight
  -index unsigned     if index is set to 1 (def.), the indices of the dimensions are randomly generated, 
                      otherwise they are read from the file datafile.index
  " << endl;
  exit(1);
}


int doarg(char* arg[], int i)
{
  int j;
	if (!strcmp(arg[i], "-index"))
    sindex = atoi(arg[i+1]);
	else if (!strcmp(arg[i], "-dim"))
    dim_input = atoi(arg[i+1]);
  else if (!strcmp(arg[i], "-npattern")) 
    npattern = atoi(arg[i+1]);
	else if (!strcmp(arg[i], "-subdim"))
    sub_dim = atoi(arg[i+1]);
	else if (!strcmp(arg[i], "-format")) 
    dataformat = atoi(arg[i+1]);
	else if (!strcmp(arg[i], "-nfiles")) 
    nfiles = atoi(arg[i+1]);
  else 	
    return 1;
  return 0;
} 
