/*******************************************************************************
* matrix.cc
*******************************************************************************/

#include "matrix.h"
#include <math.h>
#include "utils.h"

matrix::matrix(int dim1, int dim2, double val)  
{
	if (dim1<0 || dim2<0) 
  error_exit("matrix: negative dimension."); 

  d1=dim1; 
  d2=dim2; 

  if (d1 > 0) 
  { m = new vect*[d1];
    for (int i=0;i<d1;i++) m[i] = new vect(d2); 
  }
  else m = 0;
	if ((m!=0) && (val!=0)){
	for (int i=0; i<d1; i++)
	  for (int j=0; j<d2; j++)
		  m[i]->v[j] = val;
	}
}


matrix::matrix(const matrix& p)  
{ 
  d1 = p.d1;
  d2 = p.d2;
    
  if (d1 > 0) 
  { m = new vect*[d1];
    for (int i=0;i<d1;i++) m[i] = new vect(*p.m[i]); 
  }
  else m = 0;
}

matrix::matrix(int dim1, int dim2, double** p)  
{ d1=dim1; 
  d2=dim2; 
  m = new vect*[d1];
  for(int i=0;i<d1;i++) 
  { m[i] = new vect(d2); 
    for(int j=0;j<d2;j++) el(i,j) = p[i][j];
   }
}

matrix::~matrix()  
{ if (m) 
  { while(d1--) delete m[d1]; 
    delete[] m;
   }
}


void matrix::check_dimensions(const matrix& mat) const
{ if (d1 != mat.d1 || d2 != mat.d2)
   error_exit("Matrix dimensions do not match");
}

matrix::matrix(const vect& vec)
{ d1 = vec.d;
  d2 = 1;
  m = new vect*[d1];
  for(int i=0; i<d1; i++)
  { m[i] = new vect(1);
    el(i,0) = vec[i];
   }
    
}

matrix& matrix::operator=(const matrix& mat)
{ 
  register int i,j;
  if (d1 != mat.d1 || d2 != mat.d2){
	  for(i=0;i<d1;i++) delete m[i];
    delete[] m;
    d1 = mat.d1;
    d2 = mat.d2;
    m = new vect*[d1];
    for(i=0;i<d1;i++) m[i] = new vect(d2);
   }

  for(i=0;i<d1;i++)
    for(j=0;j<d2;j++) el(i,j) = mat.el(i,j);

  return *this;
}

int matrix::operator==(const matrix& x) const
{ register int i,j;
  if (d1 != x.d1 || d2 != x.d2) return false;

  for(i=0;i<d1;i++)
    for(j=0;j<d2;j++)
      if (el(i,j) != x.el(i,j)) return false;

  return true;
 }


vect& matrix::row(int i) const
{ if ( i<0 || i>=d1 )  error_exit("matrix: row index out of range");
   return *m[i];
}


double& matrix::operator()(int i, int j)
{ if ( i<0 || i>=d1 )  error_exit("matrix: row index out of range");
  if ( j<0 || j>=d2 )  error_exit("matrix: col index out of range");
  return el(i,j);
}

double matrix::operator()(int i, int j) const
{ if ( i<0 || i>=d1 )  error_exit("matrix: row index out of range");
  if ( j<0 || j>=d2 )  error_exit("matrix: col index out of range");
  return el(i,j);
}

vect matrix::col(int i)  const
{ if ( i<0 || i>=d2 )  error_exit("matrix: col index out of range");
  vect result(d1);
  int j = d1;
  while (j--) result.v[j] = el(j,i);
  return result;
}

matrix::operator vect() const
{ if (d2!=1) 
   error_exit("error: cannot make vector from matrix");
  return col(0);
}

matrix matrix::operator+(const matrix& mat)
{ register int i,j;
  check_dimensions(mat);
  matrix result(d1,d2);
  for(i=0;i<d1;i++)
   for(j=0;j<d2;j++)
      result.el(i,j) = el(i,j) + mat.el(i,j);
  return result;
}

matrix& matrix::operator+=(const matrix& mat) 
{ register int i,j;
  check_dimensions(mat);
  for(i=0;i<d1;i++)
   for(j=0;j<d2;j++)
      el(i,j) += mat.el(i,j);
  return *this;
}

matrix& matrix::operator-=(const matrix& mat) 
{ register int i,j;
  check_dimensions(mat);
  for(i=0;i<d1;i++)
   for(j=0;j<d2;j++)
      el(i,j) -= mat.el(i,j);
  return *this;
}


matrix matrix::operator-(const matrix& mat)
{ register int i,j;
  check_dimensions(mat);
  matrix result(d1,d2);
  for(i=0;i<d1;i++)
   for(j=0;j<d2;j++)
      result.el(i,j) = el(i,j) - mat.el(i,j);
  return result;
}


matrix matrix::operator-()  
{ register int i,j;
  matrix result(d1,d2);
  for(i=0;i<d1;i++)
   for(j=0;j<d2;j++)
      result.el(i,j) = -el(i,j);
  return result;
}


matrix matrix::operator*(double f)
{ register int i,j;
  matrix result(d1,d2);
  for(i=0;i<d1;i++)
   for(j=0;j<d2;j++)
      result.el(i,j) = el(i,j) *f;
  return result;
}

matrix matrix::operator*(const matrix& mat)
{ if (d2!=mat.d1)
     error_exit("matrix multiplication: incompatible matrix types\n");
  
  matrix result(d1, mat.d2);
  register int i,j;

  for (i=0;i<mat.d2;i++)
  for (j=0;j<d1;j++) result.el(j,i) = *m[j] * mat.col(i);

 return result;

}

matrix matrix::transpose() const
{ matrix result(d2,d1);
  for(int i = 0; i < d2; i++)
    for(int j = 0; j < d1; j++)
      result.el(i,j) = el(j,i);
  return result;
}


void matrix::read(istream& is)
{ for(int i = 0; i < d1; i++)
    for(int j = 0; j < d2; j++)
        is >> el(i,j);
 }

void matrix::print(ostream& os,  int j)
{ os << endl;
    for(int i = 0; i < d1; i++)
      m[i]->print(os, j);
	os << endl;  
}
     
void matrix:: initialize(double val) {
  for (int i=0; i<d1; i++)
	  for (int j=0; j<d2; j++)
		  m[i]->v[j] = val;
}
			
ostream& operator<<(ostream& os, const matrix& M)
{ os << M.d1 << " " << M.d2 << endl;
  for(int i = 0; i < M.d1; i++)
    os << *(M.m[i]);
	os << endl;
  return os;
}

istream& operator>>(istream& is, matrix& M)
{ int d1,d2;
  is >> d1 >> d2;
  matrix MM(d1,d2);
  for(int i = 0; i < d1; i++)
    for(int j = 0; j < d2; j++) 
       is >> MM(i,j);
  M = MM;
  return is;
}

