// multipleLobag.cc

#include<strstream.h>
#include"multipleLobag.h"
#include"utils.h"


MultipleLobag::MultipleLobag() 
{}
										 
             
MultipleLobag::MultipleLobag (char* modelfile, char* data, int nf, int m, int seed, 
                              char* dir, char* dirp) :
 base(NULL),  min_bias(), min_loss(), n(m)
{
	strcpy(learning_set,data);	
	diminput = get_diminput();
  B_min = L_min = B_L_min = 1;
	loss_single = loss_bag = loss_lobag = 0;
	stdev_single = stdev_bag = stdev_lobag = 0;
	B = nf;
	ndata = wc(data) - 2;
	dirsplit = dir;
	dirres = dirp;
  V = new deque<model_BV>[m];
	MultipleLobag::modelfile = modelfile;
	strcpy(basetrain, learning_set);
	delete_dir(basetrain);
}

MultipleLobag::~MultipleLobag() {
 if (base != NULL)
	    delete base;
 if (V != NULL)
	  delete [] V;
}


/**
  Performs multiple exhaustive evaluation of bias-variance decomposition.\\
	It computes the bias-variance decomposition of the error on different training sets, using the class
	Lobag. The results are collected in the vector of containers V.
	In the directory dir must be stored the files of the train data sets. Note that in dir are temporarily stored also
	the models and the predicitons of the SVMs. The oob estimates of the bias-variance decompostion are
	stored in the vector of container V. Each element of this vector stores the oob estimates of the
	bias-variance decomposition on a different training set.
	@param data data set
	@param dir directory where the train data are stored
	@param seed seed of the random generator
*/
void MultipleLobag::multiple_exhaustive_BV_decomp(char* data, char* dir, int seed) {

  const int SIZE = 256;
	Lobag * lobag;
	char train [SIZE];
	ostrstream otrain (train, SIZE, ios::out);
	
	for (int i=0; i<n; i++) {
		otrain.seekp(0,ios::beg);
		otrain << dir << "/" << data << i+1 << TRAIN_B << ends; 
		lobag = new Lobag(modelfile, train, B, seed, dir);
		lobag->exhaustive_BV_decomposition(dir, dir);
    V[i] = lobag->get_V();
		delete lobag;
  }
}


/** 
   Selects the model for the lobag ensemble.\\
	 It selects from the container Vavg the models with the minimum estimated bias (min_bias), and with the minimum
	 estimated loss (min_loss). It evaluates the minimum oob estimate of the error of the lobag ensemble setting
	 the member B_min that corrsponds to the bias of the model with the minimum oob estimate of the bias,
	 and the oob minimum of the error of the bagged ensemble, setting the member B_L_min that corrsponds to 
	 the bias of the model with the minimum oob estimate of the error. The member B_min is set to the oob estimate
	 of the error of the single SVM.
*/
void  MultipleLobag::Select_model(void) {
  
	typedef deque<model_BV>::iterator Itm;
  B_min = L_min = B_L_min = 1;
	compute_avg_BV();
		
	for (Itm p = Vavg.begin(); p != Vavg.end(); p++) {
	  model_BV m = *p;
		if (m.AvBV.loss < L_min) {
		  L_min = m.AvBV.loss;
			min_loss = m;		
		}
		if (m.AvBV.bias < B_min) {
		  B_min = m.AvBV.bias;
			min_bias = m;
		}		
	}
  B_L_min = min_loss.AvBV.bias;
}

/** 
   Computes the average bias-variance between multiple data sets.
	 It averages the bias-variance values stored in the vector of containers V and stores the results in the
	 container Vavg.
*/
void  MultipleLobag::compute_avg_BV(void) {
  
	AverageBV sdbv;
	model_BV mbv;
	float x;
	int nmodels = V[0].size();
	
  for (int i=0; i<nmodels; i++) {
	  mbv.P = V[0][i].P;
		mbv.AvBV.init(0);
		sdbv.init(0);	
	  for (int j=0; j<n; j++) {
		  x = V[j][i].AvBV.loss;
		  mbv.AvBV.loss += x;
			sdbv.loss += (x*x);
			x = V[j][i].AvBV.bias;
			mbv.AvBV.bias += x;
			sdbv.bias += (x*x);
			x = V[j][i].AvBV.net_variance;
			mbv.AvBV.net_variance += x;
			sdbv.net_variance += (x*x);
			x = V[j][i].AvBV.unbias_var;
			mbv.AvBV.unbias_var += x;
			sdbv.unbias_var += (x*x);
			x = V[j][i].AvBV.bias_var;
			mbv.AvBV.bias_var += x;
			sdbv.bias_var += (x*x);
			x = V[j][i].AvBV.variance;
			mbv.AvBV.variance += x;
			sdbv.variance += (x*x);
	  }
		mbv.AvBV.loss/=n;
		mbv.AvBV.bias/=n;
		mbv.AvBV.net_variance/=n;
		mbv.AvBV.unbias_var/=n;
		mbv.AvBV.bias_var/=n;
		mbv.AvBV.variance/=n;	
		Vavg.push_back(mbv);
		
		// SD calculus SD[x] = sqrt(E[x^2] - E[x]^2)
		sdbv.loss = sqrt(sdbv.loss/n - (mbv.AvBV.loss * mbv.AvBV.loss));
		sdbv.bias = sqrt(sdbv.bias/n - (mbv.AvBV.bias * mbv.AvBV.bias));
		sdbv.net_variance= sqrt(sdbv.net_variance/n - (mbv.AvBV.net_variance * mbv.AvBV.net_variance));
		sdbv.unbias_var = sqrt(sdbv.unbias_var/n - (mbv.AvBV.unbias_var * mbv.AvBV.unbias_var));
		sdbv.bias_var = sqrt(sdbv.bias_var/n - (mbv.AvBV.bias_var * mbv.AvBV.bias_var));
		sdbv.variance = sqrt(sdbv.variance/n - (mbv.AvBV.variance * mbv.AvBV.variance));
		Vsd.push_back(sdbv);		
	}
}


float  MultipleLobag::Calc_avg_loss(model_BV& mbv, float& stdev, char* predicted, int seed)   {

  const int SIZE = 256;
	float E[n];
	float Err=0;
	float x;
	Bagging * bag;
	BootstrapLight *b;
	char train [SIZE];
	char test [SIZE];
	char basebs [SIZE];
	char basemodel [SIZE];
	char pred [SIZE];
	ostrstream otrain (train, SIZE, ios::out);
	ostrstream otest (test, SIZE, ios::out);
	ostrstream obasebs(basebs,256,ios::out);
	obasebs << dirsplit << "/" << BS_NAME_M << ends;
	ostrstream obasemodel(basemodel,256,ios::out);
	obasemodel << dirsplit << "/" << MODEL_NAME_M << ends;
	ostrstream opred (pred, SIZE, ios::out);
	
	base = build_base_learner (mbv.P);
  for (int i=0; i<n; i++) {
	  bag = new Bagging(base);		
	  // preparing bootstrapped data for bagging
	  otrain.seekp(0,ios::beg);
		otrain << dirsplit << "/" << basetrain << i+1 << TRAIN_B << ends; 	  
		int ndatatrain = wc(train) - 2;
		b = new BootstrapLight (train, basebs, B, ndatatrain, seed);
	  //training the bagged ensemble
	  bag->train_base_model (basebs, basemodel, 1, B, false, false);
		// testing the bagged ensemble
		otest.seekp(0,ios::beg);
		otest << dirsplit << "/" << basetrain << i+1 << TEST_B << ends; 
		opred.seekp(0,ios::beg);
		opred << dirres << "/" << predicted << i+1 << ends;
		E[i] = bag->combine_models_step (test, pred, 1, B, false, false, false);	  
	  delete b;
	  delete bag;
	}
  // computing avg error and stdev
	for (int i=0; i<n; i++)
	  Err += E[i];
	Err/=n;
	stdev = 0;
	for (int i=0; i<n; i++) {
	  x = E[i] - Err;
		stdev += (x*x);
	}
	stdev = sqrt(stdev/(n-1));
	return Err;
}


float  MultipleLobag::Calc_avg_loss_base_learner (model_BV& mbv, float& stdev, char* predicted)  {

  const int SIZE = 256;
	float E[n];
	float Err=0;
	float x;
	char train [SIZE];
	char test [SIZE];
	char basemodel [SIZE];
	char pred [SIZE];
	ostrstream otrain (train, SIZE, ios::out);
	ostrstream otest (test, SIZE, ios::out);
	ostrstream obasemodel(basemodel,256,ios::out);
	obasemodel << dirsplit << "/" << MODEL_NAME_M << ends;
	ostrstream opred (pred, SIZE, ios::out);
	
	base = build_base_learner (mbv.P);
  for (int i=0; i<n; i++) {		
	  // preparing names of the training files
	  otrain.seekp(0,ios::beg);
		otrain << dirsplit << "/" << basetrain << i+1 << TRAIN_B << ends; 	  
	  //training the SVM		
	  base->train_learner (train, basemodel);
		// testing the SVM		
		otest.seekp(0,ios::beg);
		otest << dirsplit << "/" << basetrain << i+1 << TEST_B << ends; 
		opred.seekp(0,ios::beg);
		opred << dirres << "/" << predicted << i+1 << ends;  
		E[i] = base->test_learner(test, basemodel, pred);
	}
  // computing avg error and stdev
	for (int i=0; i<n; i++)
	  Err += E[i];
	Err/=n;
	stdev = 0;
	for (int i=0; i<n; i++) {
	  x = E[i] - Err;
		stdev += (x*x);
	}
	stdev = sqrt(stdev/(n-1));
	return Err;
}




BaseSVM_light_common* MultipleLobag::build_base_learner (par_model pm) {
    
		 float C, degree, sigma, s;
		// preparing the base learner
		if (base != NULL) {
	    delete base; base=NULL;}
		C = pm.C;
	  switch (pm.type) {
	  case 0:  // linear kernel						 
		        base = new BaseSVM_light_linear (C);
		        break;
		case 1:  // poly kernel
						s = 1/(float)diminput;
						degree = pm.degree;						
		        base = new BaseSVM_light_poly ((int)degree, C, s);
						break;
		case 2:  // gaussian kernel
						sigma = pm.s;
						base = new BaseSVM_light_gaussian (1.0/sigma, C);
						break;
		case 3:  // sigmoid kernel
		        s = pm.s;
						base = new BaseSVM_light_sigmoid (C, s);
						break;
    default:  cout << "Not known SVM kernel type" << endl;
	            exit(1); 
		}
		if (base == NULL)
	  	error_exit("MultipleLobag::build_base_learner", "Error creating base SVM");
		return base;
}



void  MultipleLobag::Multiple_test  (char* basepred, int seed)  {
  
  const int SIZE = 256;
	char pred[SIZE];
	ostrstream opred (pred, SIZE, ios::out);
	
	multiple_exhaustive_BV_decomp(basetrain, dirsplit, seed);
  compute_avg_BV();
  Select_model();
	opred.seekp(0,ios::beg);
	opred <<  basepred << ".pred.single" << ends;
	loss_single = Calc_avg_loss_base_learner(min_loss, stdev_single, pred);
	opred.seekp(0,ios::beg);
	opred << basepred << ".pred.bag" << ends;
  loss_bag =  Calc_avg_loss(min_loss, stdev_bag, pred, seed);
	opred.seekp(0,ios::beg);
	opred << basepred << ".pred.lobag" << ends;
	loss_lobag =  Calc_avg_loss(min_bias, stdev_lobag, pred, seed);
}

void MultipleLobag::print_oob_error(ostream& out) {
  
  const int WIDTH = 16;
	const int WIDTH_RID = 8;
  
  out.setf(ios::adjustfield, ios::left);
	// header
	out << "# " << "Out of bag estimates of the error of the lobag and bagged ensemble, and of the single SVM." <<
	endl;
	out.width(WIDTH); out << "Lobag :";
	print_oob_parameters(min_bias, out);
	out.width(WIDTH); out << "Error = "; 	
	out.width(WIDTH); out << B_min << endl; 	  
	
	out.width(WIDTH); out << "Bagging :";
	print_oob_parameters(min_loss, out);
	out.width(WIDTH); out << "Error = "; 	
	out.width(WIDTH); out << B_L_min << endl; 	  
	
	out.width(WIDTH); out << "Single SVM :";
	print_oob_parameters(min_loss, out);
	out.width(WIDTH); out << "Error = "; 	
	out.width(WIDTH); out << L_min << endl << endl; 	  
}

void MultipleLobag::print_oob_parameters(const model_BV& mbv, ostream& out) {

  const int WIDTH = 16;
	switch (mbv.P.type) {
	  case 0:  out.width(WIDTH); out << "linear, C = ";
            out.width(WIDTH); out << mbv.P.C; 	           
		        break;
		case 1:  out.width(WIDTH); out << "polynomial, C = ";
            out.width(WIDTH); out << mbv.P.C;   
		        out.width(WIDTH); out << "degree = ";
            out.width(WIDTH);  out << mbv.P.degree;
						break;
		case 2: out.width(WIDTH); out << "gaussian, C = ";
          out.width(WIDTH); out<< mbv.P.C;   
		       out.width(WIDTH); out << "sigma = ";
           out.width(WIDTH); out  << mbv.P.s;
						break;
		case 3: out.width(WIDTH); out << "sigmoid, C = ";
           out.width(WIDTH); out << mbv.P.C;   
		       out.width(WIDTH); out << "s = ";
           out.width(WIDTH); out   << mbv.P.s;
						break;
    default:  cout << "Not known SVM kernel type" << endl;
	           exit(1); 
	}
}


void MultipleLobag::print_multiple_test_error(ostream& out) {
  // header
	out << "# " << "Estimates of the error of the lobag and bagged ensemble, and of the single SVM." << endl;	
	print_test_error(out);
}

void MultipleLobag::print_test_error(ostream& out) {
  const int WIDTH = 16;
	const int WIDTH_RID = 8;
  
  out.setf(ios::adjustfield, ios::left);
  out.width(WIDTH); out << "Lobag :";
	print_oob_parameters(min_bias, out);
	out.width(WIDTH); out << "Error = "; 	
	out.width(WIDTH); out << loss_lobag; 	
	out.width(WIDTH); out << "SD = "; 	
	out.width(WIDTH); out << stdev_lobag << endl; 	  
	
	out.width(WIDTH); out << "Bagging :";
	print_oob_parameters(min_loss, out);
	out.width(WIDTH); out << "Error = "; 	
	out.width(WIDTH); out << loss_bag; 	
	out.width(WIDTH); out << "SD = "; 	
	out.width(WIDTH); out << stdev_bag << endl; 	 
	
	out.width(WIDTH); out << "Single SVM :";
	print_oob_parameters(min_loss, out);
	out.width(WIDTH); out << "Error = "; 	
	out.width(WIDTH); out << loss_single; 	
	out.width(WIDTH); out << "SD = "; 	
	out.width(WIDTH); out << stdev_single << endl; 	  
}

  
/** 
   Prints the averaged values of bias-variance decompostion for all the models.
	 It prints the values of the loss, bias, net, unbiased and biased variance and of the pure variance
	 of all the considered models, averaging on all the n training sets
	 @param out output stream where the values are printed
*/
void MultipleLobag::print_BV(ostream& out) {
  	
	typedef deque<model_BV>::iterator Itm;
	typedef deque<AverageBV>::iterator Ita;
	const int WIDTH = 16;
	const int WIDTH_RID = 8;
  
  out.setf(ios::adjustfield, ios::left);
	// header
	out << "# " << "Average BV decompostion on :" <<  endl;
	out << "# " << learning_set << endl;
	
	int svm_type =	Vavg[0].P.type;
	switch (svm_type) {
	  case 0:  out.width(WIDTH_RID); out << "# C";    
		        break;
		case 1:  out.width(WIDTH_RID); out << "# C";    
		        out.width(WIDTH_RID); out << "degree";
						break;
		case 2: out.width(WIDTH_RID); out << "# C";    
		       out.width(WIDTH_RID); out << "sigma";
						break;
		case 3: out.width(WIDTH_RID); out << "# C";    
		       out.width(WIDTH_RID); out << "s";
						break;
    default:  cout << "MultipleLobag::print_BV: Not known SVM kernel type" << endl;
	           exit(1); 
	}
	out.width(WIDTH); out << "Loss";
	out.width(WIDTH); out << "Bias";
	out.width(WIDTH); out << "Net-var";
	out.width(WIDTH); out << "Unb-var";
	out.width(WIDTH); out << "Bias-var";
	out.width(WIDTH); out << "Variance" << endl << endl;
  
	// BV values and their SD
	Ita sd = Vsd.begin();
	for (Itm p = Vavg.begin(); p != Vavg.end(); p++) {
		(*p).print(out);
		out.width(WIDTH); out << " ";
		(*sd).print(out);
		sd++;
	}
}	


/** 
   Save the values of bias-variance decompostion for all the models.
	 It saves the values of the loss, bias, net, unbiased and biased variance and of the pure variance
	 of all the considered models.
	 @param file file where the values will be saved
*/
void MultipleLobag::save_BV(char* file) {
  
	ofstream ofs(file);
	if (!ofs)
	  error_exit("Cannot open ", file);
	print_BV(ofs);
	if (ofs.bad())
	  error_exit("Error writing ", file);
}

int MultipleLobag::get_diminput(void) {
  
	int dim;
	string line;
	ifstream ifs(learning_set);
	if (!ifs)
	  error_exit("Cannot open ", learning_set);
	getline(ifs, line); getline(ifs, line); getline(ifs, line);
  dim = wc_word(line) - 1;
	if (ifs.bad())
	  error_exit("Error reading ", learning_set);
	return dim;
}



//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////


void MultipleLobagHoldOut::Split(int nsplits, float ratio, char* dir, int seed) {
  
	char samples [256];
	ostrstream osamples (samples, 256, ios::out);
	osamples << dir << "/" << basetrain << ends; 
  BootstrapLight b(learning_set, seed);	
	b.write_splitted_samples(samples, nsplits, ratio);
}


void  MultipleLobagHoldOut::Multiple_hold_out(int nsplits, float ratio, char* dir, char* basepred, int seed)  {

  Split(nsplits, ratio, dir, seed);
	Multiple_test(basepred, seed);
}

void MultipleLobagHoldOut::print_multiple_test_error(ostream& out) {
  // header
	out << "# " << "Multiple hold out estimates of the error of the lobag and bagged ensemble, and of the single SVM." << endl;	
	print_test_error(out);
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////


void MultipleLobagCrossValidation::Generate_Folds(int nfolds, char* dir, int seed) {
  
	char samples [256];
	ostrstream osamples (samples, 256, ios::out);
	osamples << dir << "/" << basetrain << ends; 
  BootstrapLight b(learning_set, seed);
	b.write_folded_samples(samples, nfolds);
}

void  MultipleLobagCrossValidation::Cross_validation  (int nfolds,  char* dir, char* basepred, int seed)  {

  Generate_Folds(nfolds, dir, seed);
	Multiple_test(basepred, seed);
}

void MultipleLobagCrossValidation::print_multiple_test_error(ostream& out) {
  // header
	out << "# " << "Cross validated estimates of the error of the lobag and bagged ensemble, and of the single SVM." << endl;	
	print_test_error(out);
}

/*
 *********************************************************************************
 *********************************************************************************
*/

MultipleLobagBootstrap::MultipleLobagBootstrap (char* modelfile, char* data, char* test, int nf, int m, 
                      int size, int seed, char* dir, char* dirp) : 
                      MultipleLobag(modelfile, data, nf, m, seed, dir, dirp), bs_size(size) {
  avg_oob_loss_lobag = avg_oob_loss_bag = avg_oob_loss_single = 0;
  stdev_avg_oob_loss_lobag = stdev_avg_oob_loss_bag = stdev_avg_oob_loss_single = 0;  
	testset = test;
	strcpy(basetest, testset);
	delete_dir(basetest);              
}


void  MultipleLobagBootstrap::Multiple_bootstrap  (char* dir, char* basepred, int seed)  {

  Bootstrap_samples(n, bs_size, dir, seed);
	Multiple_test(basepred, seed);
}

void  MultipleLobagBootstrap::Bootstrap_samples(int nsample, int size, char* dir, int seed){
  
  char samples [256];
	ostrstream osamples (samples, 256, ios::out);
	osamples << dir << "/" << basetrain << ends; 
  BootstrapLight b(learning_set, samples, nsample, size, seed);
}

void  MultipleLobagBootstrap::Multiple_test  (char* basepred, int seed)  {
  
  const int SIZE = 256;
	char pred[SIZE];
	ostrstream opred (pred, SIZE, ios::out);
	
	multiple_exhaustive_BV_decomp(basetrain, dirsplit, seed);
  Select_model();
	opred.seekp(0,ios::beg);
	opred <<  basepred << ".pred.single" << ends;
	loss_single = Calc_avg_loss_base_learner_bootstrap(min_loss_models, stdev_single, pred, errors_single);
	opred.seekp(0,ios::beg);
	opred << basepred << ".pred.bag" << ends;
  loss_bag =  Calc_avg_loss_bootstrap(min_loss_models, stdev_bag, pred, errors_bag, seed);
	opred.seekp(0,ios::beg);
	opred << basepred << ".pred.lobag" << ends;
	loss_lobag = Calc_avg_loss_bootstrap(min_bias_models, stdev_lobag, pred, errors_lobag, seed);
}


void MultipleLobagBootstrap::multiple_exhaustive_BV_decomp(char* data, char* dir, int seed) {

  const int SIZE = 256;
	Lobag * lobag;
	char train [SIZE];
	ostrstream otrain (train, SIZE, ios::out);
	
	for (int i=0; i<n; i++) {
		otrain.seekp(0,ios::beg);
		otrain << dir << "/" << data << i+1 << ends; 
		lobag = new Lobag(modelfile, train, B, seed, dir);
		lobag->exhaustive_BV_decomposition(dir, dir);
    V[i] = lobag->get_V();
		delete lobag;
  }
}


void  MultipleLobagBootstrap::Select_model(void) {
 
	float bmin, lmin, x;
  typedef deque<model_BV>::iterator Itm;
  
	
  for (int i=0; i<n; i++) {	
    bmin = lmin = 1;
	  for (Itm p = V[i].begin(); p != V[i].end(); p++) {
	    model_BV m = *p;
	    if (m.AvBV.loss < lmin) {
	      lmin = m.AvBV.loss;
	      min_loss = m;   
	    }
	    if (m.AvBV.bias < bmin) {
	      bmin = m.AvBV.bias;
	      min_bias = m;
	    }   
	  }
    min_loss_models.push_back(min_loss);
    min_bias_models.push_back(min_bias);
  }
  // calc average and stdev,
  avg_oob_loss_lobag = avg_oob_loss_bag = avg_oob_loss_single = 0;
  for (int i=0; i<n; i++) {	
    avg_oob_loss_bag    += min_loss_models[i].AvBV.bias;
    avg_oob_loss_single += min_loss_models[i].AvBV.loss;
    avg_oob_loss_lobag  += min_bias_models[i].AvBV.bias;
  }
  avg_oob_loss_bag /= n;
  avg_oob_loss_single /= n;
  avg_oob_loss_lobag /= n; 
  stdev_avg_oob_loss_lobag = stdev_avg_oob_loss_bag = stdev_avg_oob_loss_single = 0;
  for (int i=0; i<n; i++) {
	  x = min_loss_models[i].AvBV.bias - avg_oob_loss_bag;
		stdev_avg_oob_loss_bag += (x*x);
    x = min_loss_models[i].AvBV.loss - avg_oob_loss_single;
		stdev_avg_oob_loss_single += (x*x);
    x = min_bias_models[i].AvBV.bias - avg_oob_loss_lobag;
		stdev_avg_oob_loss_lobag += (x*x);
	}
	stdev_avg_oob_loss_bag = sqrt(stdev_avg_oob_loss_bag/(n-1));
  stdev_avg_oob_loss_single = sqrt(stdev_avg_oob_loss_single/(n-1));
  stdev_avg_oob_loss_lobag = sqrt(stdev_avg_oob_loss_lobag/(n-1));
  
}



void MultipleLobagBootstrap::print_oob_error(ostream& out) {
  
  const int WIDTH = 16;
	const int WIDTH_RID = 8;
  
  out.setf(ios::adjustfield, ios::left);
	// header
	out << "# " << "Out of bag estimates of the average error of the lobag and bagged ensemble, and of the single SVM." <<
	endl;
	out.width(WIDTH); out << "Lobag :";
	//print_oob_parameters(min_bias, out);
	out.width(WIDTH); out << "Error = "; 	
	out.width(WIDTH); out << avg_oob_loss_lobag;
  out.width(WIDTH); out << "SD = "; 	
	out.width(WIDTH); out << stdev_avg_oob_loss_lobag << endl; 	  
	
	out.width(WIDTH); out << "Bagging :";
	//print_oob_parameters(min_loss, out);
	out.width(WIDTH); out << "Error = "; 	
	out.width(WIDTH); out << avg_oob_loss_bag;
  out.width(WIDTH); out << "SD = "; 	
	out.width(WIDTH); out << stdev_avg_oob_loss_bag << endl; 	  
	
	out.width(WIDTH); out << "Single SVM :";
	//print_oob_parameters(min_loss, out);
	out.width(WIDTH); out << "Error = "; 	
	out.width(WIDTH); out << avg_oob_loss_single;
  out.width(WIDTH); out << "SD = "; 	
	out.width(WIDTH); out << stdev_avg_oob_loss_single << endl;
}
  

void MultipleLobagBootstrap::print_oob_error_per_dataset(ostream& out) {
  
  const int WIDTH = 16;
	const int WIDTH_RID = 10;
  
  out.setf(ios::adjustfield, ios::left);
  out.precision(9);
	// header
	out << "# " << "Out of bag estimates of the error of the lobag and bagged ensemble, and of the single SVM." <<
	endl;
  out << "# " << "Estimate on each separated learning set." << endl;
  out.width(WIDTH_RID); out << "# Data set";
	out.width(WIDTH); out << "Lobag";
	out.width(WIDTH); out << "Bag";
	out.width(WIDTH); out << "Single";
	out.width(WIDTH*2); out << "Parameters for Lobag, Bag and Single" << endl;
  
	for (int i=0; i<n; i++) {
	  out.width(WIDTH_RID); out << i+1;
	  out.width(WIDTH); out << min_bias_models[i].AvBV.bias;
	  out.width(WIDTH); out << min_loss_models[i].AvBV.bias;
	  out.width(WIDTH); out << min_loss_models[i].AvBV.loss;	
	  min_bias_models[i].P.print(out);
    min_loss_models[i].P.print(out);
    out << endl;
	}
}


/**
	 Calculates the loss of multiple bagged ensemble on a single test set .\\
	 It calculates the average loss of the bagged ensemble obtained from differet bootstrapped training set on
   a single test set. The bagged ensemble are trained using the parameters stored in dm and are tested against a separated test set. 
   The predictions made by the ensembles are stored in dirres/predicted1, ..., dirres/predictedn.
	 The error at each iteration is stored in dirres/predicted1.ens_errors, ..., dirres/predictedn.ens_errors.
	 @param dm container of the models to be tested
	 @param stdev standard deviation of the error (returned value)
	 @param predicted base file name of the classes predicted by the ensemble
   @param derr container where the errors of the ensembles trained on different training sets are stored
	 @param seed seed of the random generator.
   @return average loss
*/
float  MultipleLobagBootstrap::Calc_avg_loss_bootstrap
 (deque<model_BV>& dm,  float& stdev, char* predicted,  deque<float>& derr, int seed)   {

  const int SIZE = 256;
	float Err=0;
	float x;
	Bagging * bag;
	BootstrapLight *b;
	char train [SIZE];
	char basebs [SIZE];
	char basemodel [SIZE];
	char pred [SIZE];
	ostrstream otrain (train, SIZE, ios::out);
	ostrstream obasebs(basebs,256,ios::out);
	obasebs << dirsplit << "/" << BS_NAME_M << ends;
	ostrstream obasemodel(basemodel,256,ios::out);
	obasemodel << dirsplit << "/" << MODEL_NAME_M << ends;
	ostrstream opred (pred, SIZE, ios::out);
  model_BV mbv;
	
  for (int i=0; i<n; i++) {
    mbv.P = dm[i].P;
    base = build_base_learner(mbv.P);
	  bag = new Bagging(base);		
	  // preparing bootstrapped data for bagging
	  otrain.seekp(0,ios::beg);
		otrain << dirsplit << "/" << basetrain << i+1  << ends; 	  
		int ndatatrain = wc(train) - 2;
		b = new BootstrapLight (train, basebs, B, ndatatrain, seed);
	  //training the bagged ensemble
	  bag->train_base_model (basebs, basemodel, 1, B, false, false);
		// testing the bagged ensemble
		opred.seekp(0,ios::beg);
		opred << dirres << "/" << predicted << i+1 << ends;
		Err = bag->combine_models_step (testset, pred, 1, B, false, false, false);	
    derr.push_back(Err);  
	  delete b;
	  delete bag;
	}
  // computing avg error and stdev
  Err=0;
	for (int i=0; i<n; i++)
	  Err += derr[i];
	Err/=n;
	stdev = 0;
	for (int i=0; i<n; i++) {
	  x = derr[i] - Err;
		stdev += (x*x);
	}
	stdev = sqrt(stdev/(n-1));
	return Err;
}


float MultipleLobagBootstrap::Calc_avg_loss_base_learner_bootstrap
  (deque<model_BV>& dm, float& stdev, char* predicted,  deque<float>& derr) {
  
const int SIZE = 256;
	float Err=0;
	float x;
	char train [SIZE];
	char basemodel [SIZE];
	char pred [SIZE];
	ostrstream otrain (train, SIZE, ios::out);
	ostrstream obasemodel(basemodel,256,ios::out);
	obasemodel << dirsplit << "/" << MODEL_NAME_M << ends;
	ostrstream opred (pred, SIZE, ios::out);
  model_BV mbv;
	
  for (int i=0; i<n; i++) {
    mbv.P = dm[i].P;
    base = build_base_learner (mbv.P);
	  // preparing bootstrapped data for bagging
	  otrain.seekp(0,ios::beg);
		otrain << dirsplit << "/" << basetrain << i+1  << ends; 	  
	  //training the SVM	
	  base->train_learner (train, basemodel);
		// testing the bagged ensemble
		opred.seekp(0,ios::beg);
		opred << dirres << "/" << predicted << i+1 << ends;
		Err = base->test_learner(testset, basemodel, pred);
    derr.push_back(Err);  
	}
  // computing avg error and stdev
  Err=0;
	for (int i=0; i<n; i++)
	  Err += derr[i];
	Err/=n;
	stdev = 0;
	for (int i=0; i<n; i++) {
	  x = derr[i] - Err;
		stdev += (x*x);
	}
	stdev = sqrt(stdev/(n-1));
	return Err;
}  


void MultipleLobagBootstrap::print_multiple_test_error(ostream& out) {
  
  const int WIDTH = 16;
  
  out.setf(ios::adjustfield, ios::left);
	// header
	out << "# " << "Estimates of the average error of the lobag and bagged ensemble on the separated test set" 
	<< endl;
	
	out.width(WIDTH); out << "Lobag :";
	out.width(WIDTH); out << "Error = "; 	
	out.width(WIDTH); out << loss_lobag; 	
	out.width(WIDTH); out << "SD = "; 	
	out.width(WIDTH); out << stdev_lobag << endl; 	  
	
	out.width(WIDTH); out << "Bagging :";
	out.width(WIDTH); out << "Error = "; 	
	out.width(WIDTH); out << loss_bag; 	
	out.width(WIDTH); out << "SD = "; 	
	out.width(WIDTH); out << stdev_bag << endl; 	 
	
	out.width(WIDTH); out << "Single SVM :";
	out.width(WIDTH); out << "Error = "; 	
	out.width(WIDTH); out << loss_single; 	
	out.width(WIDTH); out << "SD = "; 	
	out.width(WIDTH); out << stdev_single << endl; 	  
}
  
void MultipleLobagBootstrap::print_multiple_test_error_per_dataset(ostream& out) {
  
	const int WIDTH = 16;
	const int WIDTH_RID = 10;

  out.setf(ios::adjustfield, ios::left);
  out.precision(9);
	// header
	out << "# " << "Estimates of the error on the separated test set for each data set"  << endl;
	out.width(WIDTH); out << "# Data set";
	out.width(WIDTH); out << "Lobag";
	out.width(WIDTH); out << "Bag";
	out.width(WIDTH); out << "Single";
	out.width(WIDTH*2); out << "Parameters for Lobag, Bag and Single" << endl;
	
	for (int i=0; i<n; i++) {
	  out.width(WIDTH); out << i+1;
	  out.width(WIDTH); out << errors_lobag[i];
	  out.width(WIDTH); out << errors_bag[i];
	  out.width(WIDTH); out << errors_single[i];
	  min_bias_models[i].P.print(out);
	  min_loss_models[i].P.print(out);
	  out << endl;
	}
}


void MultipleLobagBootstrap::save_test_error_per_dataset(char* file){
  ofstream ofs(file);
	if (!ofs)
	  error_exit("Cannot open ", file);
	print_multiple_test_error_per_dataset(ofs);
	if (ofs.bad())
	  error_exit("Error writing ", file);
}

void MultipleLobagBootstrap::save_oob_error_per_dataset(char* file) {
  ofstream ofs(file);
	if (!ofs)
	  error_exit("Cannot open ", file);
	print_oob_error_per_dataset(ofs);
	if (ofs.bad())
	  error_exit("Error writing ", file);
}


/** 
   Prints the averaged values of bias-variance decompostion for all the models.
	 It prints the values of the loss, bias, net, unbiased and biased variance and of the pure variance
	 of all the considered models, averaging on all the n training sets
	 @param out output stream where the values are printed
*/
void MultipleLobagBootstrap::print_BV(ostream& out) {
  	
	compute_avg_BV();
	MultipleLobag::print_BV(out);
}	
