// "net.cc"


#include<time.h>
#include<math.h>
#include<iostream.h>
#include<fstream.h>
#include<iomanip.h>
#include<string.h>
#include"net.h"
#include"utils.h"


#define READ_W      "OneLayer::read_weights :"
#define MISM_FUN    "Dimensioni output non corrispondenti"

static const unsigned DIMBUFNAMEFILE = 256;  
static const unsigned DIMBUF = 256;
OneLayerTest::OneLayerTest(unsigned nout, unsigned nin, act_func fout)
{
  layertest[OUTPUT] = LayerNet(nout, nin, fout);
  n_layers = 1;
  n_inputs = nin;
  n_outputs = nout;
  num_errors = 0; 
  perc_errors = 0;
}
vect& OneLayerTest::compute (vect& vinput)
{
  return (layertest[OUTPUT].compute (vinput));
}
void OneLayerTest::print_weights(void)
{
  cout << "Weight matrix of the one layer net :" << endl;
  cout << "Output layer :" << endl;
  layertest[OUTPUT].print_weights();
}
void OneLayerTest::init_weights_const(double x)
{
  layertest[OUTPUT].init_weights(x);
}  
void OneLayerTest::read_weights(istream& f)
{
  int j;
  for (j=0; j<n_layers; j++)
    layertest[j].load_weights(f);
}
unsigned OneLayerTest::test(TrainingSet& tset)
{
  unsigned i;
  int dim = layertest[OUTPUT].Read_noutputs();  
  unsigned num_train = tset.Ntrain();
  vect& t = tset.Read_target();  
  matrix& x = tset.Read_data();     
  num_errors = 0;
  for (i = 0; i < num_train; i++) {
    vect& input = x.row(i);
    vect& out = compute(input);
    if (maxclass(out, dim) != (int)t[i])
       { num_errors++;  cout <<" *******";} 
  }
  perc_errors = (double)num_errors/(double)num_train;
  return num_errors;
}
unsigned OneLayerTest::maxclass(vect& out, int dim)
{
  int j;
  double max = out[0];
  unsigned cl = 0;
  for (j=1; j<dim; j++)
    if (max < out[j])  {
      max = out[j]; cl = j;
    }
  return(cl+1);
} 
void OneLayerTest::print_errors()
{
  cout << "Neural net errors:" << endl;
  cout << "Number of errors  :  " << num_errors;
  cout << "  :  Percentual error rate  :  " << perc_errors*100 << " %" << endl;
}
TwoLayerTest::TwoLayerTest(unsigned nout, unsigned nhidden, unsigned nin, act_func fout, 
                                                                  act_func fhid1)
: OneLayerTest(nout, nhidden, fout)
{
  n_layers = 2;
  n_inputs = nin;
  n_hidden1 = nhidden;
  layertest[HIDDEN1] = LayerNet(nhidden, nin, fhid1);
}
vect& TwoLayerTest::compute (vect& vinput)
{
  vect& hidout = layertest[HIDDEN1].compute(vinput);  
  return (layertest[OUTPUT].compute (hidout));   
} 
void TwoLayerTest::init_weights_const(double x)
{
  layertest[OUTPUT].init_weights(x);
  layertest[HIDDEN1].init_weights(x); 
}  
void TwoLayerTest::print_weights(void)
{
  cout << "Weight matrix of the Two-Layer net :" << endl;
  cout << "Output layer :" << endl;
  layertest[OUTPUT].print_weights();
  cout << "Hidden layer :" << endl;
  layertest[HIDDEN1].print_weights();
}
ThreeLayerTest::ThreeLayerTest(unsigned nout, unsigned nhidden1, unsigned nhidden2, unsigned nin,
                       act_func fout, act_func fhid1, act_func fhid2)
: TwoLayerTest(nout, nhidden1, nhidden2, fout, fhid1)
{
  n_layers = 3;
  n_hidden2 = nhidden2;
  n_inputs = nin;
  layertest[HIDDEN2] = LayerNet(nhidden2, nin, fhid2);
}
vect& ThreeLayerTest::compute (vect& vinput)
{
  vect& hid2out = layertest[HIDDEN2].compute(vinput);  
  vect& hid1out = layertest[HIDDEN1].compute(hid2out);  
  return (layertest[OUTPUT].compute (hid1out));   
}
void ThreeLayerTest::init_weights_const(double x)
{
  int i;
  layertest[OUTPUT].init_weights(x);
  layertest[HIDDEN1].init_weights(x); 
  layertest[HIDDEN2].init_weights(x); 
} 
void ThreeLayerTest::print_weights(void)
{
  cout << "Weight matrix of the Three-Layer net :" << endl;
  cout << "Output layer :" << endl;
  layertest[OUTPUT].print_weights();
  cout << "First hidden layer :" << endl;
  layertest[HIDDEN1].print_weights();
  cout << "Second hidden layer :" << endl;
  layertest[HIDDEN2].print_weights();
}
OneLayer::OneLayer(unsigned nout, unsigned nin, act_func fout)
:  C(nout, nout), Qerror()
{
  n_layers = 1;
  n_inputs = nin;
  n_class = n_outputs = nout;
  layer[OUTPUT] = LayerNetTrain(nout, nin, fout);
  Init_C();
	minHam = 2;
	sigmoid_init();
	tanhyp_init ();
}
void OneLayer::init_weights_norm(int s)
{ layer[OUTPUT].init_norm_weights(s);  
  seed = s;
}
void OneLayer::init_weights_const(double x)
{
  layer[OUTPUT].init_weights(x);
}  
void OneLayer::read_weights(char* namefile)
{
  char namebuf[DIMBUFNAMEFILE];
  char buf[DIMBUF];
  unsigned nlayer, dimoutput, dimhidden1, dimhidden2, diminput;
  unsigned funout, funhid1, funhid2;
  unsigned bufint;
  act_func g_out, g_hid1, g_hid2;
  strcpy(namebuf, namefile);
  strcat(namebuf, NET_EXT);
  ifstream f(namebuf);
  if (!f) error_exit("Cannot open ", namebuf);
  f >> buf >> bufint;
  if (bufint != 1)
    error_exit("Version of file .net not known.");
  f >> buf;
  if (strcmp(buf, namebuf))   
    error ("OneLayer::read_weights :" , "mismatch namefile");
  f >> buf >> nlayer;
  if (nlayer != n_layers)
    error ("OneLayer::read_weights :" , "Number of layers mismatch");
  switch (nlayer)  {    
    case 1: f >> buf >> dimoutput;
            f >> buf >> funout; 
            g_out = Decode_act_fun(funout);
            if (dimoutput != layer[OUTPUT].Read_noutputs())
               error ("OneLayer::read_weights :" , "Output dimension do not match"); 
            if (g_out != layer[OUTPUT].Read_g()) 
               error ("OneLayer::read_weights :" , "Dimensioni output non corrispondenti" );                     
            break;    
    case 2: f >> buf >> dimoutput;
            f >> buf >> funout; 
            g_out = Decode_act_fun(funout); 
            if (dimoutput != layer[OUTPUT].Read_noutputs())
               error ("OneLayer::read_weights :" , "Output dimension do not match");
            if (g_out != layer[OUTPUT].Read_g()) 
               error ("OneLayer::read_weights :" , "Dimensioni output non corrispondenti" );
            f >> buf >> dimhidden1;
            f >> buf >> funhid1;
            g_hid1 = Decode_act_fun(funhid1);
            if (dimhidden1 != layer[HIDDEN1].Read_noutputs())
               error ("OneLayer::read_weights :" , "First hidden layer dimension do not match"); 
            if (g_hid1 != layer[HIDDEN1].Read_g()) 
               error ("OneLayer::read_weights :" , "Dimensioni output non corrispondenti" );
            break;     
    case 3: f >> buf >> dimoutput;
            f >> buf >> funout; 
            g_out = Decode_act_fun(funout); 
            if (dimoutput != layer[OUTPUT].Read_noutputs())
               error ("OneLayer::read_weights :" , "Output dimension do not match"); 
            if (g_out != layer[OUTPUT].Read_g()) 
               error ("OneLayer::read_weights :" , "Dimensioni output non corrispondenti" );
            f >> buf >> dimhidden1;
            f >> buf >> funhid1;
            g_hid1 = Decode_act_fun(funhid1);
            if (dimhidden1 != layer[HIDDEN1].Read_noutputs())
               error ("OneLayer::read_weights :" , "First hidden layer dimension do not match"); 
            if (g_hid1 != layer[HIDDEN1].Read_g()) 
               error ("OneLayer::read_weights :" , "Dimensioni output non corrispondenti" );
            f >> buf >> dimhidden2;
            if (dimhidden2 != layer[HIDDEN2].Read_noutputs())
               error ("OneLayer::read_weights :" , "Second hidden layer dimension do not match");
            f >> buf >> funhid2;
            g_hid2 = Decode_act_fun(funhid2);
            if (g_hid2 != layer[HIDDEN2].Read_g()) 
               error ("OneLayer::read_weights :" , "Dimensioni output non corrispondenti" );
            break;
    default: error ("Not supported number of layers."); break;
  }
  f >> buf >> diminput;
  if (diminput != layer[n_layers-1].Read_ninputs())
               error ("OneLayer::read_weights :" , "Input dimension mismatch");
  if (f.bad())
    error_exit("OneLayer::read_weights :" , "Error reading header file");
  read_weights(f); 
  eatwhite(f);  
  f >> buf;
  if (strcmp(buf, END))  
    error_exit("OneLayer::read_weights :" , "End of ifle expected ");
  if (f.bad())
    error_exit("OneLayer::read_weights :" , "Error reading file ");
}
void OneLayer::read_weights(istream& f)
{
  int j;
  for (j=0; j<n_layers; j++)
    layer[j].load_weights(f);
}
vect& OneLayer::compute (vect& vinput)
{
  return (layer[OUTPUT].compute (vinput));
}
void OneLayer::print_weights(void)
{
  cout << "Weight matrix of the One-Layer net :" << endl;
  cout << "Output layer :" << endl;
  layer[OUTPUT].print_weights();
}
void OneLayer::backprop(vect& t, vect& x)
{
  layer[OUTPUT].calc_grad(t, x);  
}
void OneLayer::Init_C(void)
{
  double on, off;
  int i, j;
  act_func g = layer[OUTPUT].Read_g();
  analyze (g, on, off);
  for (i=0; i<n_class; i++)
    for (j=0; j<n_outputs; j++)
      C(i,j) = off;
  for (i=0; i<n_outputs; i++)
     C(i,i) = on;       
}
void OneLayer::analyze (act_func g, double& on, double& off)
{
  if (g==sigmoid) 
    { on = SIG_ON; off = SIG_OFF;} 
  else if (g==tanhyp) 
    { on = TANH_ON; off = TANH_OFF;} 
  else if (g==NO_identity)  
    { on = ID_ON; off = ID_OFF;} 
  else
    error ("Not known activation function");
}
void OneLayer::print_C(void)
{
  cout << "Decomposition matrix coding classes : " << endl;
  C.print();
}
void OneLayer::print_info_grad(void)
{
  cout << "One-Layer net : Delta and gradient values. " << endl;
  layer[OUTPUT].print_delta();
  layer[OUTPUT].print_grad();
  layer[OUTPUT].print_grad_bias();
} 
void OneLayer::save_weights_heading(ofstream& f, char* namebuf)
{
  unsigned fun;
  fun = Code_act_fun(layer[OUTPUT].Read_g());
  f.setf(ios::left, ios::adjustfield);
  f << "Version:  " << 1 << endl;
  f << namebuf << endl;
  f.width(INTEST_SIZE);
  f << "Number_layers: " << 1 << endl;
  f.width(INTEST_SIZE);
  f << "Dim_output: " << n_outputs << "    ";
  f << "Act.function: " << fun << endl;
  f.width(INTEST_SIZE);
  f << "Dim_input: " << n_inputs << endl << endl;
	f << flush;
}
void OneLayer::save_weights_heading(char* namefile)
{
  char namebuf[DIMBUFNAMEFILE];
  strcpy(namebuf, namefile);
  strcat(namebuf, NET_EXT);
  ofstream f(namebuf);
  if (!f) error("Cannot open file ", namebuf);
  save_weights_heading(f, namebuf);
}
void OneLayer::save_weights_body(ofstream& f)
{
	f << "Weight matrix first layer :" << endl;
  layer[OUTPUT].save_weights(f);
  f << END << endl << flush;  
}
void OneLayer::save_weights_body(char* namefile)
{
  char namebuf[DIMBUFNAMEFILE];
	
	strcpy(namebuf, namefile);
	strcat(namebuf, NET_EXT);
	ofstream f(namebuf,ios::out|ios::app);
	save_weights_body(f);
  if (f.bad())
    error("Error writing file ", namebuf);
}
void OneLayer::save_weights(char* namefile)
{
  save_weights_heading(namefile);
	save_weights_body(namefile);
}
unsigned OneLayer::Code_act_fun(act_func g)
{
  unsigned fun;
  if (g==sigmoid) 
    fun = SIGMOID; 
  else if (g==tanhyp) 
    fun = TANHYP;
  else if (g==NO_identity)  
    fun = IDENTITY; 
  else
    error ("Not known activation function");
  return fun;
}
act_func OneLayer::Decode_act_fun(unsigned fun)
{
  act_func g;
  switch (fun)  {
    case SIGMOID:
        g = sigmoid; break;  
    case TANHYP:
        g = tanhyp; break; 
    case IDENTITY:
        g = NO_identity; break; 
  default:
    error ("Not known activation function");
  }
  return g;
}
stop_cond OneLayer::Learn_by_pattern(TrainingSet& t_set, Learning& learn,
                                          unsigned& iter, double& error,
                                          double errtol, unsigned maxits, bool printout)
{
  unsigned i;
  unsigned num_train = t_set.Ntrain();
  vect& t = t_set.Read_target();  
  matrix& x = t_set.Read_data();    
  double denom = calc_denom_RMS(t, num_train);  
  unsigned typefun = Code_act_fun(layer[OUTPUT].Read_g()); 
  Zero(n_layers);  
	
  for (iter = 0; iter < maxits; iter++) {
    error = 0;
    for (i = 0; i < num_train; i++) {
      vect target = C.col(((int)t[i])-1);
      vect& input = x.row(i);
      vect& out = compute(input);
      backprop(target, input);  
      learn.Update_delta_weights(n_layers, layer);
      learn.Update_weigths (n_layers, layer);      
      error += Calc_square_error(target, out);
    }
		 
    error = RMS_error(error, denom);   
    Qerror.push((float)error);
    if (printout) cout << "." << flush;
    if (error <= errtol)
      { if (printout) cout << endl;
			  return STOP_ERR_TOL;}
    learn.Update_param(error, iter);  
  }
  if (printout) cout << endl;
  return STOP_NUM_ITERAZ; 
}
stop_cond OneLayer::Learn_by_epoch(TrainingSet& t_set, Learning& learn,
                                          unsigned& iter, double& error,
                                          double errtol, unsigned maxits, bool printout)
{
  unsigned i;
  matrix*  Sum_Delta_W[MAXLAYERS];   
  vect*  Sum_delta_bias[MAXLAYERS];   
  unsigned num_train = t_set.Ntrain();
  vect& t = t_set.Read_target();  
  matrix& x = t_set.Read_data();    
  double denom = calc_denom_RMS(t, num_train);  
  unsigned typefun = Code_act_fun(layer[OUTPUT].Read_g()); 
  Build_Sum_Delta(Sum_Delta_W, Sum_delta_bias);
  Zero(n_layers);  
  do_Sum_deltas(n_layers,Sum_Delta_W,Sum_delta_bias);
  for (iter = 0; iter < maxits; iter++) {
    error = 0;
    Zero_Sum_Delta(n_layers, Sum_Delta_W, Sum_delta_bias);  
    for (i = 0; i < num_train; i++) {
      vect target = C.col(((int)t[i])-1);
      vect& input = x.row(i);
      vect& out = compute(input);
      backprop(target, input);  
      learn.Update_delta_weights(n_layers, layer);
      Update_Sum_Delta(n_layers, Sum_Delta_W, Sum_delta_bias); 
      error += Calc_square_error(target, out);
    }
    Copy_Sum_Delta_To_Delta(n_layers, layer, Sum_Delta_W, Sum_delta_bias);
    learn.Update_weigths (n_layers, layer);   
    error = RMS_error(error, denom);   
    Qerror.push((float)error);
    if (printout) cout << "." << flush;
    if (error <= errtol)  {
      if (printout) cout << endl;  
      Delete_Sum_Delta(Sum_Delta_W, Sum_delta_bias); 
      return STOP_ERR_TOL;
    }
    learn.Update_param(error, iter);  
  }
  Delete_Sum_Delta(Sum_Delta_W, Sum_delta_bias);
	if (printout) cout << endl;
  return STOP_NUM_ITERAZ;
}
void OneLayer::Update_Sum_Delta(unsigned n_layers, matrix* Sum_Delta_W[], vect* Sum_delta_bias[])
{
  int j;
  for (j=0; j<n_layers; j++)  { 
    **(Sum_Delta_W+j) += layer[j].Read_DeltaW();
    **(Sum_delta_bias+j) += layer[j].Read_Delta_bias();
  }
}
void OneLayer::do_Sum_deltas(unsigned n_layers, matrix* Sum_Delta_W[], vect* Sum_delta_bias[])
{
  int j;
  for (j=0; j<n_layers; j++)  {  
     **(Sum_Delta_W+j) = layer[j].Read_DeltaW();
     **(Sum_delta_bias+j) = layer[j].Read_Delta_bias();
  }
}
void OneLayer::Zero(unsigned n_layers) 
{
  int i,r,c;
  unsigned rig, col;
  for (i=0; i < n_layers; i++)  {
    matrix& m = layer[i].Read_DeltaW();
    vect& v = layer[i].Read_Delta_bias();
    rig  = m.dim1();
    col  = m.dim2();    
    for (r=0; r<rig; r++)  {
      v[r] = 0;
      for (c=0; c<col; c++)
        m(r,c) = 0;
    }
 }
}
void OneLayer::Zero_Sum_Delta(unsigned n_layers, matrix* Sum_Delta_W[], vect* Sum_delta_bias[]) 
{
  int i,r,c;
  unsigned rig, col;
  for (i=0; i < n_layers; i++)  {
    matrix& m = **(Sum_Delta_W+i);
    vect& v = **(Sum_delta_bias+i);
    rig  = m.dim1();
    col  = m.dim2();    
    for (r=0; r<rig; r++)  {
      v[r] = 0;
      for (c=0; c<col; c++)
        m(r,c) = 0;
    }
 }
}
void OneLayer::Build_Sum_Delta(matrix* Sum_Delta_W[], vect* Sum_delta_bias[])
{
  for (int i=0; i<MAXLAYERS; i++)  {
    Sum_Delta_W[i] = new matrix();
    Sum_delta_bias[i] = new vect();
  }
}
void OneLayer::Delete_Sum_Delta(matrix* Sum_Delta_W[], vect* Sum_delta_bias[])
{
  int j;
  for (j=0; j<MAXLAYERS; j++) {
    delete Sum_Delta_W[j];
    delete Sum_delta_bias[j];
  }
}   
void OneLayer::Copy_Sum_Delta_To_Delta(unsigned n_layers, LayerNetTrain layer[], matrix* Sum_Delta_W[],
                                       vect* Sum_delta_bias[])
{
  int j;
  for (j=0; j<n_layers; j++)  {  
     layer[j].Read_DeltaW() = **(Sum_Delta_W+j);
     layer[j].Read_Delta_bias() = **(Sum_delta_bias+j);
  }
}
double OneLayer::Calc_square_error(vect& t, vect& out)
{
  int i;
  double x;
  double err = 0;
  for (i=0; i<n_outputs; i++) {
    x = out[i] - t[i];
    err += (x*x);
  }
  return err;
}
double OneLayer::calc_denom_RMS(vect& t, unsigned n_train)
{
  int i,j;
  double x;
  double denom = 0;
  vect t_mean(n_outputs);   
  vect target (n_outputs);  
  for (i=0; i < n_train; i++)
    t_mean += C.col(((int)t[i])-1);
  t_mean = t_mean * (1.0/n_train);
  for (i=0; i<n_train; i++)  {
    target = C.col(((int)t[i])-1);
    for (j=0; j<n_outputs; j++) {
       x = t_mean[j] - target[j];
       denom += (x*x);
    }
  } 
  return denom;
} 
double OneLayer::RMS_error(double err, double denom)
{
  return(err/denom);
}
void OneLayer::save_errors(char* file)
{
  char namebuf[DIMBUFNAMEFILE];
  strcpy(namebuf, file);
  strcat(namebuf, ERR_EXT);
  ofstream f(namebuf);
  if (!f) error("Cannot open file ", namebuf);
  f.setf(ios::fixed, ios::floatfield);
  f.precision(ERR_PRECISION);
  while(!Qerror.empty()) {
    f << Qerror.front() << endl;
		Qerror.pop();
	}
  f << flush; 
  if (f.bad())
    error("Error writing ", namebuf);
}
matrix  OneLayer::test(TrainingSet& tset)
{
  unsigned i,k;
  unsigned num_train = tset.Ntrain();
  vect& t = tset.Read_target();  
  matrix& x = tset.Read_data();     
  matrix  conf(n_class, n_class, 0.0);   
  num_errors = 0;
  for (i = 0; i < num_train; i++) {
    vect& input = x.row(i);
    vect& out = compute(input);    
    if ((k=computeclass(out)) != (int)t[i])  {
      num_errors++; 
    }
    conf(k-1, (int)t[i]-1)+= 1;  
  }
  perc_errors = (double)num_errors/(double)num_train;
  return conf;
}
matrix OneLayer::test(TrainingSet& tset, deque<unsigned>& outputs)
{
  unsigned i,cl;
  unsigned num_train = tset.Ntrain();
  vect& t = tset.Read_target();  
  matrix& x = tset.Read_data();     
  matrix  conf(n_class, n_class, 0.0);  
  num_errors = 0;
	// clear the deque outputs
	outputs.clear();
  for (i = 0; i < num_train; i++) {
    vect& input = x.row(i);
    vect& out = compute(input); 
		cl = computeclass(out);
		outputs.push_back(cl);   
    if ( cl != (int)t[i])
      num_errors++; 
    conf(cl-1, (int)t[i]-1)+=1;  
  }
  perc_errors = (double)num_errors/(double)num_train;
  return conf;
}
unsigned OneLayer::computeclassWTA(vect& out)
{
  int j;
  double max = out[0];
  unsigned cl = 0;
  for (j=1; j<n_class; j++)
    if (max < out[j])  {
      max = out[j]; cl = j;
    }
  return(cl+1);
} 
unsigned OneLayer::computeclass(vect& out)
{
  unsigned n,k;
	double dist = 100000;
	double val;
  unsigned cl = 0;
		 
  for (k = 0; k < n_class; k++) {   
    val = NormL1(out, C.col(k));
      if (val < dist)  {
        dist = val;
        cl = k;
      } 
  }
  return (cl+1);   
}	


unsigned OneLayer::predictclassWTA(vect& vinput) 
{
  vect& out = compute (vinput);
  return computeclassWTA(out);
}

unsigned OneLayer::predictclass(vect& vinput)
{
  vect& out = compute (vinput);
  return computeclass(out);
}


void OneLayer::save_decomp_matrix(ofstream& f)
{
   unsigned i, j;
	 
	 f << n_outputs << "  " << n_class  << endl << endl;
   f.setf(ios::fixed, ios::floatfield);
   f.setf(ios::showpos);
   f.precision(8); 
	 for (i=0; i<n_outputs; i++)  {
    for (j=0; j<n_class; j++) 
       f << C(i,j) << " ";
    f << endl;
  }
  f << endl << endl; 
	
	 
  f << flush;
  if (f.bad())
    error("Error writing decomposition matrix ");
}
void OneLayer::save_out(TrainingSet& tset, char* file)
{
  unsigned i,j;  
  int dim = layer[OUTPUT].Read_noutputs();  
  unsigned num_train = tset.Ntrain();
  vect& t = tset.Read_target();  
  matrix& x = tset.Read_data();    
  char buf[128];
  unsigned selected_class;
  strcpy(buf, file);
  strcat(buf, OUT_EXT);
  ofstream f(buf,ios::out|ios::app);   
  if (!f) error("Cannot open file ", buf);
	
	save_decomp_matrix(f);
  f.setf(ios::fixed, ios::floatfield);
  f.setf(ios::showpos);
  f.precision(OUTPUT_PRECISION); 
  for (i = 0; i < num_train; i++) {
    vect& input = x.row(i);
    vect& out = compute(input);
    for (j=0; j<dim; j++)
      f << out[j] << " ";
    selected_class = computeclass(out);
    f << selected_class << " ";
    f << (unsigned)t[i] << endl; 
  }
  f << flush;
  if (f.bad())
    error("Error writing file ", buf);
}
void OneLayer::save_out(TrainingSet& tset, ofstream& f)
{
  unsigned i,j;  
  int dim = layer[OUTPUT].Read_noutputs();  
  unsigned num_train = tset.Ntrain();
  vect& t = tset.Read_target();  
  matrix& x = tset.Read_data();    
  char buf[128];
  unsigned selected_class;
  f.setf(ios::fixed, ios::floatfield);
  f.setf(ios::showpos);
  f.precision(4);
  for (i = 0; i < num_train; i++) {
    vect& input = x.row(i);
    vect& out = compute(input);
    for (j=0; j<dim; j++)
      f << out[j] << " ";
    selected_class = computeclass(out);
    f << selected_class << " ";
    f << (unsigned)t[i] << endl; 
  }
  f << flush;
  if (f.bad())
    error("Errore writing output file ", buf);
}

/* Saves onto a file the prredicted classes of the net */
void OneLayer::save_predicted(deque<unsigned>& outputs, char* file){
  unsigned i;
	
  ofstream f(file,ios::out);   
  if (!f) error("Cannot open file ", file);
	
	for(i=0; i<outputs.size(); i++) 
	  f << outputs[i] << endl;
		
	if (f.bad())
    error_exit("Error writing output file ", file);
}


void OneLayer::print_errors()
{
  cout << "Neural net errors:" << endl;
  cout << "Number of errors  :  " << num_errors;
  cout << "  :  Percentual error rate  :  " << perc_errors*100 << " %" << endl;
}
void OneLayer::save_weights_cv_heading(char* savecvfile, int nfold)
{
  char namebuf[DIMBUFNAMEFILE];
  strcpy(namebuf, savecvfile);
  strcat(namebuf, NETCV_EXT);
  ofstream f(namebuf);
  if (!f) error("Cannot open heading cv file ", namebuf);
  save_weights_heading(f, namebuf);
	f << "Number_of_folds: " << nfold << endl << endl;
}
void OneLayer::save_weights_cv_body(char* namefile, int fold_n)
{
  char namebuf[DIMBUFNAMEFILE];
	
	strcpy(namebuf, namefile);
	strcat(namebuf, NETCV_EXT);
	ofstream f(namebuf,ios::out|ios::app);
	f << "Fold_n. " << fold_n << endl << endl;
	save_weights_body(f);
  if (f.bad())
    error("Error writung file ", namebuf);
}
int OneLayer::CrossValidate(char *name, unsigned nfold, unsigned num_attr, 
                            char *filerr, LearnType& learninfo, char* savecvfile, 
	                    char* readcvfile,
                            char* outfile, int saveout, int seed, bool printout)
{
	
	unsigned n, num_train[2];
  vector<unsigned> iter(nfold);   
  vect abserr(nfold);          
  vect percerr(nfold);          
  TSetStrClass* trainset, *testset;
  const unsigned DIMBUF = 128;  
  char namebuf[DIMBUF];
  ostrstream ost(namebuf,DIMBUF);
  matrix conf(n_class, n_class, 0.0);   
  matrix sumconf(n_class, n_class, 0.0);
  clock_t start, end;
	unsigned int b_cvsaved = 0, b_cvread = 0;
	ofstream *f;
	
	if (strlen(savecvfile) != 0)
	  b_cvsaved = 1;
	if (strlen(readcvfile) != 0)
	  b_cvread = 1;
		
	if (b_cvsaved) 
	  save_weights_cv_heading(savecvfile, nfold); 
	  
	if (saveout)  {
    strcpy(namebuf, outfile);
    strcat(namebuf, OUT_EXT);  
    f = new ofstream(namebuf,ios::out|ios::app);   
    if (!*f) error("Cannot open file ", namebuf);
    save_decomp_matrix(*f);
  }
	
	cout.precision(4);
	start = clock();     
  for (n=0; n<nfold; n++)  {
    cout << "Fold n. " << n+1 << endl;
		 
    init_weights_norm(seed);  
		 
    ost.seekp(0);
    ost << name << ".f" << n+1 << FOLD_TRAIN_SUFFIX << '\0';
    num_train[0] = wc (namebuf); 
    trainset = new TSetStrClass(num_attr, num_train[0],  namebuf) ; 
		trainset->normalize(); 
		 
	  if (trainset->read_dict().size() != n_class)
	   error_exit("Error: Number of classes does not match between input file and parameter");
		 
    if (learninfo.mode == LearnType::BYPATTERN)
      Learn_by_pattern(*trainset, *(learninfo.learn), iter[n], percerr[n], learninfo.err_tol,
			learninfo.maxits, printout);
    else 
      Learn_by_epoch(*trainset, *(learninfo.learn), iter[n], percerr[n], learninfo.err_tol,
			learninfo.maxits, printout);
    cout << "Training results fold n. " << n+1 << endl;
    cout << "Error = " << percerr[n] << endl;
    cout << "Iterations = " << iter[n] << endl;
    cout << endl << "Testing fold n. " << n+1 << endl;
		 
    ost.seekp(0);
    ost << name << ".f" << n+1 << FOLD_TEST_SUFFIX << '\0';
    num_train[1] = wc (namebuf); 
    testset = new TSetStrClass(num_attr, num_train[1],  namebuf, trainset->read_dict());  
    testset->normalize(trainset->Read_mean(), trainset->Read_stdev());
		conf = test(*testset); 
    SumConf(sumconf, conf);   
    abserr[n] = num_errors;
    percerr[n] = perc_errors*100;
    cout << "Testing fold n. " << n+1 << endl;
		print_conf_matrix(conf, cout);
    cout << "Number of errors : " << (unsigned)abserr[n] << "   Error rate % : " << percerr[n] << endl << endl;
		if (b_cvsaved) 
		  save_weights_cv_body(savecvfile, n+1);
    if (saveout)
      save_out(*testset, *f);
    delete trainset;
		delete testset;		
  }
	end = clock(); 
  SaveCV (filerr, iter, abserr, percerr, num_train[0]+num_train[1], sumconf, start, end);
  print_conf_matrix(sumconf, cout);
  print_resultCV (iter, abserr, percerr, num_train[0]+num_train[1], start, end);
  cout << flush;
	
	if (saveout) {
    (*f).close();
    delete f;
  }
	for (int i=0; i<nfold; i++)
	  if (iter[i]>=learninfo.maxits)
		  return 2;
	return 0;
} 
int OneLayer::CrossValidateTrialWeights(char *name, unsigned nfold, unsigned num_attr, 
                             char *filerr, LearnType& learninfo, char* outfile, 
			     int saveout, int seed, int times, bool printout)
{
  unsigned n, num_train[2], t;
  vector<unsigned> iter(nfold);   
  vect abserr(nfold);          
  vect percerr(nfold);          
  TSetStrClass* trainset, *testset;
  const unsigned DIMBUF = 128;  
  char namebuf[DIMBUF];
  ostrstream ost(namebuf,DIMBUF);
  matrix conf(n_class, n_class, 0.0);   
	matrix current_conf(n_class, n_class, 0.0);   
  matrix sumconf(n_class, n_class, 0.0); 
	double minerr;   
	double current_error;   
	unsigned current_iter;   
	int s;
	clock_t start, end;
	cout.precision(4);
	start = clock();         
  for (n=0; n<nfold; n++)  { 
	  minerr = 1000000;
		s = seed;   
		cout << "Fold n. " << n+1 << endl;
		 
		 
    ost.seekp(0);
    ost << name << ".f" << n+1 << FOLD_TRAIN_SUFFIX << '\0';
    num_train[0] = wc (namebuf); 
    trainset = new TSetStrClass(num_attr, num_train[0],  namebuf) ; 
		trainset->normalize();
		 
	  if (trainset->read_dict().size() != n_class)
	   error_exit("Error: Number of classes does not match between input file and parameter");
		 
    ost.seekp(0);
    ost << name << ".f" << n+1 << FOLD_TEST_SUFFIX << '\0';
    num_train[1] = wc (namebuf); 
    testset = new TSetStrClass(num_attr, num_train[1],  namebuf, trainset->read_dict());  
    testset->normalize(trainset->Read_mean(), trainset->Read_stdev());
		 
		for (t=0; t<times; t++)  {
			 
    	init_weights_norm(s); 
			cout << "Initialization seed of the weights = " << s << endl; 			 
			 
    	if (learninfo.mode == LearnType::BYPATTERN)
      	Learn_by_pattern(*trainset, *(learninfo.learn), current_iter, current_error, learninfo.err_tol, 
				                 learninfo.maxits, printout);
    	else 
      	Learn_by_epoch(*trainset, *(learninfo.learn), current_iter, current_error, learninfo.err_tol,
				               learninfo.maxits, printout);
    	cout << "Training results fold n. " << n+1 << endl;
    	cout << "Error = " << current_error << endl;
    	cout << "Iterations = " << current_iter << endl;
    	 
    	cout << endl << "Testing fold n. " << n+1 << endl;			
			 
			current_conf = test(*testset);
			 
			if (num_errors < minerr) {
			  minerr = abserr[n] = num_errors;
				percerr[n] = perc_errors*100;
				iter[n] = current_iter;
				CopyConf(conf, current_conf);
			}
			s+=100;
	  }
    SumConf(sumconf, conf);   
    cout << "Testing fold n. " << n+1 << endl;
		print_conf_matrix(conf, cout);
    cout << "Number of errors : " << abserr[n] << "  Error rate % : " << percerr[n] << endl << endl;
    if (saveout)
      save_out(*testset, outfile);
    delete trainset;
		delete testset;		
  }
	end = clock();     
  SaveCV (filerr, iter, abserr, percerr, num_train[0]+num_train[1], sumconf, start, end);
  print_conf_matrix(sumconf, cout);
  print_resultCV (iter, abserr, percerr, num_train[0]+num_train[1], start, end);
  cout << flush;
	for (int i=0; i<nfold; i++)
	  if (iter[i]>=learninfo.maxits)
		  return 2;
	return 0;
} 
void OneLayer::SaveCV (char *name, vector<unsigned>& iter, vect& abserr, 
                       vect& percerr, unsigned ntrain, matrix& conf, clock_t start, clock_t end)
{
  int i;
  const char SEP = ' ';
  unsigned itertot = 0;
  double errtot = 0;
  double percerrtot = 0;
  double stdev = 0;
  double x;
  int nfold  = abserr.dim();
  const unsigned DIMBUF = 128;  
  char namebuf[DIMBUF];
  ostrstream ost(namebuf,DIMBUF);
  ost << name << CV_SUFFIX << '\0'; 
  ofstream fcv(namebuf);
  if (!fcv) error("Cannot open file ", namebuf);  
  fcv << "Folds number: " << nfold << endl;
  fcv.setf(ios::fixed, ios::floatfield);
  fcv.setf(ios::left, ios::adjustfield);
  fcv.precision(2);
  for (i=0; i<nfold; i++)  {
    fcv << "Fold n. " << (unsigned)i+1 << endl;
    fcv << "Iterations: " << SEP << iter[i] << SEP << SEP;
    fcv << "Errors: " << SEP << (unsigned)abserr[i] << SEP << SEP;
    fcv << SEP << percerr[i] << '%' << endl;
    itertot += iter[i];
    errtot += (unsigned)abserr[i];
  }
  percerrtot = (errtot/ntrain)*100;
  for (i=0; i<nfold; i++)  {
    x = percerr[i] - percerrtot;
    stdev += (x*x);
  }
  stdev = sqrt(stdev/nfold);
  print_conf_matrix(conf, fcv);
	fcv.setf(ios::left, ios::adjustfield);
  fcv << "Totals:" << endl;
  fcv << "Iterations : " << SEP << setw(5) << itertot << endl;
  fcv << "Errors : " << SEP << setw(5) <<(unsigned)errtot << " : ";
  fcv << "Error rate % : " << SEP << setw(5) << percerrtot << SEP << " : ";
	fcv << "St.dev : " << SEP << setw(5) << stdev << "  :  "; 
	print_elapsed_cpu_time(start, end, fcv);
  if (fcv.bad())
    error("Error writing file ", namebuf);
  fcv << flush;
}  
void OneLayer::print_resultCV (vector<unsigned>& iter, vect& abserr, 
                       vect& percerr, unsigned ntrain, clock_t start, clock_t end) 
{ 
  int i;
  const char SEP = ' ';
  unsigned itertot = 0;
  double errtot = 0;
  double percerrtot = 0;
  double stdev = 0;
  double x;
  int nfold  = abserr.dim();
  cout << "Results of the " << nfold << " cross validation:" << endl;  
  cout.precision(2);
	cout.setf(ios::fixed, ios::floatfield);
  cout.setf(ios::left, ios::adjustfield);
  for (i=0; i<nfold; i++)  {
    cout <<"Fold n. " << setw(5) << i+1 << "  ";
    cout << "Iterations: " << setw(5) << iter[i] << SEP ;
    cout << "Errors: " << setw(5) << (unsigned)abserr[i] << SEP; 
    cout << "Error rate %: " << setw(5) << percerr[i] << endl;
    itertot += iter[i];
    errtot += (unsigned)abserr[i];
  }
  percerrtot = (errtot/ntrain)*100;
  for (i=0; i<nfold; i++)  {
    x = percerr[i] - percerrtot;
    stdev += (x*x);
  }
  stdev = sqrt(stdev/nfold);
	cout.setf(ios::left, ios::adjustfield);
  cout << "Global results :" << endl;
  cout << "Total iterations :       " << setw(5) << itertot << endl;
  cout << "Total number of errors : " << setw(5) << (unsigned)errtot << " : ";
  cout << "Error rate % : " << setw(5) << percerrtot <<  " : ";
  cout << "Standard deviation : " << setw(5) << stdev << " : ";
	print_elapsed_cpu_time(start, end);
}  
TwoLayer::TwoLayer(unsigned nout, unsigned nhidden, unsigned nin, act_func fout, act_func fhid1)
:  OneLayer(nout, nhidden, fout) 
{
  n_layers = 2;
  n_inputs = nin;
  n_hidden1 = nhidden;
  layer[HIDDEN1] = LayerNetTrain(nhidden, nin, fhid1);
}
void TwoLayer::init_weights_norm(int seed)
{ 
  layer[OUTPUT].init_norm_weights(seed);
  layer[HIDDEN1].init_norm_weights(seed);
  TwoLayer::seed = seed;
}
void TwoLayer::init_weights_const(double x)
{
  layer[OUTPUT].init_weights(x);
  layer[HIDDEN1].init_weights(x);
} 
void TwoLayer::print_weights(void)
{
  cout << "Weight matrix of the Two-Layer net :" << endl;
  cout << "Output layer :" << endl;
  layer[OUTPUT].print_weights();
  cout << "Hidden layer :" << endl;
  layer[HIDDEN1].print_weights();
} 
vect& TwoLayer::compute (vect& vinput)
{
  vect& hidout = layer[HIDDEN1].compute(vinput);  
  return (layer[OUTPUT].compute (hidout));   
}
void TwoLayer::backprop(vect& t, vect& x)
{
   layer[OUTPUT].calc_grad(t, layer[HIDDEN1].Read_out());  
   layer[HIDDEN1].backprop(layer[OUTPUT].Read_delta(), layer[OUTPUT].Read_W(), x);
}
void TwoLayer::print_info_grad(void)
{
  int i;
  unsigned n_layers = Nlayers();
  cout << "Two-Layer net : Delta and gradient values. " << endl;
  for (i=0; i<n_layers; i++)  {
    if (i == 0)
      cout << "Output layer : " << endl;
    else
      cout << "Hidden layer : " << endl;
    layer[i].print_delta();
    layer[i].print_grad();
    layer[i].print_grad_bias();
  }
} 
void TwoLayer::save_weights_heading(ofstream& f, char* namebuf)
{
  unsigned fun_out, fun_hid1;
  fun_out = Code_act_fun(layer[OUTPUT].Read_g());
  fun_hid1 = Code_act_fun(layer[HIDDEN1].Read_g());
  f.setf(ios::left, ios::adjustfield);
  f << "Version:  " << 1 << endl;
  f << namebuf << endl;
  f.width(INTEST_SIZE);
  f << "Number_layers: " << 2 << endl;
  f.width(INTEST_SIZE);
  f << "Dim_output: " << n_outputs << "    ";
  f << "Act.function: " << fun_out << endl;
  f.width(INTEST_SIZE);
  f << "Dim_hidden1: " << n_hidden1 << "    ";
  f << "Act.function: " << fun_hid1 << endl;
  f.width(INTEST_SIZE);
  f << "Dim_input: " << n_inputs << endl << endl;
	f << flush;
}
void TwoLayer::save_weights_heading(char* namefile)
{
  char namebuf[DIMBUFNAMEFILE];
  strcpy(namebuf, namefile);
  strcat(namebuf, NET_EXT);
  ofstream f(namebuf);
  if (!f) error("Canno open file  ", namebuf);
  save_weights_heading(f, namebuf);
}
void TwoLayer::save_weights_body(ofstream& f)
{
	 
  f << "Weight matrix output layer :" << endl;
  layer[OUTPUT].save_weights(f);
  f << "Weight matrix hidden layer :" << endl;
  layer[HIDDEN1].save_weights(f);
  f << END << endl <<  flush;  
}
void TwoLayer::save_weights_body(char* namefile)
{
  char namebuf[DIMBUFNAMEFILE];
	
	strcpy(namebuf, namefile);
	strcat(namebuf, NET_EXT);
	ofstream f(namebuf,ios::out|ios::app);
	save_weights_body(f);
  if (f.bad())
    error("Error writing file ", namebuf);
}
void TwoLayer::save_weights(char* namefile)
{
  save_weights_heading(namefile);
	save_weights_body(namefile);
}
ThreeLayer::ThreeLayer(unsigned nout, unsigned nhidden1, unsigned nhidden2, unsigned nin,
                       act_func fout, act_func fhid1, act_func fhid2)
: TwoLayer(nout, nhidden1, nhidden2, fout, fhid1) 
{
  n_layers = 3;
  n_hidden2 = nhidden2;
  n_inputs = nin;
  layer[HIDDEN2] = LayerNetTrain(nhidden2, nin, fhid2);
}
void ThreeLayer::init_weights_norm(int seed)
{ 
  TwoLayer::init_weights_norm(seed);
  layer[HIDDEN2].init_norm_weights(seed);
}
void ThreeLayer::init_weights_const(double x)
{
  TwoLayer::init_weights_const(x);
  layer[HIDDEN2].init_weights(x);
} 
void ThreeLayer::print_weights(void)
{
  cout << "Weight matrix of the Three-Layer net :" << endl;
  cout << "Output layer :" << endl;
  layer[OUTPUT].print_weights();
  cout << "First hidden layer :" << endl;
  layer[HIDDEN1].print_weights();
  cout << "Second hidden layer :" << endl;
  layer[HIDDEN2].print_weights();
} 
vect& ThreeLayer::compute (vect& vinput)
{
  vect& hidout2 = layer[HIDDEN2].compute(vinput);  
  vect& hidout1 = layer[HIDDEN1].compute(hidout2);  
  return (layer[OUTPUT].compute (hidout1));   
}
void ThreeLayer::backprop(vect& t, vect& x)
{
   layer[OUTPUT].calc_grad(t, layer[HIDDEN1].Read_out());  
   layer[HIDDEN1].backprop(layer[OUTPUT].Read_delta(), layer[OUTPUT].Read_W(), layer[HIDDEN2].Read_out()); 
   layer[HIDDEN2].backprop(layer[HIDDEN1].Read_delta(), layer[HIDDEN1].Read_W(), x); 
}
void ThreeLayer::print_info_grad(void)
{
  int i;
  unsigned n_layers = Nlayers();
  cout << "Three-Layer net : Delta and gradient values. " << endl;
  for (i=0; i<n_layers; i++)  {
    switch(i) {
       case 0 : cout << "Output layer : " << endl; break;
       case 1 : cout << "First hidden layer : " << endl; break;
       case 2 : cout << "Second hidden layer : " << endl; break;
       default : error ("Error printing Delta and gradient values"); break;
    }    
    layer[i].print_delta();
    layer[i].print_grad();
    layer[i].print_grad_bias();
  }
} 
void ThreeLayer::save_weights_heading(ofstream& f, char* namebuf)
{
	unsigned fun_out, fun_hid1, fun_hid2;
  fun_out = Code_act_fun(layer[OUTPUT].Read_g());
  fun_hid1 = Code_act_fun(layer[HIDDEN1].Read_g());
  fun_hid2 = Code_act_fun(layer[HIDDEN2].Read_g());
  f.setf(ios::left, ios::adjustfield);
  f << "Version:  " << 1 << endl;
  f << namebuf << endl;
  f.width(INTEST_SIZE);
  f << "Number_layers: " << 3 << endl;
  f.width(INTEST_SIZE);
  f << "Dim_output: " << n_outputs << "    ";
  f << "Act.function: " << fun_out << endl;
  f.width(INTEST_SIZE);
  f << "Dim_hidden1: " << n_hidden1 << "    ";
  f << "Act.function: " << fun_hid1 << endl;
  f.width(INTEST_SIZE);
  f << "Dim_hidden2: " << n_hidden2 << "    ";
  f << "Act.function: " << fun_hid2 << endl;
  f.width(INTEST_SIZE);
  f << "Dim_input: " << n_inputs << endl << endl;
	f << flush;
}
void ThreeLayer::save_weights_heading(char* namefile)
{
  char namebuf[DIMBUFNAMEFILE];
  strcpy(namebuf, namefile);
  strcat(namebuf, NET_EXT);
  ofstream f(namebuf);
	if (!f) error("Cannot open file ", namebuf);
  save_weights_heading(f, namebuf);
}
void ThreeLayer::save_weights_body(ofstream& f)
{
	 
  f << "Weight matrix output layer :" << endl;
  layer[OUTPUT].save_weights(f);
  f << "Weight matrix first hidden layer :" << endl;
  layer[HIDDEN1].save_weights(f);
  f << "Weight matrix second hidden layer :" << endl;
  layer[HIDDEN2].save_weights(f);
  f << END << endl << flush;
}
void ThreeLayer::save_weights_body(char* namefile)
{
  char namebuf[DIMBUFNAMEFILE];
	
	strcpy(namebuf, namefile);
	strcat(namebuf, NET_EXT);
	ofstream f(namebuf,ios::out|ios::app);
	save_weights_body(f);
  if (f.bad())
    error("Error writing file ", namebuf);
}
void ThreeLayer::save_weights(char* namefile)
{  
  save_weights_heading(namefile);
	save_weights_body(namefile);	
}
