// net.h


#ifndef NET_H
#define NET_H

#include<iostream.h>
#include<vector.h>
#include<queue>
#include"act_fun.h"
#include"utils.h"
#include"layer.h"
#include"tset.h"
#include"learn.h"
#include"matrix.h"

#define MAXLAYERS 3



#define SIGMOID  1
#define TANHYP   2
#define IDENTITY 3



#define END "END"

	static const char * const NET_EXT = ".net"; 
	static const char * const NETCV_EXT = ".cvnet"; 
	static const char * const ERR_EXT = ".herr"; 
	static const char * const OUT_EXT = ".out"; 
	static const char * const CV_SUFFIX = ".cv";    

/**
  Abstract base class for Multi-Layer Perceptrons.\\
	@memo Virtual base class for MLP - HTML documentation under construction. 
*/  
  
class Network  {

public: 
   
  virtual vect& compute (vect& vinput) = 0; 
                                           
  inline unsigned Nlayers() const {return n_layers;} 
  inline unsigned Ninput() const {return n_inputs;} 
  inline unsigned Noutput() const {return n_outputs;} 
	

  static const unsigned OUTPUT = 0;
  static const unsigned HIDDEN1 = 1;
  static const unsigned HIDDEN2 = 2;
 
protected:         
  unsigned n_layers;
  unsigned n_inputs;
  unsigned n_outputs;
	
};



/**
  Simple perceptron.\\
	It Implemets a simple perceptron only for testing: no learning is allowed.\\ 
	HTML documentation under construction.
	@memo Simple perceptron  
*/  
class OneLayerTest : public Network {
  
public:
  OneLayerTest() {n_inputs = 0; n_outputs = 0; n_layers = 1; num_errors = 0; perc_errors = 0;}
  OneLayerTest(unsigned nout, unsigned nin, act_func fout=sigmoid);
  virtual ~OneLayerTest() {}
  
  virtual vect& compute (vect& vinput); 
                                            
  virtual void print_weights(void); 
  virtual void init_weights_const(double x);  
  void read_weights(istream& f); 
  unsigned test(TrainingSet& tset);
  void print_errors(); 
	
  inline unsigned read_num_errors() const {return num_errors;}
  inline double read_perc_errors() const {return perc_errors;}
  
protected:  
  
  LayerNet layertest[MAXLAYERS];    
  unsigned num_errors;
  double perc_errors;
  
  unsigned maxclass(vect& out, int dim); 
};
  




/**
  Two-Layer Perceptron.\\
	It Implemets a MLP with one hidden layer only for testing: no learning is allowed.\\ 
	@memo MLP with one hidden layer - HTML documentation under construction.  
*/  
class TwoLayerTest : public  OneLayerTest  {

public:
  TwoLayerTest() {n_hidden1 = 0; n_layers = 2;}
  TwoLayerTest(unsigned nout, unsigned nhidden, unsigned nin, act_func fout=sigmoid, 
                                                                  act_func fhid1=sigmoid);
  virtual ~TwoLayerTest() {}
  
  inline unsigned Nhidden1() const {return n_hidden1;} 
  vect& compute (vect& vinput);                                                       
  void init_weights_const(double x);  
  void print_weights(void); 
  
protected:
  unsigned n_hidden1; 
};


/**
  Three-Layer Perceptron.\\
	It Implemets a MLP with two hidden layers only for testing: no learning is allowed.\\ 
	HTML documentation under construction.
	@memo MLP with two hidden layers  
*/  
class ThreeLayerTest : public TwoLayerTest  {

public:
  ThreeLayerTest() {n_hidden2 = 0; n_layers = 3;}
  ThreeLayerTest(unsigned nout, unsigned nhidden1, unsigned nhidden2, unsigned nin,
                       act_func fout=sigmoid, act_func fhid1=sigmoid, act_func fhid2=sigmoid);
  ~ThreeLayerTest() {}
  
  inline unsigned Nhidden2() const {return n_hidden2;} 
  vect& compute (vect& vinput);                                                
  void init_weights_const(double x);  
  void print_weights(void); 
  
protected:
  unsigned n_hidden2;
};






/**
  Simple perceptron.\\
	It Implements a simple perceptron for classification with learning capabilities.\\ 
	@memo Simple perceptron - 	HTML documentation under construction. 
*/  
class OneLayer : public Network {

protected:
  // precision by which are saved error in the stream
  static const unsigned ERR_PRECISION = 8;  
  /// precision of the net output in the stream
  static const unsigned OUTPUT_PRECISION = 8;  
  
  /// default value of max. number of iterations
  static const unsigned MAXITS = 1000;
	/// default value of the tolerated RMS value
  static const double ERR_TOL = 0.1;
  
  // width of the heading field of the file for saving weigths
  static const unsigned INTEST_SIZE = 18;
	
	/// Cardinality of the classes
	unsigned n_class;
  
  /** Codex matrix one-per-class of dimension n_outputs * n_class.\\
      Ith column correspond  to the codeword of the class i+1;
      Matrix values are determined by the activation function used by the net. 
	*/
	matrix C;  
  
	/** Layers of the neurons. \\
	    They are filled completely only by the children classes.  
	    OneLayer fills only layer[0].  
	*/                 
  LayerNetTrain layer[MAXLAYERS]; 
	/// Queue for recording the errors sequence of the net along the epochs
  queue<float> Qerror; 
  /// Number of testing errors
  unsigned num_errors;
  /// Percentual of testing errors
  double perc_errors;
  /// Random initialization seed for the weights
  int seed;
	///Minimum Hamming distance among codewords of the classes
	unsigned minHam;
 
  virtual void Init_C(void);  
  void analyze (act_func g, double& on, double& off); 
                                                      
  double Calc_square_error(vect& t, vect& out);
  double calc_denom_RMS(vect& t, unsigned n_train); 
  double RMS_error(double err, double denom);
  
  void read_weights(istream& f); 
	
	/** It saves onto a file the general paramenters of the net.\\
			It saves the number of layers, size of the layers and type of the activation function  		
			@param namefile name of the file for saving the net. The suffix net. is appended to namefile
			@see save_weights
	*/
	virtual void save_weights_heading(char* namefile);
	
	/** It saves in a stream the general paramenters of the net.\\
			It saves the number of layers, size of the layers and type of the activation function  		
			@param f streamfile for saving the net. Used by save_weights_heading(char* namefile).
			@param namefile name of the file for saving the net.
			@see save_weights
	*/
	virtual void save_weights_heading(ofstream& f, char* namebuf);
		
	/** It appends onto a file the weights of the net.
			@param namefile name of the file for saving the net. The suffix net. is appended to namefile
			@see save_weights
	*/
	virtual void save_weights_body(char* namefile);
	
	/** It appends to a stream the weights of the net.
			@param f name stream file for saving the net. Used by save_weights_body(char* namefile).
			@see save_weights
	*/
	virtual void save_weights_body(ofstream& f);
	
	/** It saves onto a file the general paramenters of the nets of the cross validation.\\
			It saves the number of layers, size of the layers and type of the activation function  		
			@param savecvfile name of the file for saving the nets. The suffix cvnet. is appended to namefile
			@param nfold total number of the folds
	*/
	void save_weights_cv_heading(char* savecvfile, int nfold);
	
	/** It appends onto a file the weights of the net of the cross validation.
			@param namefile name of the file for saving the net. The suffix cvnet. is appended to namefile
			@param fold_n number of the fold
	*/
	void save_weights_cv_body(char* namefile, int fold_n);
	
  void do_Sum_deltas(unsigned n_layers, matrix* Sum_Delta_W[], vect* Sum_delta_bias[]);
  void Update_Sum_Delta(unsigned n_layers, matrix* Sum_Delta_W[], vect* Sum_delta_bias[]);
  void Zero(unsigned n_layers); 
 
  void Zero_Sum_Delta(unsigned n_layers, matrix* Sum_Delta_W[], vect* Sum_delta_bias[]); 
  void Build_Sum_Delta(matrix* Sum_Delta_W[], vect* Sum_delta_bias[]);
  void Delete_Sum_Delta(matrix* Sum_Delta_W[], vect* Sum_delta_bias[]);
  void Copy_Sum_Delta_To_Delta(unsigned n_layers, LayerNetTrain layer[], matrix* Sum_Delta_W[],
                                       vect* Sum_delta_bias[]);
  
	virtual unsigned computeclassWTA(vect& out); 
	
	virtual unsigned computeclass(vect& out); 

// ------------------------------------------------------------------------------------------- 
public:
  /// Activation, deactivation and threshold values with respectively sigmoid, hyperbolic tangent
	/// and identity activation functions
  static const double SIG_ON = 0.9;
  static const double SIG_OFF = 0.1;
  static const double SIG_TRESHOLD = 0.5;
  static const double TANH_ON = 0.9;
  static const double TANH_OFF = - 0.9;
  static const double TANH_TRESHOLD = 0.0;
  static const double ID_ON = 1.0;
  static const double ID_OFF = 0.0;
  static const double ID_TRESHOLD = 0.0;
  
	/** Default constructor.\\
	    it does nothing.	
	*/
  OneLayer() {n_inputs = 0; n_outputs = 0; n_class=0;}
	
	/** Constructor.\\
	    It builds a one layer perceptron with nout output units, nin input units and fout activation
			function.	
			@param nout number of output units
			@param nin  number of input units
			@param fout type of activation function
	*/
  OneLayer(unsigned nout, unsigned nin, act_func fout=sigmoid);
	
	/** Constructor.\\
	    It builds only the codex matrix and initiailizes the queue of historical errors.
	*/
	OneLayer(unsigned dummy)
	: C(dummy, dummy), Qerror() {}
	
	/// Destructor
  virtual ~OneLayer() {}
  
	/// initializes all values according to a noraml distribution centerd in 0 with unitary standard deviation           
  virtual void init_weights_norm(int s = 0);
	/// initializes all values to x  
  virtual void init_weights_const(double x); 
	
	/** It reads a matrix of weights.\\
	    The matrix is read from the file namefile. The weights are read only if the 
			dimension of matrix agrees with the dimension of the reading net.
			@param namefile file name of the recorded weights to be read.
	*/
  virtual void read_weights(char* namefile); 
	
	/// It prints on stdout the weight matrix.
  virtual void print_weights(void); 
	
	/** It computes the output vector of the net.\\
	    @param vinput input vector of the net
			@return output vector computed by the net
	*/  
  virtual vect& compute (vect& vinput);   
	
	/** It outputs the predicted class (WTA criterion).\\
	    Given an input vector, it predict a class using the winner-takes-all criterion.
	    @param vinput input vector of the net
			@return predicted class  
	*/  
  virtual unsigned predictclassWTA(vect& vinput); 
	
	/** It outputs the predicted class (min. Hamming distance criterion).\\
	    Given an input vector, it predict a class using the minimum Hamming distance criterion.
	    @param vinput input vector of the net
			@return predicted class  
	*/  
	virtual unsigned predictclass(vect& vinput); 
  
	/**  It computes the error gradient of the net. \\
	     It records the error gradient respect to the weights.
			 @param t target vector
			 @param x input vector
	*/
  virtual void backprop(vect& t, vect& x); 
	
	/// It outputs the delta values, i.e. the error gradient of the net.
  virtual void print_info_grad(void); 
	
	/// It prints on stdout the decomposition matrix C
  void print_C(void); 
	
	/** It saves onto a file the weights and the paramenters of the net.\\
			It saves onto the  file namefile.net the parameters of the net and 
			its weights.\\			
  		File format: \\
  		"Version:  " version number (unsigned)  (actually always 1)\\ 
  		namefile (string) \\
  		"Number_layers: " Layers number (unsigned) \\
  		"Dim_output: " Output layer size (unsigned) "Act.function: " Number of the activation function
			(unsigned) \\
  		"Dim_hidden1: " I hidden layer size (unsigned) "Act.function: " Number of the activation function
			(unsigned) // present if number layers > 1 \\
  		"Dim_hidden2: " II hidden layer size (unsigned) "Act.function: " Number of the activation function
			(unsigned) // present if number layers > 2 \\
  		"Dim_input: " Input size (unsigned) \\
  		"Weight matrix first layer :"   \\
  		Weight matrix output layer (double)  \\
  		"Bias vector :"   \\
  		Bias vector output layer (double) \\
  		"Weight matrix first hidden layer :"        // present if number layers > 1  \\
  		Matrix   (matrix of double)                 // present if number layers > 1  \\
  		"Bias vector :"                             // present if number layers > 1  \\
  		Vector (vector of double)                   // present if number layers > 1  \\
  		"Weight matrix second hidden layer :"       // present if number layers > 2  \\
  		Matrix   (matrix of double)                 // present if number layers > 2  \\
  		"Bias vector :"                             // present if number layers > 2  \\
  		Vector (vector of double)                   // present if number layers > 2  \\
  		"END" // File termination string  \\
  		NOTE : the activation function can be: SIGMOID = 1, TANHYP = 2, IDENTITY = 3.
			@param namefile name of the file for saving the net. To namefile is appended the suffix .net.
	*/
  virtual void save_weights(char* namefile); 
	
	/// It returns the queue of the historical errors along the epochs. 
  inline queue<float>& read_qerror()  {return Qerror;}
	
	
  void save_errors(char* file); 
  
  virtual stop_cond Learn_by_pattern(TrainingSet& t_set, Learning& learn,
                                     unsigned& iter, double& error,  
                                     double errtol = ERR_TOL, unsigned maxits = MAXITS,
																		 bool printout = true);
  
	
  virtual stop_cond Learn_by_epoch(TrainingSet& t_set, Learning& learn,
                                   unsigned& iter, double& error,
                                   double errtol = ERR_TOL, unsigned maxits = MAXITS,
																	 bool printout = true);           
  
	/** Testing the neural net.\\
  		It performs the testing of the net on the testing set tset.
			The number of errors is recorded in num_errors and the percentual of errors
			is recorded in perc_errors.
			@param tset training set 
  		@return confusion matrix
	*/
  matrix test(TrainingSet& tset); 
	
	/** Testing the neural net.\\
  		It performs the testing of the net on the testing set tset.
			The number of errors is recorded in num_errors and the percentual of errors
			is recorded in perc_errors.
	    The computed classes are pushed back to the queue outputs.
			@param tset training set
			@param outputs vector of the output classes computed by the network. 
  		@return confusion matrix
	*/
	matrix test(TrainingSet& tset, deque<unsigned>& outputs);    
  
	
  virtual void save_out(TrainingSet& tset, char* file);
	
	/**
  	It saves onto a stream the outputs of the net\\ 
		File format: \\
		y1 y2 ... yn computed_class right_class\\
		Where yi (double) are the bits of the codeword coding the class, 
		computed_class (unsigned) is the class compute by the net,
		right_class (unsigned) is the correct class
  	@param tset training set 
		@param f stream where the outputs are saved 
  */
  void save_out(TrainingSet& tset, ofstream& f); 
 
	/**
  	Saves onto a file the classes predicted by the net.\\ 
		The labels are stored one for each line.
		@param outputs vector of the output classes computed by the network 
		@param file file of the predicted classes 
  */
  void save_predicted(deque<unsigned>& outputs, char* file);
	
  void save_decomp_matrix(ofstream& f);
  
  
  int CrossValidate(char *name, unsigned nfold, unsigned num_attr, 
                            char *filerr, LearnType& learninfo, char* savecvfile = "", 
														char* readcvfile = "", char* outfile = "", int saveout = 0, 
														int seed=0, bool printout=true);
  
  
	virtual int CrossValidateTrialWeights(char *name, unsigned nfold, unsigned num_attr, 
                             char *filerr, LearnType& learninfo, char* outfile = "", 
														 int saveout = 0, int seed=0, int times=3, bool printout=true);
			
	
                 
  void SaveCV (char *name, vector<unsigned>& iter, vect& abserr, 
               vect& percerr, unsigned ntrain, matrix& conf, clock_t start, clock_t end);
                             
  void print_resultCV (vector<unsigned>& iter, vect& abserr, 
                       vect& percerr, unsigned ntrain, clock_t start, clock_t end); 
                    
  void print_errors(); 
  unsigned Code_act_fun(act_func g);
  act_func Decode_act_fun(unsigned fun);
	/// It returns the the random initialization seed for the weights
  int Read_seed(void) {return seed;}
	
	/// Return the testing percentual of errors. 
	double get_perc_errors(void){return perc_errors;}
	
	unsigned Read_minHam(void) {return minHam;}
	
};




/**
  Two-Layer Perceptron.\\
	It Implemets a Two-Layer Perceptron for classification with learning capabilities.\\ 
	@memo MLP with one hidden layer - HTML documentation under construction.  
*/  
class TwoLayer : public OneLayer { 

public:
  TwoLayer() {n_hidden1 = 0; n_layers = 2;}
  TwoLayer(unsigned nout, unsigned nhidden, unsigned nin, act_func fout=sigmoid, 
                                                                  act_func fhid1=sigmoid);
  ~TwoLayer() {}
  
  inline unsigned Nhidden1() const {return n_hidden1;} 
  void init_weights_norm(int seed = 0); 
  void init_weights_const(double x); 
  void print_weights(void);   
  vect& compute (vect& vinput);  
  
  void backprop(vect& t, vect& x); 
                                                
  void print_info_grad(void); 
   
  void save_weights(char* namefile);  
  
protected:
  unsigned n_hidden1;
  /** It saves onto a file the general paramenters of the net.\\
			It saves the number of layers, size of the layers and type of the activation function  		
			@param namefile name of the file for saving the net. The suffix net. is appended to namefile
			@see save_weights
	*/
	virtual void save_weights_heading(char* namefile);
	
	/** It saves in a stream the general paramenters of the net.\\
			It saves the number of layers, size of the layers and type of the activation function  		
			@param f streamfile for saving the net. Used by save_weights_heading(char* namefile).
			@param namebuf name of the file for saving the net.
			@see save_weights
	*/
	virtual void save_weights_heading(ofstream& f, char* namebuf);
		
	/** It appends onto a file the weights of the net.
			@param namefile name of the file for saving the net. The suffix net. is appended to namefile
			@see save_weights
	*/
	virtual void save_weights_body(char* namefile);
	
	/** It appends to a stream the weights of the net.
			@param f name stream file for saving the net. Used by save_weights_body(char* namefile).
			@see save_weights
	*/
	virtual void save_weights_body(ofstream& f);
};



/**
  Three-Layer Perceptron.\\
	It Implemets a Three-Layer Perceptron for classification with learning capabilities.\\ 
	@memo MLP with two hidden layers - 	HTML documentation under construction.  
*/  
class ThreeLayer : public TwoLayer {

public:
  ThreeLayer() {n_hidden2 = 0; n_layers = 3;}
  ThreeLayer(unsigned nout, unsigned nhidden1, unsigned nhidden2, unsigned nin,
                       act_func fout=sigmoid, act_func fhid1=sigmoid, act_func fhid2=sigmoid);
  ~ThreeLayer() {}
  
  void init_weights_norm(int seed = 0);
  void init_weights_const(double x); 
  void print_weights(void);   
  vect& compute (vect& vinput);  
  
  void backprop(vect& t, vect& x); 
                                              
  void print_info_grad(void); 
  void save_weights(char* namefile);  
 
protected:
  unsigned n_hidden2;
  /** It saves onto a file the general paramenters of the net.\\
			It saves the number of layers, size of the layers and type of the activation function  		
			@param namefile name of the file for saving the net. The suffix net. is appended to namefile
			@see save_weights
	*/
	virtual void save_weights_heading(char* namefile);
	
	/** It saves in a stream the general paramenters of the net.\\
			It saves the number of layers, size of the layers and type of the activation function  		
			@param f streamfile for saving the net. Used by save_weights_heading(char* namefile).
			@param namebuf name of the file for saving the net.
			@see save_weights
	*/
	virtual void save_weights_heading(ofstream& f, char* namebuf);
		
	/** It appends onto a file the weights of the net.
			@param namefile name of the file for saving the net. The suffix net. is appended to namefile
			@see save_weights
	*/
	virtual void save_weights_body(char* namefile);
	
	/** It appends to a stream the weights of the net.
			@param f name stream file for saving the net. Used by save_weights_body(char* namefile).
			@see save_weights
	*/
	virtual void save_weights_body(ofstream& f);
};



#endif
  
