// "nn_cv.cc"


#include<stdlib.h>
#include"utils.h"
#include"tset.h"
#include"net.h"
#include"ecocnet.h"
#include"build_net.h"
#include"act_fun.h"
#include"appl.h"



static unsigned nntype = NORMAL;  
static unsigned length = 0;  
static int num_attr = NATTR;    
static int nclass = NCLASS;  
static unsigned nfold = NFOLD;  
static unsigned nlayers = NLAYERS;  
static unsigned nhidden1 = NHIDDEN1;  
static unsigned nhidden2 = NHIDDEN2;  
static char namefile[128];  
static char resultfile[128];   
static char outfile[128];   
static char savecvfile[128];  
static char readcvfile[128];  
static unsigned maxits = MAXITS;  
static double errtol = ERRTOL;     
static int learntype = LEARNTYPE;   
static int learnmode = LEARNMODE;  
static double eta = ETA;    
static double alpha = ALPHA;   
static double incr = INCR;   
static double decr = DECR;   
static double decrmom = DECRMOM;   
static unsigned saveout = 0;   
static unsigned b_savecvnet = 0;  
static unsigned b_readcvnet = 0;  
static bool b_printout = true;
static int seed = 0;
static void print_info_use(void);
static int doarg(char* arg[], int i);
int main(int argc, char* argv[])
{
  int num_train, i;  
  clock_t start, end;
  Learning* gd;
  OneLayer* net;
  unsigned iter = 0;  
  double err = 0.0;   
	unsigned noutput;   
  int bigerror = 0;   
	int times= 4;       
  if (argc < 2) 
     print_info_use();
  if ((argc%2) == 1)
     print_info_use();
  strcpy(namefile, argv[1]);
  strcpy(resultfile, argv[1]);
  strcpy(outfile, argv[1]);
  for (i=2; i<argc; i+=2)
         if (doarg(argv, i))
              print_info_use();
	
	 
	if (length == 0)
	  length = DoDefaultcw(nclass); 
	 
	if (b_savecvnet == 0)
	   savecvfile[0]='\0';    
	 
	if (b_readcvnet == 0)
	   readcvfile[0]='\0';     
		
  sigmoid_init(1.0);
  tanhyp_init(0.8);
  switch (nlayers)  {   
    case 1: 
		        if (nntype == NORMAL)
						  net = new OneLayer(nclass, num_attr); 
						else 
						  net = new OneLayerECOC(nclass, num_attr, length); 					                  
            break;    
    case 2: if (nntype == NORMAL)
		          net = new TwoLayer(nclass, nhidden2, num_attr);
						else 
						  net = new TwoLayerECOC(nclass, nhidden2, num_attr, length); 	
            break;     
    case 3: if (nntype == NORMAL)
		          net = new ThreeLayer(nclass, nhidden1, nhidden2, num_attr);
            else 
						  net = new ThreeLayerECOC(nclass, nhidden1, nhidden2, num_attr, length);
						break;
    default: error ("Not supported number of layers."); break;
  }
  noutput = net->Noutput();
	
	 
	cout << "Number of folds: " << nfold << endl;
	if (nntype == NORMAL)
	  cout << "Standard Multi-Layer Perceptron:" << endl;
	else 
	  cout << "ECOC Multi-Layer Perceptron:" << endl;
	cout << "Number of layers: " << nlayers << endl;
	cout << "Input dimension: " << num_attr << endl;
	if (nlayers == 2)
	  cout << "Units of the hidden layer: " << nhidden2 << endl;
	else if (nlayers == 3)  {
	  cout << "Units of the I hidden layer: " << nhidden1 << endl;
		cout << "Units of the II hidden layer: " << nhidden2 << endl;
  } 
  cout << "Number of classes: " << nclass << endl;
	cout << "Output dimension: " << net->Noutput() << endl;
	cout << "Minimum Hamming distance between codewords: " << net->Read_minHam() << endl;
	
	 
	cout << "Stop conditions:" << endl;
	cout << "Maximum number of iterations : " << maxits << endl;  
	cout << "RMS error threshold : " << errtol << endl; 
  cout << "Learning algorithm: ";
  switch (learntype) {
	  case GRADIENT_DESCENT: 
	            gd = new GradientDescent(eta);
	            cout << "Alg. Gradient descent" << endl;
	            cout << "Learning rate : : " << eta << endl;
	            break;
	  case GRADIENT_DESCENT_DECR_LIN: 
	            gd = new GradDescDecrEtaLin(eta, decrmom);
	            cout << "Alg. Gradient descent linear decrement" << endl;
	            cout << "Learning rate : " << eta << endl;
	            cout << "Linear decrement rate : " << decrmom << endl;
	            break;
	  case GRADIENT_DESCENT_DECR_EXP: 
	            gd = new GradDescDecrEtaExp(eta, decrmom);
	            cout << "Alg. Gradient descent exponential decrement" << endl;
	            cout << "Learning rate : " << eta << endl;
	            cout << "Exponential decrement rate : " << decrmom << endl;
	            break;
	  case MOMENTUM_DESCENT:
	            switch(nlayers)  {
	              case 1: 
	                      gd = new MomentumDescent (noutput, num_attr, eta, alpha);
	                      break;
	              case 2: 
	                      gd = new MomentumDescent(noutput, nhidden2, num_attr, eta, alpha);
	                      break;
	              case 3: 
	                      gd = new MomentumDescent(noutput, nhidden1, nhidden2, num_attr, eta, alpha);
	                      break;
	            }
	            cout << "Alg. Gradient descent momentum" << endl;
	            cout << "Learning rate : " << eta << endl;
	            cout << "Momentum coefficient : " << alpha << endl;
	            break;
	  case MOMENTUM_DESCENT_DECR_LIN:
	            switch(nlayers)  {
	              case 1: 
	                      gd = new MomentumDescentDecrEtaLin(noutput, num_attr, eta, alpha, decrmom);
	                      break;
	              case 2: 
	                      gd = new MomentumDescentDecrEtaLin(noutput, nhidden2, num_attr, eta, alpha, decrmom);
	                      break;
	              case 3: 
	                      gd = new MomentumDescentDecrEtaLin(noutput, nhidden1, nhidden2, num_attr, eta, alpha,
	                                                         decrmom);
	                      break;
	            }
	            cout << "Alg. Momentum descent linear decrement" << endl;
	            cout << "Learning rate : " << eta << endl;
	            cout << "Momentum coefficient : " << alpha << endl;
	            cout << "Linear decrement rate : " << decrmom << endl;
	            break;
	  case MOMENTUM_DESCENT_DECR_EXP:
	            switch(nlayers)  {
	              case 1: 
	                      gd = new MomentumDescentDecrEtaExp(noutput, num_attr, eta, alpha, decrmom);
	                      break;
	              case 2: 
	                      gd = new MomentumDescentDecrEtaExp(noutput, nhidden2, num_attr, eta, alpha, decrmom);
	                      break;
	              case 3: 
	                      gd = new MomentumDescentDecrEtaExp(noutput, nhidden1, nhidden2, num_attr, eta, alpha,
	                                                         decrmom);
	                      break;
	            }
	            cout << "Alg. Momentum descent exponential decrement" << endl;
	            cout << "Learning rate : " << eta << endl;
	            cout << "Momentum coefficient : " << alpha << endl;
	            cout << "Exponential decrement rate : " << decrmom << endl;
	            break;
	  case BOLD_DRIVER: 
	            gd = new BoldDriver(eta, incr, decr);
	            cout << "Alg. Bold Driver" << endl;
	            cout << "Initial learning rate : " << eta << endl;
	            cout << "Increment rate : " << incr << endl;
	            cout << "Decrement rate : " << decr << endl;
	            break;
	  case BOLD_DRIVER_MOMENTUM:
	            switch(nlayers)  {
	              case 1: 
	                      gd = new BoldMomentum(noutput, num_attr, eta, alpha, incr, decr);
	                      break;
	              case 2: 
	                      gd = new BoldMomentum(noutput, nhidden2, num_attr, eta, alpha, incr, decr);
	                      break;
	              case 3: 
	                      gd = new BoldMomentum(noutput, nhidden1, nhidden2, num_attr, eta, alpha, incr, decr);
	                      break;
	            }
	            cout << "Alg. Bold Driver with momentum" << endl;
	            cout << "Initial learning rate : " << eta << endl;
	            cout << "Momentum coefficient : " << alpha << endl;
	            cout << "Increment rate : " << incr << endl;
	            cout << "Decrement rate : " << decr << endl;
	            break;
	  default:  cout << "Not known learning algorithm" << endl;
	            exit(1);
	};
  LearnType::learn_mode mode;
  if (learnmode == BYPATT) {   
     mode = LearnType::BYPATTERN;
     cout << "By pattern learning." << endl;
  }
  else {
     mode = LearnType::BYEPOCH; 
     cout << "By epoch learning." << endl;
  }    
  LearnType learninfo(gd, maxits, errtol, mode);
  start = clock(); 
  bigerror = net->CrossValidate(namefile, nfold, num_attr, resultfile, 
	                             learninfo, savecvfile, readcvfile, outfile, saveout, seed, b_printout);
  end = clock(); 
  delete gd;
  delete net;
  print_elapsed_cpu_time(start, end);
  return bigerror;
}  
static void print_info_use(void)
{
  cerr << "
  nn_cv.  NEURObjects application for training and testing
          Multi-Layer-Perceptrons, using cross-validation techniques.			
  Usage: nn_cv trainfile [options]
  Options:
  -res  string        file summarizing cross-validation results 
  -type string        MLP type:
                      normal     (MLP-OPC) (default)
                      ecoc       (MLP-ECOC)
  -lencw unsigned     codeword length
  -nc unsigned        number of the classes
  -nf unsigned        number of the folds
  -nl unsigned        number of the layers (considering also the input layer):
                      1: one layer (no hidden units)
                      2: two layers (1 hidden layer)
                      3: three layers (2 hidden layers)
  -h1 unsigned        number of hidden units of the first hidden layer of the three layers MLP
  -h2 unsigned        number of hidden units of the second hidden layer of the three layers MLP or
                      number of hidden units of the two layers MLP (default 5)
  -d  unsigned        sample dimension/number of input units
  -maxit unsigned     maximal number of iterations of the backpro algorithm  (default 500)
  -maxerr double      maximal normalized root mean square error (RMS) (default 0.1)
  -alg string         Backpropagation learning algorithm type:
                      gd         (gradient descent)
                      gd_dl      (gradient descent linear decrement)
                      gd_de      (gradient descent exponential decrement)
                      md         (gradient descent with momentum)
                      md_dl      (gradient descent with momentum and linear decrement)
                      md_de      (gradient descent with momentum and exponential decrement)
                      bold       (bold driver)
                      boldmom    (bold driver with momentum)
  -rate double        learning rate (default 0.07)
  -mom double         momentum rate (default 0.5)
  -incbold double     bold driver increment rate
  -decbold double     bold driver decrement rate
  -decr double        gradient decrement
  -seed              seed for the random initialization of the weights. 
                     If 0 the initialization is performed using current computer time.  
  -s string          save parameters and weights of all the cross validated MLP onto a file
  -out string        save neural net outputs onto a file
  -w string          read parameters and weights for all the cross validated MLP from a file 
  -p integer         1 = printing a point at each iteration; 0 = noprint
  " << endl;
  exit(1);
}  
int doarg(char* arg[], int i)
{
  int j;
  if (!strcmp(arg[i], "-res"))
    strcpy(resultfile, arg[i+1]);
	else if (!strcmp(arg[i], "-lencw"))
    length = atoi(arg[i+1]);
  else if (!strcmp(arg[i], "-nc")) {
    nclass = atoi(arg[i+1]);
		if (nclass < 2) error_exit("Classes must be 2 or more");}
  else if (!strcmp(arg[i], "-nf")) {
    nfold = atoi(arg[i+1]);
		if (nfold < 2) error_exit("Folds must be 2 or more");}
  else if (!strcmp(arg[i], "-nl")) {
    nlayers = atoi(arg[i+1]);
		if ((nlayers < 1) || (nlayers > 3)) error_exit("Number of layers must be between 1 and 3");}
  else if (!strcmp(arg[i], "-h1"))
    nhidden1 = atoi(arg[i+1]);
  else if (!strcmp(arg[i], "-h2"))
    nhidden2 = atoi(arg[i+1]);
  else if (!strcmp(arg[i], "-d")) {
    num_attr = atoi(arg[i+1]);
		if (num_attr < 1) error_exit("Number of attributes must be 1 or more");}
  else if (!strcmp(arg[i], "-maxit"))
    maxits = atoi(arg[i+1]);
  else if (!strcmp(arg[i], "-maxerr"))
    errtol = atof(arg[i+1]);
  else if (!strcmp(arg[i], "-type")) { 
    if ((nntype = Analyze_type(arg[i+1])) >= NUMTYPE)
       print_info_use();
  }
	else if (!strcmp(arg[i], "-alg")) { 
    if ((learntype = Analyze_alg(arg[i+1])) >= NUMALG)
       print_info_use();
  }
  else if (!strcmp(arg[i], "-mode")) {
    if ((learnmode = Analyze_mode(arg[i+1])) >= NUMMODE)
       print_info_use();
  }
  else if (!strcmp(arg[i], "-rate"))
    eta = atof(arg[i+1]);
  else if (!strcmp(arg[i], "-mom"))
    alpha = atof(arg[i+1]);
  else if (!strcmp(arg[i], "-incbold"))
    incr = atof(arg[i+1]);
  else if (!strcmp(arg[i], "-decbold"))
    decr = atof(arg[i+1]);
  else if (!strcmp(arg[i], "-decr"))
    decrmom = atof(arg[i+1]);
  else if (!strcmp(arg[i], "-out")) {
    saveout = 1; 
    strcpy(outfile, arg[i+1]);
  }
	else if (!strcmp(arg[i], "-s")) {
    b_savecvnet = 1; 
    strcpy(savecvfile, arg[i+1]);
  }
	else if (!strcmp(arg[i], "-w")) {
    b_readcvnet = 1; 
    strcpy(readcvfile, arg[i+1]);
  }
	else if (!strcmp(arg[i], "-p")) {
    j = atoi(arg[i+1]);
		if (j == 0)
		   b_printout = false;
	}
	else if (!strcmp(arg[i], "-seed"))
    seed = atoi(arg[i+1]);
  else 
    return 1;
  return 0;
} 
