// outerr.cc

#include<stdlib.h>
#include<math.h>
#include<iostream.h>
#include<fstream.h>
#include<string.h>

#include"outerr.h"
#include"utils.h"


// =============== Class Outerr ===========================

// Constructor.
Outerr::Outerr(char* infile, char *outfile)
{
  
	int i,j;
	char outbuf[128]; 

  strcpy(outbuf, outfile);
  strcat(outbuf, OUTERR_EXT);  
	
	// apertura file di lettura
  ifstream inpf(infile);
  if (!inpf) error_exit("Cannot open input file ", infile);
  
  
  // Read dimensions of ecoc matrix
  inpf >> ndico >> nclass;
	
	// build ecoc matrix
	M = new float*[ndico];
	for (i=0; i<ndico; i++)
	  M[i] = new float[nclass];
		
	// build codewords and error vectors
	cc = new float[ndico];
	e  = new float[ndico];
	
	// read ecoc matrix	
	for (i=0; i<ndico; i++)
	  for (j=0; j<nclass; j++)
		  inpf >> M[i][j]; 
			
	
	ofstream outf(outbuf); 
  if (!outf) error_exit("Cannot open error output file", outbuf);
	outf.setf(ios::fixed, ios::floatfield);
  outf.setf(ios::showpos);
  outf.precision(8); 
	
	Read_codeword_and_class(inpf);
	while (!inpf.eof()) {
		Calc_error();
	  Save_error(outf);
		Read_codeword_and_class(inpf);
	}
	
	// fine scrittura file
  outf << flush;
  if (outf.bad() || !inpf.eof())
    error_exit("Error saving output error file ", outbuf);
	
}


// Destructor
Outerr::~Outerr()
{
	int i;
	
	for (i=0; i<ndico; i++)
	  delete [] M[i];
	delete [] M;
		
	delete [] cc;
	delete [] e;
}


// Read the codeword and class computed from stream inpf
void Outerr::Read_codeword_and_class(ifstream& inpf)
{
  int i;
	
	for(i=0; i<ndico; i++)
	  inpf >> cc[i];
	inpf >> k >> k;  // the first is the computed, the second is the correct class
}

// It computes the error codeword
void Outerr::Calc_error()
{
  int i;
	int cl;
	
	cl = k - 1;	
	for(i=0; i<ndico; i++) 
	  e[i] = fabs(cc[i] - M[i][cl]);
}
	
// It saves the error codeword
void Outerr::Save_error(ofstream& f)
{
  int j;
		
	for (j=0; j<ndico; j++) 
    f << e[j] << " ";
  f << endl;
}	
	







