// outerr.h
 


#ifndef OUTERR_H
#define OUTERR_H

#include<iostream.h>
#include<fstream.h>

   // Suffix of the output file of errors
	static const char OUTERR_EXT[] = ".outerr";
/**
   Class for generating error files of the outputs of the ecoc nets.\\
   It reads files .out of the outputs of an ECOC-MLP or file .pout od the PND and
	 outputs the error file computing the absolute error between the outputs of the net
	 and the correct output. 
	 It reads from file the codewords computed by the net and the correct codewords
	 and outputs in the output file .outerr the errors for each bit of the codeword.
	 The file .outerr computed can be used by the classes for evaluating
	 mutual information and mutual information errors.
	 @see MutualInfo2
	 @see MIError2
	 @see MutualMatrix
*/ 


class Outerr {

private:
  /// Number of dichotomies (rows)
	unsigned int ndico;
	/// Number of c;asses (columns)
	unsigned int nclass;	
	/// codeword computed by the net. 
	float *cc;
	/// codeword error |c-cc|
	float *e;
	/// correct class
	unsigned int k;
	/// ECOC matrix
	float **M;
	

  /// Read the codeword and class computed from stream inpf
  void Read_codeword_and_class(ifstream& inpf);
	
	/// It computes the error codeword
  void Calc_error(void);
	
	/// It saves the error codeword
  void Save_error(ofstream& f);


public:
  /** Constructor. \\
	    It reads from file the codewords computed by the net and the correct codewords
			and outputs in the output file .outerr the errors for each bit of the codeword
	    @param infile input file
			@param outfile output file
	*/
  Outerr(char* infile, char *outfile);

  /// Destructor
  ~Outerr();
  
  

};
#endif
