// "patgen.cc"

#include"patgen.h"


const char* DATA_EXT = ".data";
const char* INFO_EXT = ".info";  
static const unsigned DIMBUFNAMEFILE = 64;  
static const unsigned DIMBUF = 256;
static  const char* LONG_FORMAT =  "Longformat";   
static  const char* SHORT_FORMAT =  "Shortformat";  
static  const char SEPARATOR = ',';
static  const char* VERSION = "Version: ";
static  const char* NAME = "File_name: ";
static  const char* N_CLASS = "Cluster_number: ";
static  const char* CENTRUM_RANGE = "Centrum_range: ";
static  const char* PAT_DIM = "Pattern_dimension: ";
static  const char* SPACE = "  ";
static  const char* CLASS = "Class: ";
static  const char* COORD = "Centrum_coord: ";
static  const char* DEV_ST = "Standard_deviation: ";
static  const char* NUM_PAT = "Pattern_number: ";
static  const char* TOT_PAT = "Pattern_total: ";
void Patgen::create_ran_spheric(char* namefile, unsigned n_class, unsigned n_pattern, unsigned dim, 
                           double rangemin, double rangemax, double sigma)
{
  unsigned j,k,pat;
  long seed = (long) -time(NULL);
  vect centrum(dim);
  vect pattern(dim);
  char datafile[DIMBUFNAMEFILE];
  strcpy(datafile, namefile);
  strcat(datafile, DATA_EXT);
  char infofile[DIMBUFNAMEFILE];
  strcpy(infofile, namefile);
  strcat(infofile, INFO_EXT);
  ofstream fdata(datafile);
  if (!fdata) error("Cannot open data file ", datafile);
  ofstream finfo(infofile);
  if (!finfo) error("Cannot open info file ", infofile);
  do_header_info(finfo, 1, infofile, n_class, rangemin, rangemax, dim);  
  for (k=1; k<=n_class; k++)  {
    make_rand_vector (dim, rangemin, rangemax, centrum);
    do_info_record(finfo, centrum, k, sigma, dim, n_pattern);
    for (pat=0; pat < n_pattern; pat++)  {
       make_pattern(centrum, sigma, dim, pattern);
       save_pattern(fdata, pattern, k, dim);       
    }
  }
  do_pattern_total(finfo, n_class * n_pattern); 
  if (fdata.bad())
    error("Error writing data file ", datafile);
  fdata << flush;
  if (finfo.bad())
    error("Error writing info file  ", infofile);
  finfo << flush;
}
void Patgen::make_rand_vector (unsigned dim, double rangemin, double rangemax, vect& centrum)
{
  int j;
  for(j=0; j<dim; j++) 
    centrum[j] = rangemin + ran1(&seed) * (rangemax-rangemin); 
}
void Patgen::make_pattern(vect& centrum, double sigma, unsigned dim, vect& pattern) 
{
    int j;
    for(j=0; j<dim; j++) 
      pattern[j] = centrum[j] + gasdev(&seed) * sigma;  
}
void Patgen::make_pattern(vect& centrum, vect& sigma, unsigned dim, vect& pattern) 
{
    int j;
    for(j=0; j<dim; j++) 
      pattern[j] = centrum[j] + gasdev(&seed) * sigma[j];  
}
void Patgen::save_pattern(ofstream& fdata, vect& patt, unsigned k, unsigned dim)
{
  for (int j=0; j<dim; j++)  
    fdata << patt[j] << SEPARATOR;
  fdata << k << endl;
}
void Patgen::do_header_info(ofstream& finfo, unsigned v_num, char* filename, unsigned n_class,
                            double rmin, double rmax, unsigned dim)
{
  finfo << VERSION << v_num << endl;
  finfo << NAME << filename << endl;
  finfo << N_CLASS << n_class << endl;
  finfo << CENTRUM_RANGE << rmin << SPACE << rmax << endl;
  finfo << PAT_DIM << dim << endl << endl;
}
void Patgen::do_info_record(ofstream& finfo, vect& centrum, unsigned k, double sigma,
                            unsigned dim, unsigned n_pattern)
{ 
  int j;
  finfo << CLASS  << k << endl;
  finfo << COORD << endl;
  for (j=0; j<dim; j++)  
     finfo << centrum[j] << SEPARATOR;
  finfo << endl << DEV_ST << endl;
  for (j=0; j<dim; j++)  
     finfo <<  sigma << SEPARATOR;
  finfo << endl << NUM_PAT << n_pattern << endl;
} 
void Patgen::do_info_record_sigma(ofstream& finfo, vect& centrum, unsigned k, vect& sigma,
                            unsigned dim, unsigned n_pattern)
{ 
  int j;
  finfo << CLASS  << k << endl;
  finfo << COORD << endl;
  for (j=0; j<dim; j++)  
     finfo << centrum[j] << SEPARATOR;
  finfo << endl << DEV_ST << endl;
  for (j=0; j<dim; j++)  
     finfo <<  sigma[j] << SEPARATOR;
  finfo << endl << NUM_PAT << n_pattern << endl;
} 
void Patgen::do_pattern_total(ofstream& finfo, unsigned pat_tot)
{
  finfo << endl << TOT_PAT << pat_tot << endl;
}
void Patgen::create_ran_variable(char* namefile, unsigned n_class, unsigned patmin, unsigned patmax, 
                           unsigned dim, double rangemin, double rangemax, double sigmin, double sigmax)
{
  unsigned j,k,pat;
  unsigned n_pattern;
  unsigned tot_pat = 0;
  long seed = (long) -time(NULL);
  vect centrum(dim);
  vect pattern(dim);
  vect sigma(dim); 
  char datafile[DIMBUFNAMEFILE];
  strcpy(datafile, namefile);
  strcat(datafile, DATA_EXT);
  char infofile[DIMBUFNAMEFILE];
  strcpy(infofile, namefile);
  strcat(infofile, INFO_EXT);
  ofstream fdata(datafile);
  if (!fdata) error("Cannot open data file  ", datafile);
  ofstream finfo(infofile);
  if (!finfo) error("Cannot open info file ", infofile);
  do_header_info(finfo, 1, infofile, n_class, rangemin, rangemax, dim);  
  for (k=1; k<=n_class; k++)  {
    make_rand_vector (dim, rangemin, rangemax, centrum);
    n_pattern = patmin + (unsigned)(ran1(&seed) * (patmax-patmin));
    tot_pat += n_pattern;
    make_rand_vector (dim, sigmin, sigmax, sigma);
    do_info_record_sigma(finfo, centrum, k, sigma, dim, n_pattern);
    for (pat=0; pat < n_pattern; pat++)  {
       make_pattern(centrum, sigma, dim, pattern);
       save_pattern(fdata, pattern, k, dim);       
    }
  }
  do_pattern_total(finfo, tot_pat); 
  if (fdata.bad())
    error("Error writing data file ", datafile);
  fdata << flush;
  if (finfo.bad())
    error("Error writing info file  ", infofile);
  finfo << flush;
}
void Patgen::create_reading_info(char* infof, char* dataf)
{
  unsigned j,k,pat;
  unsigned n_pattern;
  unsigned version, n_class, dim;
  double rangemin, rangemax;
  unsigned target_class, num_pattern;
  seed = (long) -time(NULL);
  char datafile[DIMBUFNAMEFILE];
  strcpy(datafile, dataf);
  strcat(datafile, DATA_EXT);
  char infofile[DIMBUFNAMEFILE];
  strcpy(infofile, infof);
  strcat(infofile, INFO_EXT);
  ofstream fdata(datafile);
  if (!fdata) error("Cannot open data file ", datafile);
  ifstream finfo(infofile);
  if (!finfo) error("Cannot open info file  ", infofile);
  read_header_info(finfo, version, n_class, rangemin, rangemax, dim);  
  vect centrum(dim);
  vect pattern(dim);
  vect sigma(dim); 
  for (k=1; k<=n_class; k++)  {
    read_class_record(finfo, target_class, centrum, sigma, num_pattern);
    for (pat=0; pat < num_pattern; pat++)  {
      make_pattern(centrum, sigma, dim, pattern);
      save_pattern(fdata, pattern, target_class, dim);
    }
  }
  finfo >> infofile >> num_pattern; 
  if (fdata.bad())
    error("Error writing data file ", datafile);
  fdata << flush;
  if (finfo.bad())
    error("Error reading info file ", infofile);
}
void Patgen::read_header_info(ifstream& finfo, unsigned& v_num, unsigned& n_class,
                              double& rmin, double& rmax, unsigned& dim)
{
  char buf[DIMBUF];
  finfo >> buf >> v_num;
  if (strcmp(buf, "Version:"))   
    error ("Formato del file non .info");
  if (v_num != 1)
    error("File version .info non recognized.");
  finfo >> buf >> buf;
  finfo >> buf >> n_class;
  finfo >> buf >> rmin >> rmax;
  finfo >> buf >> dim;
  if (finfo.bad())
    error("Patgen::read_header_info", "Errore reading file ");
}  
void Patgen::read_class_record(ifstream& finfo, unsigned& target, vect& centrum, 
                               vect& sigma, unsigned& num_pattern) 
{
  char buf[DIMBUF];
  char c;
  int t, dim = centrum.dim();
  finfo >> buf >> target; 
  finfo >> buf;
  for (t=0; t<dim; t++)
    finfo >> centrum[t] >> c;
  finfo >> buf;
  for (t=0; t<dim; t++)
    finfo >> sigma[t] >> c;
  finfo >> buf >> num_pattern;
  if (finfo.bad())
    error("Patgen::read_class_record", "Errore reading file ");
}
void Patgen::create_reading_info_num_pattern(char* infof, char* dataf, unsigned patmin, unsigned patmax)  
{
  unsigned j,k,pat;
  unsigned n_pattern;
  unsigned version, n_class, dim;
  double rangemin, rangemax;
  unsigned target_class, num_pattern;
  unsigned tot_pattern = 0;
  seed = (long) -time(NULL);
  char datafile[DIMBUFNAMEFILE];
  strcpy(datafile, dataf);
  strcat(datafile, DATA_EXT);
  char infofile[DIMBUFNAMEFILE];
  strcpy(infofile, infof);
  strcat(infofile, INFO_EXT);
  char newinfofile[DIMBUFNAMEFILE];
  strcpy(newinfofile, dataf);
  strcat(newinfofile, INFO_EXT);
  ofstream fdata(datafile);
  if (!fdata) error("Cannot open data file ", datafile);
  ofstream finfonew(newinfofile);
  if (!finfonew) error("Cannot open new info file ", newinfofile);
  ifstream finfo(infofile);
  if (!finfo) error("Cannot open info file ", infofile);
  read_header_info(finfo, version, n_class, rangemin, rangemax, dim); 
  do_header_info(finfonew, version, newinfofile, n_class, rangemin, rangemax, dim);  
  vect centrum(dim);
  vect pattern(dim);
  vect sigma(dim); 
  for (k=1; k<=n_class; k++)  {
    read_class_record(finfo, target_class, centrum, sigma, num_pattern);    
    num_pattern = patmin + (int)(ran1(&seed) * (patmax-patmin));
    tot_pattern += num_pattern;
    do_info_record_sigma(finfonew, centrum, target_class, sigma, dim, num_pattern);
    for (pat=0; pat < num_pattern; pat++)  {
      make_pattern(centrum, sigma, dim, pattern);
      save_pattern(fdata, pattern, target_class, dim);
    }
  }
  finfo >> infofile >> num_pattern; 
  do_pattern_total(finfonew, tot_pattern); 
  if (fdata.bad())
    error("Error writing data file ", datafile);
  fdata << flush;
  if (finfonew.bad())
    error("Error writing info file ", newinfofile);
  finfonew << flush;
  if (finfo.bad())
    error("Error reading info file ", infofile);
}
