// "pnd.cc"


#include<stdlib.h>
#include"utils.h"
#include"tset.h"
#include"act_fun.h"
#include"polidico.h"
#include"appl.h"


static int num_attr = NATTR;    
static int nclass = NCLASS;  
static unsigned ndico = 0;  
static unsigned nlayers = NLAYERS;  
static unsigned nhidden1 = NHIDDEN1;  
static unsigned nhidden2 = NHIDDEN2;  
static char trainfile[128];  
static char testfile[128];   
static char savepoli[128];   
static char outfile[128];   
static char saverr[128];   
static unsigned maxits = MAXITS;  
static double errtol = ERRTOL;     
static int learntype = LEARNTYPE;   
static int learnmode = LEARNMODE;  
static double eta = ETA;    
static double alpha = ALPHA;   
static double incr = INCR;   
static double decr = DECR;   
static double decrmom = DECRMOM;   
static unsigned decomptype = OPC;   
static unsigned savenet = 0;   
static unsigned saverrflag = 0;   
static unsigned saveout = 0;   
static int seed = 0;   
static bool b_printout = true;
static void print_info_use(void);
int doarg(char* arg[], int i);
int main(int argc, char* argv[])
{
  int num_train, i;
  clock_t start, end;
  Learning* gd;
  int bigerror = 0;  
  if (argc < 2) 
     print_info_use();
  if ((argc%2) == 1)
     print_info_use();
  strcpy(trainfile, argv[1]);
  strcpy(testfile, argv[1]);
  strcpy(outfile, argv[1]);
  strcpy(saverr, argv[1]);
  for (i=2; i<argc; i+=2)
         if (doarg(argv, i))
              print_info_use();
							
	 
	if (ndico == 0)
	  ndico = DoDefaultcw(nclass); 
  matrix conf(nclass,nclass,0.0);  
  PoliDico<d_OPC>*       poliOPC;
  PoliDico<d_PWC>*       poliPWC;
  PoliDico<d_CC>*        poliCC;
  PoliDico<d_ECOC_ex>*   poliECOC_ex;
	PoliDico<d_ECOC_BCH>*  poliECOC_BCH;
  sigmoid_init(1.0);
  tanhyp_init(0.8);
  num_train = wc (trainfile); 
  TSetStrClass trainset( num_attr, num_train, trainfile);   
  trainset.normalize();
	if (trainset.read_dict().size() != nclass)
	   error_exit("Number of classes does not match between input file and parameter");
		 
  num_train = wc (testfile); 
  TSetStrClass testset( num_attr, num_train, testfile, trainset.read_dict());     
  testset.normalize(trainset.Read_mean(), trainset.Read_stdev());
	unsigned num_neuro[3] = {num_attr, nhidden2, nhidden1};
	act_func t_fun [3] = {sigmoid, sigmoid, sigmoid}; 
	DicoType dico(nlayers, num_neuro, t_fun);
	 
	switch (decomptype) {
	  case OPC: poliOPC  = new PoliDico<d_OPC>(nclass, dico, sigmoid);
	            cout << "OPC-PND" << endl;
	            cout << "Number of classes = " << poliOPC->Read_n_class() << endl;
	            cout << "Number of dichotomizers = " << poliOPC->Read_n_dico() << endl;
	            cout << "Minimum Hamming distance between codewords = " << poliOPC->Read_scomp().MinHamming(nclass) << endl;
							break;
	  case PWC: poliPWC  = new PoliDico<d_PWC>(nclass, dico, sigmoid);
	            cout << "PWC-PND" << endl;
	            cout << "Number of classes = " << poliPWC->Read_n_class() << endl;
	            cout << "Number of dichotomizers = " << poliPWC->Read_n_dico() << endl;
	            cout << "Minimum Hamming distance between codewords = " << poliPWC->Read_scomp().MinHamming(nclass) << endl;
							break;
	  case CC:  poliCC  = new PoliDico<d_CC>(nclass, dico, sigmoid);
	            cout << "CC-PND" << endl;
	            cout << "Number of classes = " << poliCC->Read_n_class() << endl;
	            cout << "Number of dichotomizers = " << poliCC->Read_n_dico() << endl;
							cout << "Minimum Hamming distance between codewords = " << poliCC->Read_scomp().MinHamming(nclass) << endl;
	            break;
	  case ECOC_ex:  
	            poliECOC_ex  = new PoliDico<d_ECOC_ex>(nclass, dico, sigmoid);
	            cout << "ECOC-PND exhaustive" << endl;
	            cout << "Number of classesi = " << poliECOC_ex->Read_n_class() << endl;
	            cout << "Number of dichotomizers = " << poliECOC_ex->Read_n_dico() << endl;
							cout << "Minimum Hamming distance between codewords = " << poliECOC_ex->Read_scomp().MinHamming(nclass) << endl;
	            break;
		case ECOC_BCH:  
	            poliECOC_BCH  = new PoliDico<d_ECOC_BCH>(nclass, dico, sigmoid, ndico);
	            cout << "ECOC-PND BCH" << endl;
	            cout << "Number of classes = " << poliECOC_BCH->Read_n_class() << endl;
	            cout << "Number of dichotomizers = " << poliECOC_BCH->Read_n_dico() << endl;
							cout << "Minimum Hamming distance between codewords = " << poliECOC_BCH->Read_scomp().MinHamming(nclass) << endl;
	            break;
	  default:  cout << "Not known PND" << endl;
	            exit(1);
	};
	
	
	
	 
	cout << "Dichotomizers specifications:" << endl;
	cout << "Number of layers: " << nlayers << endl;
	cout << "Input dimension: " << num_attr << endl;
	if (nlayers == 2)
	  cout << "Units of the hidden layer: " << nhidden2 << endl;
	else if (nlayers == 3)  {
	  cout << "Units of the I hidden layer: " << nhidden1 << endl;
		cout << "Units of the II hidden layer: " << nhidden2 << endl;
  }
	
	 
	cout << "Stop conditions:" << endl;
	cout << "Maximum number of iterations : " << maxits << endl;  
	cout << "RMS error threshold  : " << errtol << endl; 
  cout << "Learning algorithm: ";
  switch (learntype) {
	  case GRADIENT_DESCENT: 
	            gd = new GradientDescent(eta);
	            cout << "Alg. Gradient descent" << endl;
	            cout << "Learning rate : " << eta << endl;
	            break;
	  case GRADIENT_DESCENT_DECR_LIN: 
	            gd = new GradDescDecrEtaLin(eta, decrmom);
	            cout << "Alg. Gradient descent linear decrement " << endl;
	            cout << "Learning rate : " << eta << endl;
	            cout << "Linear decrement rate  : " << decrmom << endl;
	            break;
	  case GRADIENT_DESCENT_DECR_EXP: 
	            gd = new GradDescDecrEtaExp(eta, decrmom);
	            cout << "Alg. Gradient descent exponential decrement" << endl;
	            cout << "Learning rate : " << eta << endl;
	            cout << "Exponential decrement rate : " << decrmom << endl;
	            break;
	  case MOMENTUM_DESCENT:
	            switch(nlayers)  {
	              case 1: 
	                      gd = new MomentumDescent (1, num_attr, eta, alpha);
	                      break;
	              case 2: 
	                      gd = new MomentumDescent(1, nhidden2, num_attr, eta, alpha);
	                      break;
	              case 3: 
	                      gd = new MomentumDescent(1, nhidden1, nhidden2, num_attr, eta, alpha);
	                      break;
	            }
	            cout << "Alg. Gradient descent with momentum" << endl;
	            cout << "Learning rate : " << eta << endl;
	            cout << "Momentum coefficient : " << alpha << endl;
	            break;
	  case MOMENTUM_DESCENT_DECR_LIN:
	            switch(nlayers)  {
	              case 1: 
	                      gd = new MomentumDescentDecrEtaLin(1, num_attr, eta, alpha, decrmom);
	                      break;
	              case 2: 
	                      gd = new MomentumDescentDecrEtaLin(1, nhidden2, num_attr, eta, alpha, decrmom);
	                      break;
	              case 3: 
	                      gd = new MomentumDescentDecrEtaLin(1, nhidden1, nhidden2, num_attr, eta, alpha,
	                      decrmom);
	                      break;
	            }
	            cout << "Alg. Momentum descent linear decrement " << endl;
	            cout << "Learning rate : " << eta << endl;
	            cout << "Momentum coefficient : " << alpha << endl;
	            cout << "Linear decrement rate : " << decrmom << endl;
	            break;
	  case MOMENTUM_DESCENT_DECR_EXP:
	            switch(nlayers)  {
	              case 1: 
	                      gd = new MomentumDescentDecrEtaExp(1, num_attr, eta, alpha, decrmom);
	                      break;
	              case 2: 
	                      gd = new MomentumDescentDecrEtaExp(1, nhidden2, num_attr, eta, alpha, decrmom);
	                      break;
	              case 3: 
	                      gd = new MomentumDescentDecrEtaExp(1, nhidden1, nhidden2, num_attr, eta, alpha,
	                      decrmom);
	                      break;
	            }
	            cout << "Alg. Momentum descent decremento esponenziale" << endl;
	            cout << "Learning rate : " << eta << endl;
	            cout << "Momentum coefficient : " << alpha << endl;
	            cout << "Exponential decrement rate : " << decrmom << endl;
	            break;
	  case BOLD_DRIVER: 
	            gd = new BoldDriver(eta, incr, decr);
	            cout << "Alg. Bold Driver" << endl;
	            cout << "Initial learning rate : " << eta << endl;
	            cout << "Increment rate : " << incr << endl;
	            cout << "Decrement rate : " << decr << endl;
	            break;
	  case BOLD_DRIVER_MOMENTUM:
	            switch(nlayers)  {
	              case 1: 
	                      gd = new BoldMomentum(1, num_attr, eta, alpha, incr, decr);
	                      break;
	              case 2: 
	                      gd = new BoldMomentum(1, nhidden2, num_attr, eta, alpha, incr, decr);
	                      break;
	              case 3: 
	                      gd = new BoldMomentum(1, nhidden1, nhidden2, num_attr, eta, alpha, incr, decr);
	                      break;
	            }
	            cout << "Alg. Bold Driver with momentum" << endl;
	            cout << "Initial learning rate : " << eta << endl;
	            cout << "Momentum coefficient : " << alpha << endl;
	            cout << "Increment rate : " << incr << endl;
	            cout << "Decrement rate : " << decr << endl;
	            break;
	  default:  cout << "Not known learning algorithm" << endl;
	            exit(1);
	};
  LearnType::learn_mode mode;
  if (learnmode == BYPATT) {   
     mode = LearnType::BYPATTERN;
     cout << "By pattern learning." << endl;
  }
  else {
     mode = LearnType::BYEPOCH; 
     cout << "By epoch learning." << endl;
  }    
  LearnType learninfo(gd, maxits, errtol, mode);
  start = clock();
  switch (decomptype) {
	  case OPC: poliOPC->init_weights_norm_poli(seed);
	            bigerror = poliOPC->LearnPoli (trainset, learninfo, b_printout);
	            conf = poliOPC->test(testset);
	            end = clock();
	            if (saveout)
	              poliOPC->save_out(testset, outfile);		          
	            if (saverrflag)
	              poliOPC->save_errors(saverr);
	            if (savenet)
	              poliOPC->SavePoli (savepoli);	            
	            print_conf_matrix(conf, cout);
	            poliOPC->print_errors();
							print_elapsed_cpu_time(start, end);
	            delete poliOPC;
	            break;
	  case PWC: poliPWC->init_weights_norm_poli(seed);
	             bigerror = poliPWC->LearnPoli (trainset, learninfo, b_printout);
	            conf = poliPWC->test(testset);
	            end = clock();
	            if (saveout)
	              poliPWC->save_out(testset, outfile);
	            if (saverrflag)
	              poliPWC->save_errors(saverr);
	            if (savenet)
	              poliPWC->SavePoli (savepoli);	            
	            print_conf_matrix(conf, cout);
	            poliPWC->print_errors();
							print_elapsed_cpu_time(start, end);
	            delete poliPWC;
	            break;
	  case CC:  poliCC->init_weights_norm_poli(seed);
	             bigerror = poliCC->LearnPoli (trainset, learninfo, b_printout);
	            conf = poliCC->test(testset);
	            end = clock();
	            if (saveout)
	              poliCC->save_out(testset, outfile);
	            if (saverrflag)
	              poliCC->save_errors(saverr); 
	            if (savenet)
	              poliCC->SavePoli (savepoli);
	            print_conf_matrix(conf, cout);
	            poliCC->print_errors();
							print_elapsed_cpu_time(start, end);
	            delete poliCC;
	            break;
	  case ECOC_ex:  
	            poliECOC_ex->init_weights_norm_poli(seed);
	             bigerror = poliECOC_ex->LearnPoli (trainset, learninfo, b_printout);
	            conf = poliECOC_ex->test(testset);
	            end = clock();
	            if (saveout)
	              poliECOC_ex->save_out(testset, outfile);
	            if (saverrflag)
	              poliECOC_ex->save_errors(saverr);  
	            if (savenet)
	              poliECOC_ex->SavePoli (savepoli);
	            print_conf_matrix(conf, cout);
	            poliECOC_ex->print_errors();
							print_elapsed_cpu_time(start, end);
	            delete poliECOC_ex;
	            break;
	  case ECOC_BCH:  
	            poliECOC_BCH->init_weights_norm_poli(seed);
	            bigerror = poliECOC_BCH->LearnPoli (trainset, learninfo, b_printout);
	            conf = poliECOC_BCH->test(testset);
	            end = clock();
	            if (saveout)
	              poliECOC_BCH->save_out(testset, outfile);
	            if (saverrflag)
	              poliECOC_BCH->save_errors(saverr);  
	            if (savenet)
	              poliECOC_BCH->SavePoli (savepoli);
	            print_conf_matrix(conf, cout);
	            poliECOC_BCH->print_errors();
							print_elapsed_cpu_time(start, end);
	            delete poliECOC_BCH;
	            break;
	  default:  cout << "Not known PND" << endl;
	            exit(1);
	}  
  delete gd;
  return bigerror;
}
void print_info_use(void)
{
  cerr << "
  pnd. NEURObjects application for training and testing
       Parallel Non linear Dichotomizers (PND) ensembles.
  Usage:  pnd trainfile [options]
  Options:
  -test string       test file name 
  -decomp string     Decomposition schemes:  
                     OPC        (one per class)  
                     PWC        (pairwise coupling)  
                     CC         (correcting classifiers)
                     ECOC_ex    (Error Correcting Ouput Code - exaustive algorithm)
                     ECOC_BCH   (Error Correcting Ouput Code - BCH algorithm)
  -ndico unsigned    codeword length  
  -nc unsigned       number of the classes 
  -nl unsigned       number of the layers (considering also the input layer):                         
                     1: one layer (no hidden units) 
                     2: two layers (1 hidden layer) 
                     3: three layers (2 hidden layers)                      
  -h1 unsigned       number of hidden units of the first hidden layer of the three layers MLP 
  -h2 unsigned       number of hidden units of the second hidden layer of the three layers MLP or
                     number of hidden units of the two layers MLP (default 4)
  -d  unsigned       sample dimension/number of input units 
  -maxit unsigned    maximal number of iterations of the backpro algorithm  (default 500) 
  -maxerr double     maximal normalized root mean square error (RMS) (default 0.1) 
  -alg string        Backpropagation learning algorithm type:                      
                     gd         (gradient descent)  
                     gd_dl      (gradient descent linear decrement) 
                     gd_de      (gradient descent exponential decrement) 
                     md         (gradient descent with momentum) 
                     md_dl      (gradient descent with momentum and linear decrement) 
                     md_de      (gradient descent with momentum and exponential decrement) 
                     bold       (bold driver) 
                     boldmom    (bold driver with momentum)
  -rate double       learning rate (default 0.07) 
  -mom double        momentum rate (default 0.5) 
  -incbold double    bold driver increment rate 
  -decbold double    bold driver decrement rate 
  -decr double       gradient decrement  (for gradient descent with 
                     momentum and linear or exponential decrement) 
  -seed              seed for the random initialization of the weights. 
                     If 0 the initialization is performed
                     using current computer time.  
  -s string          save pnd parameters and weights onto a file 
  -out string        save pnd ouptuts onto a file 
  -serr string       save pnd history RMS errors onto a file 
  -p integer          1 = printing a point at each iteration; 0 = noprint
	" << endl;
  exit(1);
}
int doarg(char* arg[], int i)
{
  int j;
	
  if (!strcmp(arg[i], "-test"))
    strcpy(testfile, arg[i+1]);
  else if (!strcmp(arg[i], "-decomp")) { 
    if ((decomptype = Analyze_dec(arg[i+1])) >= NUMDEC)
       print_info_use();
  }
  else if (!strcmp(arg[i], "-nc")) {
    nclass = atoi(arg[i+1]);
		if (nclass < 2) error_exit("Classes must be 2 or more");}
	else if (!strcmp(arg[i], "-ndico"))
    ndico = atoi(arg[i+1]);
  else if (!strcmp(arg[i], "-nl")) {
    nlayers = atoi(arg[i+1]);
		if ((nlayers < 1) || (nlayers > 3)) error_exit("Number of layers must be between 1 and 3");}
  else if (!strcmp(arg[i], "-h1"))
    nhidden1 = atoi(arg[i+1]);
  else if (!strcmp(arg[i], "-h2"))
    nhidden2 = atoi(arg[i+1]);
  else if (!strcmp(arg[i], "-d")) {
    num_attr = atoi(arg[i+1]);
		if (num_attr < 1) error_exit("Number of attributes must be 1 or more");}
  else if (!strcmp(arg[i], "-maxit"))
    maxits = atoi(arg[i+1]);
  else if (!strcmp(arg[i], "-maxerr"))
    errtol = atof(arg[i+1]);
  else if (!strcmp(arg[i], "-alg")) { 
    if ((learntype = Analyze_alg(arg[i+1])) >= NUMALG)
       print_info_use();
  }
  else if (!strcmp(arg[i], "-mode")) {
    if ((learnmode = Analyze_mode(arg[i+1])) >= NUMMODE)
       print_info_use();
  }
  else if (!strcmp(arg[i], "-rate"))
    eta = atof(arg[i+1]);
  else if (!strcmp(arg[i], "-mom"))
    alpha = atof(arg[i+1]);
  else if (!strcmp(arg[i], "-incbold"))
    incr = atof(arg[i+1]);
  else if (!strcmp(arg[i], "-decbold"))
    decr = atof(arg[i+1]);
  else if (!strcmp(arg[i], "-decr"))
    decrmom = atof(arg[i+1]);
  else if (!strcmp(arg[i], "-s")) {
    savenet = 1;
    strcpy(savepoli, arg[i+1]);
  }
  else if (!strcmp(arg[i], "-out"))  {
    saveout = 1; 
    strcpy(outfile, arg[i+1]);
  }
  else if (!strcmp(arg[i], "-serr")) {
    saverrflag = 1; 
    strcpy(saverr, arg[i+1]);
  }
	else if (!strcmp(arg[i], "-seed"))
    seed = atoi(arg[i+1]);
  else if (!strcmp(arg[i], "-p")) {
    j = atoi(arg[i+1]);
		if (j == 0)
		   b_printout = false;
	}
	else 
    return 1;
  return 0;
} 
