// "polidico.cc"


#pragma implementation
#include <math.h>
#include <sstream>
#include <time.h>
#include<string.h>
#include"polidico.h"

#define DIMBUFNAMEFILE  128
#define POLI_EXT  ".poli"


template <class SCOMP> PoliDico<SCOMP>::PoliDico (unsigned num_class, DicoType& dico, act_func funact, 
                                                  unsigned num_dico)
: scomp(num_class, num_dico)
{
  n_class = num_class;
  num_errors = num_errorsL2 = num_errorsHD = 0;
  perc_errors = 0;
  funout = funact;  
  Build_dico(dico);
  err = new vect(n_dico);
  iter = new vector<unsigned>(n_dico); 
	vDicoerr = new unsigned[n_dico]; 
}


template <class SCOMP> PoliDico<SCOMP>::PoliDico (DicoType& dico, act_func funact,  char* DMfile)
: scomp(DMfile)
{
  n_class = scomp.Read_n_class();
	n_dico = scomp.Read_n_dico();
  num_errors = num_errorsL2 = num_errorsHD = 0;
  perc_errors = 0;
  funout = funact;  
  Build_dico(dico);
  err = new vect(n_dico);
  iter = new vector<unsigned>(n_dico);
	vDicoerr = new unsigned[n_dico];
}


template <class SCOMP> void PoliDico<SCOMP>::Build_dico(DicoType& dico)
{
  unsigned i;
  matrix& D = scomp.Read_matrix();
  unsigned nin = dico.n_neuro[0];
  unsigned nhid2 = dico.n_neuro[1];
  unsigned nhid1 = dico.n_neuro[2];
  n_dico = scomp.Read_n_dico();
  vDico = new Dico1*[n_dico]; 
  switch (dico.n_layers)  {
    case 1:  for (i=0; i<n_dico; i++)
               vDico[i] = new Dico1(nin, D.row(i), funout);
             break;
    case 2:  for (i=0; i<n_dico; i++)
               vDico[i] = new Dico2(nhid2, nin, D.row(i), funout, dico.type_fun[1]);
             break;
    case 3:  for (i=0; i<n_dico; i++)
               vDico[i] = new Dico3(nhid1, nhid2, nin, D.row(i), funout,
                                    dico.type_fun[1], dico.type_fun[2]);
             break;
    default: error ("template <class SCOMP> PoliDico<SCOMP>::Build_dico", "error");
             break;
  }   
}
template <class SCOMP> PoliDico<SCOMP>::~PoliDico()
{
  unsigned i;
  for (i=0; i<n_dico; i++)  
    delete vDico[i];
  delete [] vDico;
	// ATTENZIONE qui forse ci sono cadaveri 
	if (err!=0);
    delete err;
	if (!iter->empty())
	  delete iter;
	delete [] vDicoerr;
}  
template <class SCOMP> void  PoliDico<SCOMP>::print_info(void)
{ 
  int i;
  cout << "Output Coding ensemble features" << endl;
  cout << "Number of classes " << n_class << endl;
  cout << "Number of dichotomizers " << n_dico << endl << endl;
	cout << "Decopmosition matrix:" << endl;
	scomp.print_matrix();
  cout << "Dichotomizers features:" << endl;
  for (i=0; i<n_dico; i++)   {
    cout << "Dichotomizer n. " << i+1 << "*********************" << endl;
    vDico[i]->print_weights();
  }
}
template <class SCOMP> int PoliDico<SCOMP>::LearnPoli (TrainingSet& tset, LearnType& learninfo, 
                       bool printout)
{
  unsigned i;
  unsigned rep;
  const unsigned NUM_TRAIL = 1;
  const double MAX_ERR = 0.30;
  const double K = 4.0;
  int running_seed;
  unsigned n_train = tset.Ntrain();
  unsigned iter_dico;
  iter_tot = 0;   
  int bigerror = 0;  
  double rate = (learninfo.learn)->Read_rate();   
  if (learninfo.mode == LearnType::BYPATTERN)
    for (i=0; i<n_dico; i++)  {
      cout << "Dichotomizer n. " << i+1 << endl;
      (*iter)[i] = (unsigned) 0;
      (*err)[i] = 0.0; 			
      rep = 0;			
      running_seed = vDico[i]->Read_seed();
			(learninfo.learn)->Init_rate(rate);   
      do { 
				vDico[i]->Learn_by_pattern(tset, *(learninfo.learn), iter_dico, (*err)[i], 
                                	 learninfo.err_tol, learninfo.maxits, printout);
        (*iter)[i] += iter_dico;
      	print_dico_err_iteraz_info(i, iter_dico);
      	if (((*err)[i] > (K*learninfo.err_tol)) && ((*err)[i] > MAX_ERR))  {
        	running_seed = (int) -time(NULL);
        	vDico[i]->init_weights_norm(running_seed);
					rep++;
					(learninfo.learn)->Init_rate(rate);
      	}
      	else {
        	break; 
        	--rep;
        }
        if (rep <NUM_TRAIL) {
          cout << "Too much error : Reinitialization dichotomizer n. " << i+1 << endl;
					cerr << "Too much error : Reinitialization dichotomizer n. " << i+1 << endl;
				}
      } while (rep < NUM_TRAIL);
      iter_tot += (*iter)[i];
      if (rep>=NUM_TRAIL)
        bigerror = 2;
    } 
  else if (learninfo.mode == LearnType::BYEPOCH)
    for (i=0; i<n_dico; i++)  {
      cout << "Dichotomizer n. " << i+1 << endl;
      (*iter)[i] = (unsigned) 0;
      (*err)[i] = 0.0;    
      rep = 0;
      running_seed = vDico[i]->Read_seed();
			(learninfo.learn)->Init_rate(rate);   
      do { 
      	vDico[i]->Learn_by_epoch(tset, *(learninfo.learn), iter_dico, (*err)[i], 
                                	 learninfo.err_tol, learninfo.maxits, printout);
      	(*iter)[i] += iter_dico;
      	print_dico_err_iteraz_info(i, iter_dico);
      	if (((*err)[i] > (K*learninfo.err_tol)) && ((*err)[i] > MAX_ERR))    {
        	running_seed += 100;
        	vDico[i]->init_weights_norm(running_seed);
					rep++;
					(learninfo.learn)->Init_rate(rate);
      	}
      	else { 
        	break; 
        	--rep;
        }
        if (rep <NUM_TRAIL) {
          cout << "Too much error : Reinitialization dichotomizer n. " << i+1 << endl;
					cerr << "Too much error : Reinitialization dichotomizer n. " << i+1 << endl;
				}
      } while (rep < NUM_TRAIL);
      iter_tot += (*iter)[i];   
      if (rep>=NUM_TRAIL)
         bigerror = 2;
    } 
  else
    error_exit("PoliDico<SCOMP>::LearnPoli", "Learning mode not known");
  (learninfo.learn)->Init_rate(rate);   
	return bigerror;
}   
template <class SCOMP> void PoliDico<SCOMP>::print_dico_err_iteraz_info(unsigned n, unsigned iter_dico)
{
  cout << "Iterations : " << iter_dico << endl;
  cout.precision(4);
  cout << "RMS Error: " << (*err)[n] << endl;
}
template <class SCOMP> void PoliDico<SCOMP>::save_errors(char* name)
{
  unsigned i;
  char namebuf[256];
  int len = strlen(name);
  ostrstream ost(namebuf,256);
	ost << name;
  for (i=0; i<n_dico; i++)  {
    ost.seekp(len);  ost << i+1 << '\0';    
    vDico[i]->save_errors(namebuf);
  }
}
template <class SCOMP> matrix PoliDico<SCOMP>::test(TrainingSet& tset)
{
  unsigned i, k;
  double dist;   
  num_errors = num_errorsL2 = num_errorsHD = 0;
  unsigned num_train = tset.Ntrain();
  vect& t = tset.Read_target();  
  matrix& x = tset.Read_data(); 
	double threshold = vDico[0]->Calc_treshold();    
  vect f(n_dico); 
  matrix DM(n_dico, n_class); 
  TransformMatrix(DM);
  matrix conf(n_class,n_class,0.0);   
  for (i = 0; i < num_train; i++) {
    vect& input = x.row(i);
		// Computes class according to L1 norm      
    k = Calc_class(f, DM, input, dist);    
    if (k!=((int)t[i]))  {
      num_errors++;
    }
    conf(k-1, (int)t[i]-1)+=1; 
		// Computes class according to Hamming distance 
		k = Calc_class_Hamming(f, DM, input, dist);    
    if (k!=((int)t[i]))  {
      num_errorsHD++;
    }
		// Computes class according to L2 norm distance 
		k = Calc_class_L2Norm(f, DM, input, dist);    
    if (k!=((int)t[i]))  {
      num_errorsL2++;
    } 
		// Computes the errors of each dichotomizer
		Calc_dico_errors(f, DM, (unsigned)t[i], threshold);  
  }
  perc_errors = (double)num_errors/(double)num_train;
  return conf;  
}

template <class SCOMP> unsigned PoliDico<SCOMP>::Calc_class(vect& f, matrix& DM, vect& input, double& dist)
{
  unsigned n,k;
  dist = 100000;
  double val;
  unsigned cl = 0;
  for (n = 0; n < n_dico; n++) {       
    vect& out = vDico[n]->compute(input);   
    f[n] = out[0];   
  }
  for (k = 0; k < n_class; k++) {   
    val = NormL1(f, DM.col(k));
      if (val < dist)  {
        dist = val;
        cl = k;
      } 
  }
  return (cl+1);   
}

template <class SCOMP> unsigned PoliDico<SCOMP>::Calc_class_L2Norm(vect& f, matrix& DM, vect& input, double& dist) 
{                 
  unsigned n,k;
  dist = 100000;
  double val;
  unsigned cl = 0;
  for (n = 0; n < n_dico; n++) {       
    vect& out = vDico[n]->compute(input);   
    f[n] = out[0];   
  }
  for (k = 0; k < n_class; k++) {   
    val = NormL2(f, DM.col(k));
      if (val < dist)  {
        dist = val;
        cl = k;
      } 
  }
  return (cl+1);   
}

template <class SCOMP> unsigned PoliDico<SCOMP>::Calc_class_Hamming(vect& f, matrix& DM, vect& input, double& dist)    
{                                                
  unsigned n,k;
  dist = 100000;
  double val;
  unsigned cl = 0;
  for (n = 0; n < n_dico; n++) {       
    vect& out = vDico[n]->compute(input);   
    f[n] = out[0];   
  }
  for (k = 0; k < n_class; k++) {   
    val = Hamming(f, DM.col(k));
      if (val < dist)  {
        dist = val;
        cl = k;
      } 
  }
  return (cl+1);   
}



template <class SCOMP> void PoliDico<SCOMP>::Calc_dico_errors(vect& f, matrix& DM, unsigned targetclass, double threshold)
{
  vect out(1);
	for (int i=0; i < n_dico; i++) {
	  out[0] = f[i];
		if (vDico[i]->bin_class(out,DM(i,targetclass-1), threshold) == Dico1::NOTOK)
		  vDicoerr[i]++;
	}
}


template <class SCOMP> void PoliDico<SCOMP>::TransformMatrix(matrix& m)
{
  unsigned i,j,type_fun;
  matrix& D = scomp.Read_matrix();  
  type_fun = vDico[0]->Code_act_fun(funout);
  switch (type_fun)  {
    case SIGMOID:
           for (i=0; i<n_dico; i++)
             for (j=0; j<n_class; j++)
            	 switch((int)D(i,j)) {
              	 case d_OPC::HIGH:   m(i,j) = OneLayer::SIG_ON; break;
              	 case d_OPC::LOW:    m(i,j) = OneLayer::SIG_OFF; break;
              	 case d_OPC::IGNORE: m(i,j) = OneLayer::SIG_TRESHOLD; break;
            	 }              
           break;
    case TANHYP:
           for (i=0; i<n_dico; i++)
             for (j=0; j<n_class; j++)
            	 switch((int)D(i,j)) {
              	 case d_OPC::HIGH:   m(i,j) = OneLayer::TANH_ON; break;
              	 case d_OPC::LOW:    m(i,j) = OneLayer::TANH_OFF; break;
              	 case d_OPC::IGNORE: m(i,j) = OneLayer::TANH_TRESHOLD; break;
            	 }  
           break;
    case IDENTITY:
           for (i=0; i<n_dico; i++)
             for (j=0; j<n_class; j++)
            	 switch((int)D(i,j)) {
              	 case d_OPC::HIGH:   m(i,j) = OneLayer::ID_ON; break;
              	 case d_OPC::LOW:    m(i,j) = OneLayer::ID_OFF; break;
              	 case d_OPC::IGNORE: m(i,j) = OneLayer::ID_TRESHOLD; break;
            	 }  
           break;
    default:
           error ("Not known activation function");
           break;
  }
}  
template <class SCOMP> void PoliDico<SCOMP>::save_decomp_matrix(ofstream& f)
{
  unsigned i, j;
  matrix DM(n_dico, n_class); 
  TransformMatrix(DM);
  f << n_dico << "  " << n_class  << endl << endl;
  f.setf(ios::fixed, ios::floatfield);
  f.setf(ios::showpos);
  f.precision(8); 
  for (i=0; i<n_dico; i++)  {
    for (j=0; j<n_class; j++) 
      f << DM(i,j) << " ";
    f << endl;
  }
  f << endl;
  f << flush;
  if (f.bad())
    error("Error writing decomposition matrix");
}
template <class SCOMP> void PoliDico<SCOMP>::save_out(TrainingSet& tset, ofstream& f)
{
  unsigned i, j;  
  double dist;
  char buf[128];  
  unsigned num_train = tset.Ntrain();
	unsigned cl;  
	
  vect& t = tset.Read_target();  
  matrix& x = tset.Read_data();     
  vect v(n_dico);  
  matrix DM(n_dico, n_class); 
  TransformMatrix(DM);
  f.setf(ios::fixed, ios::floatfield);
  f.setf(ios::showpos);
  f.precision(8); 
  for (i = 0; i < num_train; i++) {
    vect& input = x.row(i);      
    cl = Calc_class(v, DM, input, dist);
    for (j=0; j<n_dico; j++) 
      f << v[j] << " ";
    f << cl << " ";
		f << (unsigned)t[i] << endl;
  }
  f << flush;
  if (f.bad())
    error("Errore in scrittura del file di output del policotomizzatore ", buf);
}  
template <class SCOMP> void PoliDico<SCOMP>::save_out(TrainingSet& tset, char* outfile)
{
  unsigned i, j; 
	unsigned k = 0; 
  double dist;
  char buf[128];  
  unsigned num_train = tset.Ntrain();
  vect& t = tset.Read_target();  
  matrix& x = tset.Read_data();     
  vect v(n_dico);  
  matrix DM(n_dico, n_class); 
  TransformMatrix(DM);
  strcpy(buf, outfile);
  strcat(buf, POUT_EXT);  
  ofstream f(buf); 
  if (!f) error("Non riesco ad aprire il file di output del policotomizzatore", buf);
  f << n_dico << "  " << n_class  << endl << endl;
  f.setf(ios::fixed, ios::floatfield);
  f.setf(ios::showpos);
  f.precision(8); 
  for (i=0; i<n_dico; i++)  {
    for (j=0; j<n_class; j++) 
      f << DM(i,j) << " ";
    f << endl;
  }
  f << endl;
  for (i = 0; i < num_train; i++) {
    vect& input = x.row(i);      
    k = Calc_class(v, DM, input, dist);
    for (j=0; j<n_dico; j++) 
      f << v[j] << " ";
    f << (unsigned)t[i] << " " << k << endl;
  }
  f << flush;
  if (f.bad())
    error("Errore writing file ", buf);
}  
template <class SCOMP> void PoliDico<SCOMP>::SavePoli (char *namefile)
{ 
  unsigned i;
  char namebuf[128 ];
  strcpy(namebuf, namefile);
  strcat(namebuf, ".poli" );
  ofstream f(namebuf);
  if (!f) error("Cannot open file ", namebuf);
  f.setf(ios::left, ios::adjustfield);
  f << "Version:  " << 1 << endl;
  f << namebuf << endl << endl;
  f << "Number_class: " << n_class << endl;
  f << "Number_dico: " << n_dico << endl;
  f << "Decomp_matrix:" << endl;
  f << scomp.Read_matrix() << endl;
  for (i=0; i<n_dico; i++)  {
    f << "Dichotomizer n. " << i+1 << endl;
    vDico[i]->save_weights(f);
  }
  f << END << flush;
  if (f.bad())
    error("Error writing file ", namebuf);
}
template <class SCOMP> int PoliDico<SCOMP>::CrossValidate(char *name, unsigned nfold, unsigned num_attr, 
                                                           char *filerr, LearnType& learninfo, 
                                                           char* outfile, int saveout , 
							   int seed, bool printout)
{
  unsigned n, num_train[2];
  vector<unsigned> itercv(nfold);   
  
	abserr = new unsigned[nfold];
	abserrL2 = new unsigned[nfold];
	abserrHD = new unsigned[nfold];
	percerr = new float[nfold];  
	Init_vDicoerr();      
  TSetStrClass* trainset, *testset;
  const unsigned DIMBUF = 128;
  char outbuf[DIMBUF];  
  char namebuf[DIMBUF];
  ostrstream ost(namebuf,DIMBUF);    
  ofstream *f;
  matrix conf(n_class, n_class, 0.0);   
  matrix sumconf(n_class, n_class, 0.0);   
	int maxits_reached = 0;   
	                          
	int bigerror;
	clock_t start, end;
	start = clock(); 
  if (saveout)  {
    strcpy(outbuf, outfile);
    strcat(outbuf, POUT_EXT);  
    f = new ofstream(outbuf,ios::out|ios::app);   
    if (!*f) error("Cannot open file ", outbuf);
    save_decomp_matrix(*f);
  }
  for (n=0; n<nfold; n++)  {
    cout << endl << "Fold n. " << n+1 << " *********************" << endl;
		 
    init_weights_norm_poli(seed); 
		 
    ost.seekp(0);
    ost << name << ".f" << n+1 << FOLD_TRAIN_SUFFIX << '\0';
    num_train[0] = wc (namebuf); 
		trainset = new TSetStrClass(num_attr, num_train[0],  namebuf) ; 
		trainset->normalize();
		if (trainset->read_dict().size() != n_class)
	   error_exit("Error: Number of classes does not match between input file and parameter");
		 
    bigerror = LearnPoli (*trainset, learninfo, printout);
		if (bigerror == 2)
			maxits_reached = 2;
    itercv[n] = iter_tot;  
		cout << endl << "Testing fold n. " << n+1 << endl;
		 
    ost.seekp(0);
    ost << name << ".f" << n+1 << FOLD_TEST_SUFFIX << '\0';
    num_train[1] = wc (namebuf); 
		testset = new TSetStrClass(num_attr, num_train[1],  namebuf, trainset->read_dict());  
    testset->normalize(trainset->Read_mean(), trainset->Read_stdev()); 
		 
    conf = test(*testset); 
    SumConf(sumconf, conf);     
    abserr[n] = num_errors;
    percerr[n] = perc_errors;
		abserrL2[n] = num_errorsL2;
		abserrHD[n] = num_errorsHD;
    print_errors();
    if (saveout) 
      save_out(*testset, *f);
    delete trainset;
		delete testset;	
  } 
	end = clock(); 
  SaveCV (filerr, itercv, num_train[0]+num_train[1], sumconf, nfold, start, end);
  print_conf_matrix(sumconf, cout);
  print_resultCV (itercv, num_train[0]+num_train[1], nfold, start, end);                                                             
  cout << flush;
  if (saveout) {
    (*f).close();
    delete f;
  }
	
	delete [] abserr;
	delete [] abserrL2;
	delete [] abserrHD;
	delete [] percerr;      
	return maxits_reached;
} 



template <class SCOMP> void PoliDico<SCOMP>::PrintFoldResults (ostream& out, unsigned nfold, vector<unsigned>& itercv){

  out << "Folds number: " << nfold << endl;
  out.setf(ios::fixed, ios::floatfield);
  out.setf(ios::left, ios::adjustfield);
  out.precision(6);
  for (int i=0; i<nfold; i++)  {
    out << "Fold n. " << (unsigned)i+1 << endl;
    out << "Iterations: " << SEP << itercv[i] << SEP << SEP;
    out << "Errors: " << SEP << (unsigned)abserr[i] << SEP << SEP;
    out << SEP << "Error rate : " << percerr[i] << endl;
  }
}


template <class SCOMP> void PoliDico<SCOMP>::Calc_total_cv_errors (unsigned nfold, vector<unsigned>& itercv, unsigned& itertot, float& errtot, 
                                                               float& errtotL2, float& errtotHD){
  for (int i=0; i<nfold; i++)  {    
    itertot += itercv[i];
    errtot += abserr[i];
		errtotL2 += abserrL2[i];
		errtotHD += abserrHD[i];
  }
}


template <class SCOMP> float PoliDico<SCOMP>::Calc_cv_stdev (int nfold, int ntrain, float err_rate, unsigned* fold_tot_err){
  
  float x;
	float stdev = 0;
	for (int i=0; i<nfold; i++)  {	  
    x = (((float)fold_tot_err[i]/ntrain) * nfold) - err_rate;
    stdev += (x*x);
  }
  return (stdev = sqrt(stdev/(nfold-1)));
}

template <class SCOMP> void PoliDico<SCOMP>::Calc_cv_dico_accuracy (int ntrain, float& minerrordico, float& maxerrordico, 
                                                                float& avgerrordico, float& stdverrordico) { 
  float x, errtotdico = 0;
	avgerrordico = stdverrordico = 0;
	minerrordico = maxerrordico = vDicoerr[0];
	for(int i=0; i <n_dico; i++) {	 
	  errtotdico += vDicoerr[i]; 
		if (vDicoerr[i] < minerrordico)
		  minerrordico = vDicoerr[i];
		if (vDicoerr[i] > maxerrordico)	
		  maxerrordico = vDicoerr[i];			
	}
	avgerrordico = (float)errtotdico/(ntrain*n_dico);
	maxerrordico = maxerrordico/ntrain;
	minerrordico = minerrordico/ntrain;
	for (int i=0; i<n_dico; i++)  {
    x = ((float)vDicoerr[i]/ntrain) - avgerrordico;
    stdverrordico += (x*x);
  }
	stdverrordico = sqrt(stdverrordico/(n_dico-1));
}

template <class SCOMP> void PoliDico<SCOMP>::Calc_HD (float& MHD, float& AHD, float& ERP, float& MDD, float& ADD){
  
	MHD = scomp.MinHamming();
	AHD = scomp.Avg_Hamming_distance();
	ERP = MHD/n_dico;
	MDD = scomp.Min_Dichotomy_distance();
	ADD = scomp.Avg_Dichotomy_distance();
}




template <class SCOMP> void PoliDico<SCOMP>::SaveCV (char *name, vector<unsigned>& itercv, 
                     unsigned ntrain, matrix& conf, unsigned nfold, clock_t start, clock_t end)
{
	const unsigned WIDTH = 8;
  unsigned itertot = 0;
  float errtot, errtotL2, errtotHD;
  float percerrtot, percerrtotL2, percerrtotHD;
  float stdev, stdevL2, stdevHD;
	errtot = errtotL2 = errtotHD = 0;
	percerrtot = percerrtotL2 = percerrtotHD = 0;
	stdev = stdevL2 = stdevHD = 0;
	float avgerrordico, minerrordico, maxerrordico, stdverrordico;
	avgerrordico = minerrordico = maxerrordico = stdverrordico = 0;
  float x;
  const unsigned DIMBUF = 128;  
  char namebuf[DIMBUF];
	
	// opening file
  ostrstream ost(namebuf,DIMBUF);
  ost << name << CV_SUFFIX << '\0'; 
  ofstream fcv(namebuf);
  if (!fcv) error("Cannot open file ", namebuf); 
	
	// Save CV results
	PrintFoldResults (fcv, nfold, itercv); 
	// Computes total number of errors on all the folds
	Calc_total_cv_errors (nfold, itercv, itertot, errtot, errtotL2, errtotHD);
  
	// Computes the error and stdev using L1, L2 norm and Hamming distance
	percerrtot = (errtot/ntrain);
	percerrtotL2 = (errtotL2/ntrain);
	percerrtotHD = (errtotHD/ntrain);
	stdev = Calc_cv_stdev (nfold, ntrain, percerrtot, abserr);
  stdevL2 = Calc_cv_stdev (nfold, ntrain, percerrtotL2, abserrL2);
	stdevHD = Calc_cv_stdev (nfold, ntrain, percerrtotHD, abserrHD);
	
	
	// Computes accuracy of the dichotomizers
	Calc_cv_dico_accuracy (ntrain, minerrordico, maxerrordico, avgerrordico, stdverrordico);
	
	//Computes the Minimum and Average Hamming distance and the error recovering power of the decomposition
	// and the minumum and average dichotomy distance
	float MHD, AHD, ERP, MDD, ADD;
	Calc_HD (MHD, AHD, ERP, MDD, ADD);
			
	// printing onto a file the results
  print_conf_matrix(conf, fcv);
	fcv.setf(ios::left, ios::adjustfield);
  fcv << "Results summary:" << endl;
	fcv << "Error recovering power of the decomposition  : " << ERP << endl;
	fcv << "Minimum Hamming distance between codewords   : " << MHD << endl;
	fcv << "Average Hamming distance between codewords   : " << AHD << endl;
	fcv << "Minimum dichotomy distance between codewords : " << MDD << endl;
	fcv << "Average dichotomy distance between codewords : " << ADD << endl;
  fcv << "Iterations : " << SEP << setw(WIDTH) << itertot << endl;
  fcv << "Errors L1 norm: " << SEP << setw(WIDTH) <<(unsigned)errtot << " : ";
  fcv << "Error rate : " << SEP << setw(WIDTH) << percerrtot << SEP << " : ";
	fcv << "St.dev : " << SEP << setw(WIDTH) << stdev << endl;
	fcv << "Errors L2 norm: " << SEP << setw(WIDTH) <<(unsigned)errtotL2 << " : ";
  fcv << "Error rate : " << SEP << setw(WIDTH) << percerrtotL2 << SEP << " : ";
	fcv << "St.dev : " << SEP << setw(WIDTH) << stdevL2 << endl;
	fcv << "Errors Hamm. dist.: " << SEP << setw(WIDTH) <<(unsigned)errtotHD << " : ";
  fcv << "Error rate : " << SEP << setw(WIDTH) << percerrtotHD << SEP << " : ";
	fcv << "St.dev : " << SEP << setw(WIDTH) << stdevHD << endl;
	
	fcv << "Errors of the dichotomizers: " << endl;
	fcv << "Avg. error : " << setw(WIDTH) << avgerrordico << endl;
	fcv << "Min. error : " << setw(WIDTH) << minerrordico << endl;
	fcv << "Max. error : " << setw(WIDTH) << maxerrordico << endl;
	fcv << "Stdev : " << setw(WIDTH) << stdverrordico << endl << endl;
	
  print_elapsed_cpu_time(start, end, fcv);
	 
  if (fcv.bad())
    error("Error writing file ", namebuf);
  fcv << flush;
  
  // opening tail file
  ostrstream ost2(namebuf,DIMBUF);
  ost2 << name << ".tail" << '\0'; 
  ofstream ft(namebuf);
  if (!ft) error("Cannot open file ", namebuf); 
  
  ft << n_class << endl;
  ft << n_dico << endl;
  ft << ERP << endl;
  ft << MHD << endl;
  ft << AHD << endl;
  ft << MDD << endl;
  ft << ADD << endl;
  ft << percerrtot << endl;
  ft << stdev << endl;
  ft << percerrtotL2 << endl;
  ft << stdevL2 << endl;
  ft << percerrtotHD << endl;
  ft << stdevHD << endl;
  ft << avgerrordico << endl;
  ft << minerrordico << endl;
  ft << maxerrordico << endl;
  ft << stdverrordico << endl;
  print_elapsed_cpu_time(start, end, ft);
  
  if (ft.bad())
    error("Error writing file ", namebuf);
  ft << flush;
}  

template <class SCOMP> void PoliDico<SCOMP>::print_resultCV (vector<unsigned>& itercv, 
                                   unsigned ntrain, unsigned nfold, clock_t start, clock_t end) 
{ 
  int i;
	const unsigned WIDTH = 8;
  unsigned itertot = 0;
  float errtot, errtotL2, errtotHD;
  float percerrtot, percerrtotL2, percerrtotHD;
  float stdev, stdevL2, stdevHD;
	errtot = errtotL2 = errtotHD = 0;
	percerrtot = percerrtotL2 = percerrtotHD = 0;
	stdev = stdevL2 = stdevHD = 0;
	float avgerrordico, minerrordico, maxerrordico, stdverrordico;
	avgerrordico = minerrordico = maxerrordico = stdverrordico = 0;
  float x;
	
	/// Save CV results
	PrintFoldResults (cout, nfold, itercv); 
	// Computes total number of errors on all the folds
	Calc_total_cv_errors (nfold, itercv, itertot, errtot, errtotL2, errtotHD);
	
	// Computes the error and stdev using L1, L2 norm and Hamming distance
	percerrtot = (errtot/ntrain);
	percerrtotL2 = (errtotL2/ntrain);
	percerrtotHD = (errtotHD/ntrain);
	stdev = Calc_cv_stdev (nfold, ntrain, percerrtot, abserr);
  stdevL2 = Calc_cv_stdev (nfold, ntrain, percerrtotL2, abserrL2);
	stdevHD = Calc_cv_stdev (nfold, ntrain, percerrtotHD, abserrHD);
	
	
	// Computes accuracy of the dichotomizers
	Calc_cv_dico_accuracy (ntrain, minerrordico, maxerrordico, avgerrordico, stdverrordico);
	
	//Computes the Minimum and Average Hamming distance and the error recovering power of the decomposition
	// and the minumum and average dichotomy distance
	float MHD, AHD, ERP, MDD, ADD;
	Calc_HD (MHD, AHD, ERP, MDD, ADD);
			
	// printing onto a file the results
	cout.setf(ios::left, ios::adjustfield);
  cout << "Results summary:" << endl;
	cout << "Error recovering power of the decomposition : " << ERP << endl;
	cout << "Minimum Hamming distance between codewords : " << MHD << endl;
	cout << "Average Hamming distance between codewords : " << AHD << endl;
	cout << "Minimum dichotomy distance between codewords : " << MDD << endl;
	cout << "Average dichotomy distance between codewords : " << ADD << endl;
  cout << "Iterations : " << SEP << setw(WIDTH) << itertot << endl;
  cout << "Errors L1 norm: " << SEP << setw(WIDTH) <<(unsigned)errtot << " : ";
  cout << "Error rate : " << SEP << setw(WIDTH) << percerrtot << SEP << " : ";
	cout << "St.dev : " << SEP << setw(WIDTH) << stdev << endl;
	cout << "Errors L2 norm: " << SEP << setw(WIDTH) <<(unsigned)errtotL2 << " : ";
  cout << "Error rate : " << SEP << setw(WIDTH) << percerrtotL2 << SEP << " : ";
	cout << "St.dev : " << SEP << setw(WIDTH) << stdevL2 << endl;
	cout << "Errors Hamm. dist.: " << SEP << setw(WIDTH) <<(unsigned)errtotHD << " : ";
  cout << "Error rate : " << SEP << setw(WIDTH) << percerrtotHD << SEP << " : ";
	cout << "St.dev : " << SEP << setw(WIDTH) << stdevHD << endl;
	
	cout << "Errors of the dichotomizers: " << endl;
	cout << "Avg. error : " << setw(WIDTH) << avgerrordico << endl;
	cout << "Min. error : " << setw(WIDTH) << minerrordico << endl;
	cout << "Max. error : " << setw(WIDTH) << maxerrordico << endl;
	cout << "Stdev : " << setw(WIDTH) << stdverrordico << endl << endl;
	
  print_elapsed_cpu_time(start, end, cout);
	 
  cout << flush;
}  


template <class SCOMP> void PoliDico<SCOMP>::Init_vDicoerr(void) {
  for (int i=0; i<n_dico; i++)
	  vDicoerr[i] = 0;
} 


/*
Instantiation of templates is forced. You need to add a new line
template class PoliDico<new_type>;
for each new type to be instantiated
In the makefile the file pnd.cc must be compiled with -fno-implicit-templates
*/

// Force instantiation of templates (explicit instantiation)
template class PoliDico<d_OPC>;
template class PoliDico<d_PWC>;
template class PoliDico<d_CC>;
template class PoliDico<d_ECOC_ex>;
template class PoliDico<d_ECOC_BCH>;
template class PoliDico<d_Rand>;

/*
typedef PoliDico<d_OPC> p1;
typedef PoliDico<d_PWC> p2;
typedef PoliDico<d_CC> p3;
typedef PoliDico<d_ECOC_ex> p4;
typedef PoliDico<d_ECOC_BCH> p5;
typedef PoliDico<d_Rand> p6;
*/
