/* ******************************************************************************
NEURObjects 2.04
rabdomsubspace.h May 2004
Giorgio Valentini
****************************************************************************** */
#ifndef RANDOMSUBSPACE_H
#define RANDOMSUBSPACE_H

#include <iostream>
#include <fstream>
#include <string>
#include <deque>
#include "matrix.h"
#include "tset.h"
#include "baselearner.h"
#include "bagging.h"

using namespace std;


/**
   Random subspace ensemble class.\\
	 This class implements the general Ho's learning algorithm of random subspace, selecting a class by majority voting.
	 The base learners must belong to a derived class of BaseLearner. A pointer to a specific base learner (base) is passed to the
	 constructor: each new added base learner will be a copy  of base. 
	 Random subspace data sets must be stored in files in advance. The  data sets are stored in files consecutively
	 numbered (the number are the last character of the common name of the data sets (that is name1, name2, ... , namen).
	 This class permits to easily merge different bagged ensemble, and it allows to use different base learners 
	 (as long as the base learners are derived classes of the abstract virtual base class Baselearner) 
*/ 
class RandomSubspace : public Bagging{

public:


// members


// methods ///////////////////////////

/**
	 Dummy constructor.	
	 It does nothing 
*/
  RandomSubspace();

/**
	 Constructor.
	 @param baselearner base learner to be used by the bagging algorithm
	 @param num_class number of classes;
*/
RandomSubspace(BaseLearner* baselearner, unsigned num_class = DEFAULT_NUM_CLASS);

/**
	 Copy Constructor.
*/
  RandomSubspace(const RandomSubspace& Rs);


/**
	 Destructor.
*/
 virtual ~RandomSubspace();


/**
	 Test the trained base learners stored in files.\\
	 Testing of the base learners on the files whose name begins with testfile, using models whose names are stored in the list
	 modellist. The format of the testfiles is free, but it must be understandable to the base learner.
	 The base learners are tested with respect to different testfiles, as each base learner is trained usin their own random subspace.
	 The name of the different testfiles must be testfilestartiteration, testfilestartiteration+1, ... testfilemaxiterations, and the
	 files must be generated previously using the appropriate methods of the class TrainingSet.
	 Predicted classes are stored in files basresn, where n range from startiteration to maxiterations.
	 @param testfile  name of the testing set. 
	 @param startiteration number of the first base learner to be used. 
	 @param maxiterations number of the last base learner to be used. 
	 @param dir_res	directory where the computed classes are stored
	 @see TrainingSet
*/
virtual void test_base_learners(char* testfile, unsigned startiteration, 
                       unsigned maxiterations, char* dir_res="");

};





#endif
