// "snr.cc"


#include<ctype.h>
#include<stdlib.h>
#include<time.h>
#include<math.h>
#include<iostream.h>
#include<fstream.h>
#include"snr.h"
#include"utils.h"


SNRTest::SNRTest ( unsigned num_attr, unsigned num_patt,  char *file)
:  tset(num_attr, num_patt, file), n_attr(num_attr), n_patt(num_patt), class_rand(num_patt),
   SNR(num_attr),  SNR_rand(num_attr),  s_counter(num_attr), ranked_indices(num_attr), ranked_values(num_attr)                                
{
  //vect &classes = tset.Read_target(); // *********
	//data = tset.Read_data();
	srand(-time(NULL));
  class_rand=tset.Read_target();
}

/**
	 Computes the SNR of a single feature
	 @param index index of the feature
	 @param cl class for which the SNR has to be computed
	 @param label vector of the classes
	 */
double SNRTest::single_feature_SNR_compute(int index, int cl, vect& label) {
	 double m1=0;
	 double m2 = 0;
	 double s1 = 0;
	 double s2 = 0;
	 double snr_ratio;
	 int n1 = 0;
	 int n2 = 0;
	 const double ZERO_THRESH = 0.000001;

	 matrix& data = tset.Read_data();
	 // compute means
	 for (int i=0; i<n_patt; i++) {
	   if (label[i] == cl) {
		   m1 += data(i,index);
			 n1++;
		 }
		 else {
		   m2 += data(i,index);
			 n2++;
		}
	}
	m1/=n1;
	m2/=n2;
	// computes stdev
	double x;
	for (int i=0; i<n_patt; i++) {
	  if (label[i] == cl) {
		  x = data(i,index) - m1;
			s1 += (x*x);
		}
		else {
		  x = data(i,index) - m2;
			s2 += (x*x);
		}
	}
	s1 = sqrt(s1/(n1-1)); 					 
	s2 = sqrt(s2/(n2-1)); 
	if (s1<=ZERO_THRESH) s1=1;
	if (s2<=ZERO_THRESH) s2=1;		 
	// computes SNR ratio 	
	snr_ratio = (m1 - m2)/(s1 + s2); 
	return snr_ratio; 							 
}	
			
	 

void SNRTest::SNR_compute(int cl){
  for (int i=0; i<n_attr; i++) 
	  SNR[i] = single_feature_SNR_compute(i, cl, tset.Read_target());
}

void SNRTest::SNR_print(ostream& o){
  o.setf(ios::left, ios::adjustfield);
  for (int i=0; i<n_attr; i++) {
	  o.width(8);
		o << i << SNR[i] << endl; 
	}
}

void SNRTest::SNR_rand_compute(int cl){
  permute_class_labels();
  for (int i=0; i<n_attr; i++) 
	  SNR_rand[i] = single_feature_SNR_compute(i, cl, class_rand);
}

void SNRTest::SNR_rand_print(ostream& o) {
  o.setf(ios::left, ios::adjustfield);
  for (int i=0; i<n_attr; i++) {
	  o.width(8);
		o << i << SNR[i] << "     " << SNR_rand[i] << endl; 
	}
}
	 
void SNRTest::permute_class_labels(void) {
  int pos;
	int x;
  for (int i=0; i<n_patt; i++) {
	  pos = (int) ((float)n_patt*rand()/(RAND_MAX+1.0));
		// swap i and pos positions
		x = (int)class_rand[pos];
		class_rand[pos] = class_rand[i];
		class_rand[i] = x;		
	}
}	 
	 
 /**
	 Performs random permutation test for SNR
	 @param n number of permutations
	 */
void SNRTest::permutation_test(int n, int cl) {
   for (int j=0; j<n_attr; j++)
	   s_counter[j] = 0;
	 for (int i=0; i<n; i++) {
	    SNR_rand_compute(cl);
	    for (int j=0; j<n_attr; j++) {
			  if (SNR_rand[j] >= SNR[j])
				  s_counter[j]+=1;
				//else if ((SNR[j] < 0) && (SNR_rand[j] <= SNR[j]))
				//  s_counter[j]+=1;
			}
			cout << "." << flush;
	 }
	 for (int j=0; j<n_attr; j++) {
	    s_counter[j] /= n;		
			// cout <<  j  << "   " <<  s_counter[j] << endl; // *******
	}
}


 /**
	 Selects a set of features
	 @param significance significance of the selected features
	 */
void SNRTest::select_features(float significance) {
   selected_features.clear();
   for (int i=0; i<n_attr; i++) {
	   if (s_counter[i] <= significance)
		   selected_features.push_back(i);
	 }
}
	 

void SNRTest::save_feature_indices(char* indexfile){
  ofstream index(indexfile);
	if (!index) 
		error_exit("Cannot open ", indexfile);
	
	// writing indexfile
	int size = 	selected_features.size();
	index << size << endl;
	index.setf(ios::left, ios::adjustfield);
	for (int i=0; i <size; i++) {
	  index.width(8);
	  index << selected_features[i] <<  SNR[selected_features[i]] << endl;
	}
	if (index.bad()) 
		error_exit("Error writing ", indexfile); 
}

void SNRTest::load_feature_indices(char* indexfile){
  float buf;
	int size;
	int val;
  ifstream index(indexfile);
	if (!index) 
		error_exit("Cannot open ", indexfile);
	
	// reading indexfile
	selected_features.clear();
	index >> size;
	for (int i=0; i <size; i++) {
	   index >> val;
		 selected_features.push_back(val);
		 index >> buf;
	}
	if (index.bad()) 
		error_exit("Error reading ", indexfile); 
}


void SNRTest::save_selected_features(char* patternfile){
  vect& classes = tset.Read_target();
	matrix& data = tset.Read_data();
	ofstream patt(patternfile);
	if (!patt) 
		error_exit("Cannot open ", patternfile);
		
	// writing patternfile
	int size = 	selected_features.size();
	int pos = 0; 
	int selected;
	for (int i=0; i<n_patt; i++) {
	  for (int j=0; j<size; j++)
		  patt << data(i,selected_features[j]) << ',';	
		patt << classes[i] << endl;
	}
	if (patt.bad()) 
		error_exit("Error writing ", patternfile);	 	
}


void SNRTest::save_features(char* indexfile, char* patternfile){
  save_feature_indices(indexfile);
	save_selected_features(patternfile);	
}




void SNRTest::rank_features(void) {
  for (int i=0; i<n_attr; i++) {
    ranked_indices[i] = i;
		ranked_values[i] = SNR[i];
	}
	
	// bubblesort
	float x;
	int x_int;
	for (int n=0; n<(n_attr-1); n++) {
	  for (int j=n_attr-1, k=n_attr-2; j>n; j--,k--) {
		  if (ranked_values[j] > ranked_values[k]) {
			  x = ranked_values[k];
				x_int = (int)ranked_indices[k];
				ranked_values[k] = ranked_values[j];
				ranked_indices[k] = ranked_indices[j];
				ranked_indices[j] = x_int;
				ranked_values[j] = x;
			}
    }
  }
}

void SNRTest::print_ranked_feature_indices(int n, ostream& o) {
  o << n << endl;
	o.setf(ios::left, ios::adjustfield);
  for (int i=0; i<n; i++) {
	  o.width(8);
		o << ranked_indices[i] << ranked_values[i] << endl; 
	}
}


void SNRTest::save_ranked_feature_indices(int n, char* ranked_indices_file) {
  ofstream r_index(ranked_indices_file);
	if (!r_index) 
		error_exit("Cannot open ", ranked_indices_file);
		
	r_index << n << endl;
	r_index.setf(ios::left, ios::adjustfield);
	for (int i=0; i <n; i++) {
	  r_index.width(8);
	  r_index << ranked_indices[i] << ranked_values[i] << endl;
	}
	if (r_index.bad()) 
		error_exit("Error writing ", ranked_indices_file); 
}


void SNRTest::save_ranked_features(int n, char* ranked_feature_file){
  vect& classes = tset.Read_target();
	matrix& data = tset.Read_data();
	ofstream patt(ranked_feature_file);
	if (!patt) 
		error_exit("Cannot open ", ranked_feature_file);
		
	// writing patternfile
	for (int i=0; i<n_patt; i++) {
	  for (int j=0; j<n; j++)
		  patt << data(i,(int)ranked_indices[j]) << ',';	
		patt << classes[i] << endl;
	}
	if (patt.bad()) 
		error_exit("Error writing ", ranked_feature_file);	 	
}
