/*******************************************************************************
vect.cc
*******************************************************************************/


#include <math.h>
#include<stdlib.h>
#include "vect.h"
#include "utils.h"

void vect::control_dimensions(const vect& vec) const
{ if (d!=vec.d)
   error_exit("vector arguments have different dimensions.");
}


vect::vect() 
{ d = 0; 
  v = 0;
}


vect::vect(int n, double val) 
{ 
 if (n<0) error_exit("vector: negative dimension."); 
 d = n; 
 v = 0;
 if (d > 0)
 { v = new double[d];
   for(int i=0; i<d; i++) 
     v[i] = val;
 }
}


vect::vect(double* array, int n) 
{ 
 if (n<0) error_exit("vector: negative dimension."); 
 d = n; 
 v = 0;
 if (d > 0)
 { v = new double[d];
   for(int i=0; i<d; i++) 
     v[i] = array[i];
 }
}

vect::~vect() 
{ if (v) delete[] v; v = 0;}


vect::vect(const vect& p) 
{ d = p.d; 
  v = 0;
  if (d > 0)
  { v = new double[d];
    for(int i=0; i<d; i++) 
		   v[i] = p.v[i];
  }
}


double  vect::operator[](int i) const
{ if (i<0 || i>=d)  error_exit("vect: index out of range ");
  return v[i]; 
}

double& vect::operator[](int i)
{ if (i<0 || i>=d)  error_exit("vect: index out of range ");
  return v[i]; 
}


vect& vect::operator+=(const vect& vec)
{ control_dimensions(vec);
  int n = d;
  while (n--) v[n] += vec.v[n];
  return *this;
}

vect& vect::operator-=(const vect& vec)
{ control_dimensions(vec);
  int n = d;
  while (n--) v[n] -= vec.v[n];
  return *this;
}

vect vect::operator+(const vect& vec) const
{ control_dimensions(vec);
  int n = d;
  vect res(n);
  while (n--) res.v[n] = v[n]+vec.v[n];
  return res;
}

vect vect::operator-(const vect& vec) const
{ control_dimensions(vec);
  int n = d;
  vect res(n);
  while (n--) res.v[n] = v[n]-vec.v[n];
  return res;
}

vect vect::operator-() const  
{ int n = d;
  vect res(n);
  while (n--) res.v[n] = -v[n];
  return res;
}


vect vect::operator*(double x) const
{ int n = d;
  vect res(n);
  while (n--) res.v[n] = v[n] * x;
  return res;
}

vect vect::operator/(double x) const
{ int n = d;
  vect res(n);
  while (n--) res.v[n] = v[n] / x;
  return res;
}

double vect::operator*(const vect& vec) const
{ control_dimensions(vec);
  double res=0;
  int n = d;
  while (n--) res = res+v[n]*vec.v[n];
  return res;
}

vect& vect::operator=(const vect& vec)
{ 
  // if the dimension is different deletes v and reallocates
	if (d != vec.d) {
	  if (v) {
      delete[] v; 
			v = 0;
		}
    d = vec.d;
    if (d > 0) 
      v = new double[d];
    else
      v = 0;
  }
	// assignment elementwise
  for(int i=0; i<d; i++) v[i] = vec.v[i];

  return *this;
}

bool vect::operator==(const vect& vec)  const
{ if (vec.d != d) return false;
  int i = 0;
  while ((i<d) && (v[i]==vec.v[i])) i++;
  return (i==d) ? true : false;
 }
 
double vect::max (void) {
  int i;
  unsigned size = this->dim();
	double m = v[0];
	for (i=1; i<size; i++) 
	  if (v[i] > m)
		  m = v[i];
  return m;
}

double vect::min (void) {
  int i;
  unsigned size = this->dim();
	double m = v[0];
	for (i=1; i<size; i++) 
	  if (v[i] < m)
		  m = v[i];
  return m;
}

double vect::avg (void) {
  int i;
  unsigned size = this->dim();
	double m = 0.0;
	for (i=0; i<size; i++) 
		  m += v[i];
  return (m/size);
}


void vect::read(istream& is) 
{ for(int i=0;i<d;i++) is  >> v[i]; }

void vect::print(ostream& os, int i) 
{ 
  for(int j=0;j<d;j++) {
	   os.setf(ios::fixed, ios::floatfield);
		 if (i == 0) {
       os.precision(4);
	     os.width(12);
		 }
		 else {
		   os.precision(0);
	     os.width(8);
		 }
	   os << v[j];
	}
	os << endl;
	os.precision(4);
}



ostream& operator<<(ostream& os, const vect& v)
{ 
  for (int i=0;i<v.d;i++) {
	   os.setf(ios::fixed, ios::floatfield);
     os.precision(4);
	   os.width(12);
	   os << v[i];
	}
	os << endl;
  return os;
}


istream& operator>>(istream& is, vect& v) 
{ /* to be implemnted 
  int d;
  is >> d;
  vect w(d);
  for (int i=0;i<d;i++) is >> w[i];
  v = w;
	*/
  return is; 
} 


double vect::sqr_length() const { return *this * *this; }

double vect::length() const { return sqrt(sqr_length()); }



