/**
@name NEURObjects.  
{\Large Giorgio Valentini, Francesco Masulli}\\ 

DISI - Dipartimento di Informatica e \\ 
Scienze dell'Informazione -  Universita' di Genova\\
 via Dodecaneso 35, 16146 Genova, Italy\\
E-mail:{valenti, masulli}@disi.unige.it


\begin{center}
{\bf Introduction.}
\end{center}
NEURObjects is a relatively small C++ set of library classes for neural networks development, 
exploiting the  potentialities of object oriented design and programming [Str94]. \\
The starting point of the project
is to provide  a set of basic neural networks classes to support
design, implementation and comparative testing of classification algorithms. 
Our aim is to develop a general set of classes
to  support training, research and application in the neural network field.

\begin{center}
{\bf Why C++ classes for neural networks development ?}
\end{center}
There are many reasons to choose C++: 
  C++ is a good general - purpose language which has many useful features of object - oriented 
  programming 
  [Str94] 
  [Str97]
  C++ has also a large flexibility, allowing to avoid the object - oriented paradigm when needed, as  we can use it as a "improved C language";
  the language's objected - oriented features allow us to decompose the library into a set of modular
  objects which can be tested independently; 
   it is easy to expand and to reuse C++ code; 
  C++ is very efficient: none of his own features (overriding, overloading, function inlining, etc)
  lowers execution performances; 
  C++ is gaining popular and widely accepted among researchers' community.

On the other side, there are a number of reasons to use C++ specifically for neural networks development.
In fact, object oriented design provides classes corresponding to application domain  concepts: in our
case, this leads to the possibility to represent neural  networks concepts directly by classes.
Moreover, C++ linguistic constructions enable to represent in a concise way features and  
functionalities of neural networks; data abstraction enables to define in a  
comprehensive way sets of neural structures and operations; ensembles of neural nets  
can be easily accomplished by net-objects generated by C++ constructors; polymorphism  
of virtual functions leads up to generalization and flexibility  regarding  
methods of net-classes. By inheritance we can easily expand existing classes to experiment with new
learning algorithms without waste of time owing to programming by scratch.


\begin{center}
{\bf Related work.}
\end{center} 
Only few libraries in C++ for neural networks development are free accessible to researchers.
Often they  use C++ as a "better C", according to a well-known Stroustrup's
statement [Str97], employing just in part the potentialities of object oriented  
programming. Of this kind are the  libraries  enclosed into Timothy Masters's book [Mas].  

Others need remarkable hw and sw resources, as well as considerable learning  
period. For instance, MLC++, developed at Stanford University [Koh94], although it presents 
a wide variety of machine learning algorithms, doesn' t provide remarkable neural networks algorithms;
moreover, his huge high quality code requires a considerable educational effort.

Another kind of C++ libraries has been developed with emphasis on specific neural networks models. 
NNO [Kun96] has focused its efforts on self organizing incremental networks [Fri93], 
providing only a simple implementation of classes dealing with multi layer perceptrons: for instance, 
learning of gradients by backpropagation is  combined only with simple gradient descent to update network's
weights, and no other more powerful update algorithm is provided.


\begin{center}
{\bf NEURObjects goals}
\end{center}  
The main goal of NEURObjects is to assist in experimenting, training, testing and   
comparative studying of neural networks learning algorithms.  \\
More in detail we aim to provide capability to do the following tasks easily and quickly
\begin{itemize}
  \item Use of standard neural networks algorithms to solve specific classification
        and regression problems;
  \item Implement new neural learning algorithms or variants of existing neural algorithms;
  \item Implement ensembles of neural nets to recombine them to improve classification capabilities;
  \item Compare learning algorithms among them on a specified classification problem;
  \item Compare learning algorithms on different datasets;
  \item Generate synthetic training and test sets;
  \item Generate performance statistics such us accuracy, confidence level, rejection curves,
        learning rates and confusion matrices.         
\end{itemize}
NEURObjects can be used not only by researchers or by skilled software developers but also 
by simple end - users with little or no experience in object oriented programming. \\
Training time to learn using NEURObjects is fast and so technological transfer to industry is foreseen not expensive
and time consuming.
Moreover the library can also assists in teaching neural networks  
courses, providing facilities to look at neural parameters during training and to analyze training and
testing results.


\begin{center}
{\bf Design of NEURObjects library}
\end{center}   
Essentially the library develops classes representing concepts in neural  
networks domain and use educational and public domain software for general operations not specific to
neural networks. Schematically, using a functional approach,  we can divide the library in several groups
of classes:
\begin{itemize}
  \item {\bf General support classes}. These are classes not specific to neural networks. For instance,
  LEDA's [Nae98] library classes are used to implement basic classes used by NEURObjects (lists,  
  queues, vectors, matrices, balanced trees, dictionaries). For other basic data structures we use for now GNU libg++,
  but we plan to use the C++ Standard Template Library (STL), because is likely to be a widely accepted
  standard and has the static safety of templates.
  \item {\bf I/O and pre-processing classes}. Classes for reading and writing data files,
  converting attribute types between formats ({\em e.g.} automatic conversion of classes from  
  string-labeled into numeric-labeled format), normalizing attribute values, etc.
  \item {\bf Automatic data generation classes}. Classes generating  synthetic data files for training
  and testing by controlled random or deterministic techniques.
  \item {\bf Neural networks building classes}. These are classes to build arbitrary multi-layers
  perceptrons and radial basis function networks and to train and test them.
  \item {\bf Learning algorithms classes}. They implement several standard algorithms, from simple gradient
  descent to variations of conjugate gradients [Bar92] or quasi - Newton methods 
  (such us BFGS algorithm) [Bat] and many others methods.
  \item {\bf Ensemble of neural networks classes}. Classes that use ensemble of neural networks to improve
  classification tasks using error correcting output codes [Die95] and pertinent dichotomies
  [May97].
  \item {\bf Performance statistics classes}. Classes handling performance statistics such us accuracy, 
  confidence levels, rejection curves, learning rates and confusion matrices
  \item {\bf Comparing algorithms classes}. Classes handling statistical test for comparing algorithms on a
  a particular learning task: they detect whether exists a real (statistically significant) difference
  among different learning algorithms [Die98].
\end{itemize}



Each class is fully documented and code also is well documented.
For now we have used only GNU g++ compiler on PC-Linux and Sun-Solaris workstation platforms, but 
NEURObjects is easily portable on other platforms, included Window PC's.


\begin{center}
{\bf NEURObjects for End-Users}
\end{center}  
By NEURObjects, developing programs for neural networks training and testing requires  
just few lines of C++ code, each of that corresponding to a concept or a  
operation of the application domain:  multi-layer perceptron building,  
training by a learning algorithm and testing of the resulting neural network, all need  
only one or few more lines of code. \\
Programming with NEURObjects consists in selecting the proper components useful for the desired task and,
if it is needed, in overriding aspects of some components to fully fit them to a specified task. The work
can be done producing at each step a fully working program writing at each stage only few lines of code.
To show that, in this paper we will present some simple example of NEURObjects programming. 
Then we will analyse results and performances of  different kinds  of learning algorithms implemented
by NEURObjects.
      
On the other side, as source code is free accessible, it' s  easy to develop derived class suitable  
for specific researching works. Really NEURObjects is both for simple users  and for skilled developers.


\begin{center}
{\bf On-going developments}
\end{center} 
Actually  NEURObjects consists of about 5000 lines of code (ignoring public domain code included).
For now multi-layers  perceptrons have a relevant prominence in the library, but next other typologies  
of neural network classes will be developed. \\
Actually communication toward other applications is simply  implemented by file 
(mostly for data visualization by MATLAB or SCILAB), but we plan to develop  on-line 
communication protocols. \\
The next developing step will be to add specialized classes to handle decomposition of polichotomies 
into dichotomies [Die95,May97] cite, to provide direct support for actual experimenting with this promising researching field. 


\begin{center}
{\bf References}
\end{center}  
[Str94]
Stroustrup B.,
{\em The design and evolution of C++},   Addison Wesley, 1994.

[Str97]
Stroustrup B.,
{\em The C++ programming language},  Addison Wesley, 1997.


[Mas]
Masters T.,
{\em Practical Neural Networks Recipes in C++},  
Academic Press, Boston.


[Koh94]
Kohavi R., John G., Long R., Manley D., Pfleger K.
"MLC++: a Machine Learning library in C++",
{\em Tools with Artificial Intelligence},
IEEE Computer Society Press, pp.740-743, 1994. 
URL{http://www.sgi.com/Technology/mlc.}

[Kun96]
Kunze M., Steffens J.
"The neural network objects",
\in: {\em Proceedings of the fifth AIHENP Workshop},
Lausanne, World Scientific, 1996. \\
URL{{http://www.neuroinformatik.ruhr-uni-bochum.de/~marcel}

[Fri93]
Fritzke B.,
"Growing Cell Structures - A self organizing network for unsupervised and
           supervised learning",
{\em International Computer Science Institute}, TR-93-026, Berkeley, 1993.

[Nae98]
Naeher S.
{\em LEDA: A Library of Efficient Data Types and Algorithms},
3.7 vers., Max-Planck-Institut fr Informatik, 66123 Saarbrcken, Germany. 
URL{www.mpi-sb.mpg.de/LEDA/leda.html.}

[Bar92]
Barnard E.
"Optimization for training neural nets",
{\em IEEE Transaction on Neural Networks}, {\bf 3}, (2) pp. 232-240, 1992.

[Bat]
Battiti R., Masulli F.
"BFGS Optimization for Faster and Automated Supervised Learning",


[May97]
Mayoraz E., Moreira M.,
"On the Decomposition of Polychotomies into Dichotomies",
in: {\em The XIV International Conference on Machine Learning},
pp. 219-226, Nashville, TN, 1997.
 

[Die95]
Diettrich T., Bakiri G.,
"Solving multiclass learning problems via error-correcting output codes",
{\em Journal of Artificial Intelligence Research}, 2, pp. 263-286, 1995. 



[Die98]
Diettrich T.G.
"Approximate statistical tests for comparing supervised classification learning algorithms",
{\em Neural Computation}, {\bf 10}, (7), 1998.

@memo Extended abstract soco-99.
*/ 
